//-----------------------------------------------------------------------
// <copyright file="ActivityManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage activities.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer; 

    /// <summary>
    /// Class used to manage activities. Implements Singleton pattern. Use 'Instance' property to access object.
    /// </summary>
    public sealed class ActivityManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static ActivityManager objects.</summary>
        private static volatile ActivityManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the ActivityManager class from being created.
        /// </summary>
        private ActivityManager()
        {
        }

        #endregion Constructor

        #region Public Static Members

        /// <summary>
        /// Gets instance to an ActivityManager object. 
        /// </summary>
        public static ActivityManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new ActivityManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Members

        #region Public Methods

        /// <summary>
        /// Adds new activity.
        /// </summary>
        /// <param name="jobCodeId">Job Code that the activity belongs to.</param>
        /// <param name="code">Code of activity.</param>
        /// <param name="name">Name of activity.</param>
        /// <returns>Id of new activity.</returns>
        public long AddActivity(long jobCodeId, string code, string name)
        {
            try
            {
                if (string.IsNullOrEmpty(code))
                {
                    throw new ArgumentNullException("code");
                }

                if (string.IsNullOrEmpty(name))
                {
                    throw new ArgumentNullException("name");
                }

                return ActivityDataObject.Instance.AddActivity(jobCodeId, code, name);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing activity.
        /// </summary>
        /// <param name="activity">Activity to update.</param>
        public void UpdateActivity(Activity activity)
        {
            try
            {
                if (activity == null)
                {
                    throw new ArgumentNullException("activity");
                }

                ActivityDataObject.Instance.UpdateActivity(activity);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to delete.</param>
        public void DeleteActivity(long activityId)
        {
            try
            {
                ActivityDataObject.Instance.DeleteActivity(activityId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of activities that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public ActivityCollection GetActivities(ActivityFilter filter)
        {
            try
            {
                return ActivityDataObject.Instance.GetActivities(filter);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets an activity that matches the id.
        /// </summary>
        /// <param name="activityId">Id to use when retrieving data.</param>
        /// <returns>Matching activity.</returns>
        public Activity GetActivity(long activityId)
        {
            try
            {
                return ActivityDataObject.Instance.GetActivity(activityId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disables existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to disable.</param>
        public void DisableActivity(long activityId)
        {
            try
            {
                ActivityDataObject.Instance.DisableActivity(activityId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to enable.</param>
        public void EnableActivity(long activityId)
        {
            try
            {
                ActivityDataObject.Instance.EnableActivity(activityId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        #endregion Public Methods
    }
}
