//-----------------------------------------------------------------------
// <copyright file="GenderManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage genders.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer; 

    /// <summary>
    /// Class used to manage genders.
    /// </summary>
    public sealed class GenderManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static GenderManager objects.</summary>
        private static volatile GenderManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the GenderManager class from being created.
        /// </summary>
        private GenderManager()
        {
        }

        #endregion Constructor

        #region Public Static Members

        /// <summary>
        /// Gets instance to an GenderManager object. 
        /// </summary>
        public static GenderManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new GenderManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Members

        #region Public Methods

        /// <summary>
        /// Adds new Gender.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <param name="name">Name of Gender.</param>
        /// <returns>Id of new Gender.</returns>
        public short AddGender(byte genderId, string name)
        {
            try
            {
                return GenderDataObject.Instance.AddGender(genderId, name);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Gender.
        /// </summary>
        /// <param name="gender">Gender to update.</param>
        public void UpdateGender(Gender gender)
        {
            try
            {
                GenderDataObject.Instance.UpdateGender(gender);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Gender.
        /// </summary>
        /// <param name="genderId">Id of Gender to delete.</param>
        public void DeleteGender(byte genderId)
        {
            try
            {
                GenderDataObject.Instance.DeleteGender(genderId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a Gender that matches the id.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <returns>gender matching the id.</returns>
        public Gender GetGender(byte genderId)
        {
            try
            {
                return GenderDataObject.Instance.GetGender(genderId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Genders that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public GenderCollection GetGenders(GenderFilter filter)
        {
            try
            {
                return GenderDataObject.Instance.GetGenders(filter);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        #endregion Public Methods
    }
}
