//-----------------------------------------------------------------------
// <copyright file="JobCodeManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage Job Codes.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer; 

    /// <summary>
    /// Class used to manage Job Codes.
    /// </summary>
    public sealed class JobCodeManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static JobCodeManager objects.</summary>
        private static volatile JobCodeManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the JobCodeManager class from being created.
        /// </summary>
        private JobCodeManager()
        {
        }

        #endregion Constructor

        #region Public Static Members

        /// <summary>
        /// Gets instance to an JobCodeManager object. 
        /// </summary>
        public static JobCodeManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new JobCodeManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Members

        #region Public Methods

        /// <summary>
        /// Adds new Job Code.
        /// </summary>
        /// <param name="code">The job code.</param>
        /// <param name="name">Name of Job Code.</param>
        /// <param name="organisationId">Organisation that the job code belongs to.</param>
        /// <returns>Id of new JobCode.</returns>
        public long AddJobCode(string code, string name, int organisationId)
        {
            try
            {
                return JobCodeDataObject.Instance.AddJobCode(code, name, organisationId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Job Code.
        /// </summary>
        /// <param name="jobCode">Job Code to update.</param>
        public void UpdateJobCode(JobCode jobCode)
        {
            try
            {
                if (jobCode == null)
                {
                    throw new ArgumentNullException("jobCode");
                }

                JobCodeDataObject.Instance.UpdateJobCode(jobCode);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to delete.</param>
        public void DeleteJobCode(long jobCodeId)
        {
            try
            {
                JobCodeDataObject.Instance.DeleteJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to get.</param>
        /// <returns>matching Job Code.</returns>
        public JobCode GetJobCode(long jobCodeId)
        {
            try
            {
                return JobCodeDataObject.Instance.GetJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Gets a collection of Job Codes that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public JobCodeCollection GetJobCodes(JobCodeFilter filter)
        {
            try
            {
                return JobCodeDataObject.Instance.GetJobCodes(filter);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to disable.</param>
        public void DisableJobCode(long jobCodeId)
        {
            try
            {
                JobCodeDataObject.Instance.DisableJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to enable.</param>
        public void EnableJobCode(long jobCodeId)
        {
            try
            {
                JobCodeDataObject.Instance.EnableJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        #endregion Public Methods
    }
}
