//-----------------------------------------------------------------------
// <copyright file="OrganisationManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage Organisations.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer; 

    /// <summary>
    /// Class used to manage Organisations.
    /// </summary>
    public sealed class OrganisationManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static OrganisationManager objects.</summary>
        private static volatile OrganisationManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the OrganisationManager class from being created.
        /// </summary>
        private OrganisationManager()
        {
        }

        #endregion Constructor

        #region Public Static Members

        /// <summary>
        /// Gets instance to an OrganisationManager object. 
        /// </summary>
        public static OrganisationManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new OrganisationManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Members

        #region Public Methods

        /// <summary>
        /// Adds new Organisation.
        /// </summary>
        /// <param name="name">Name of Organisation.</param>
        /// <returns>Id of new Organisation.</returns>
        public int AddOrganisation(string name)
        {
            try
            {
                return OrganisationDataObject.Instance.AddOrganisation(name);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Organisation.
        /// </summary>
        /// <param name="organisation">Organisation to update.</param>
        public void UpdateOrganisation(Organisation organisation)
        {
            try
            {
                OrganisationDataObject.Instance.UpdateOrganisation(organisation);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to delete.</param>
        public void DeleteOrganisation(int organisationId)
        {
            try
            {
                OrganisationDataObject.Instance.DeleteOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Organisations that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public OrganisationCollection GetOrganisations(OrganisationFilter filter)
        {
            try
            {
                return OrganisationDataObject.Instance.GetOrganisations(filter);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets an Organisations that matches the id.
        /// </summary>
        /// <param name="organisationId">Id to use when retrieving data.</param>
        /// <returns>Elements that match the id.</returns>
        public Organisation GetOrganisation(int organisationId)
        {
            try
            {
                return OrganisationDataObject.Instance.GetOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disbales existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to disbale.</param>
        public void DisableOrganisation(int organisationId)
        {
            try
            {
                OrganisationDataObject.Instance.DisableOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to enable.</param>
        public void EnableOrganisation(int organisationId)
        {
            try
            {
                OrganisationDataObject.Instance.EnableOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        #endregion Public Methods
    }
}