//-----------------------------------------------------------------------
// <copyright file="RoleManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage Roles.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer; 

    /// <summary>
    /// Class used to manage Roles.
    /// </summary>
    public sealed class RoleManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static RoleManager objects.</summary>
        private static volatile RoleManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the RoleManager class from being created.
        /// </summary>
        private RoleManager()
        {
        }

        #endregion Constructor

        #region Public Static Members

        /// <summary>
        /// Gets instance to an RoleManager object. 
        /// </summary>
        public static RoleManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new RoleManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Members

        #region Public Methods

        /// <summary>
        /// Adds new Role.
        /// </summary>
        /// <param name="roleId">Id of role</param>
        /// <param name="name">Name of Role.</param>
        /// <returns>Id of new Role.</returns>
        public short AddRole(short roleId, string name)
        {
            try
            {
                return RoleDataObject.Instance.AddRole(roleId, name);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Role.
        /// </summary>
        /// <param name="role">Role to update.</param>
        public void UpdateRole(Role role)
        {
            try
            {
                RoleDataObject.Instance.UpdateRole(role);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Role.
        /// </summary>
        /// <param name="roleId">Id of Role to delete.</param>
        public void DeleteRole(short roleId)
        {
            try
            {
                RoleDataObject.Instance.DeleteRole(roleId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a Role that matches the id.
        /// </summary>
        /// <param name="roleId">Id of role.</param>
        /// <returns>Matching role.</returns>
        public Role GetRole(short roleId)
        {
            try
            {
                return RoleDataObject.Instance.GetRole(roleId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Roles that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public RoleCollection GetRoles(RoleFilter filter)
        {
            try
            {
                return RoleDataObject.Instance.GetRoles(filter);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        #endregion Public Methods
    }
}
