//-----------------------------------------------------------------------
// <copyright file="Timesheet.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage Timesheet.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using System.Collections.Generic;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer;

    /// <summary>
    /// Class thats holds timesheet.
    /// </summary>
    [Serializable]
    public class Timesheet
    {
        /// <summary>Holds timesheet.</summary>
        private BusinessObjects.Timesheet timesheet;

        /// <summary>Holds Timesheet Attendance Entries.</summary>
        private TimesheetAttendanceEntryCollection timesheetAttendanceEntries;

        /// <summary>Holds Timesheet Activity Entries.</summary>
        private TimesheetActivityEntryCollection timesheetActivityEntries;

        /// <summary>
        /// Initializes a new instance of the Timesheet class.
        /// </summary>
        /// <param name="timesheet">Timesheet data.</param>
        /// <param name="timesheetAttendanceEntries">Attendence data</param>
        /// <param name="timesheetActivityEntries">Activity Data.</param>
        public Timesheet(
            BusinessObjects.Timesheet timesheet,
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            this.timesheet = timesheet;
            this.timesheetAttendanceEntries = timesheetAttendanceEntries;
            this.timesheetActivityEntries = timesheetActivityEntries;
        }

        /// <summary>
        /// Initializes a new instance of the Timesheet class.
        /// </summary>
        /// <param name="userId">Id of user.</param>
        /// <param name="weekEndDate">Week End Date of the timesheet.</param>
        public Timesheet(long userId, DateTime weekEndDate)
        {
            this.timesheet = new BusinessObjects.Timesheet(userId, weekEndDate);
            this.timesheetAttendanceEntries = new TimesheetAttendanceEntryCollection();
            this.timesheetActivityEntries = new TimesheetActivityEntryCollection();
        }

        /// <summary>Gets timesheet.</summary>
        public BusinessObjects.Timesheet TimeSheet
        {
            get { return this.timesheet; }
        }

        /// <summary>Gets Timesheet Attendance Entries.</summary>
        public TimesheetAttendanceEntryCollection TimesheetAttendanceEntries
        {
            get { return this.timesheetAttendanceEntries; }
        }

        /// <summary>Gets Timesheet Activity Entries.</summary>
        public TimesheetActivityEntryCollection TimesheetActivityEntries
        {
            get { return this.timesheetActivityEntries; }
        }

        /// <summary>
        /// Gets the max number of in outs that occurred in a day.
        /// </summary>
        public int MaxInOutsInDay
        {
            get
            {
                Dictionary<DateTime, int> rowCounts = new Dictionary<DateTime, int>();
                int maxRows = 0;
                if (this.timesheetAttendanceEntries != null)
                {
                    foreach (TimesheetAttendanceEntry entry in this.timesheetAttendanceEntries)
                    {
                        if (rowCounts.ContainsKey(entry.StartDateTime.Date))
                        {
                            rowCounts[entry.StartDateTime.Date]++;
                            if (rowCounts[entry.StartDateTime.Date] > maxRows)
                            {
                                maxRows = rowCounts[entry.StartDateTime.Date];
                            }
                        }
                        else
                        {
                            rowCounts.Add(entry.StartDateTime.Date, 1);
                            if (rowCounts[entry.StartDateTime.Date] > maxRows)
                            {
                                maxRows = rowCounts[entry.StartDateTime.Date];
                            }
                        }
                    }
                }

                return maxRows;
            }
        }

        /// <summary>
        /// Gets the max number of activities that occurred in a day.
        /// </summary>
        public int MaxActivitiesInDay
        {
            get
            {
                Dictionary<DateTime, int> rowCounts = new Dictionary<DateTime, int>();
                int maxRows = 0;
                if (this.timesheetActivityEntries != null)
                {
                    foreach (TimesheetActivityEntry entry in this.timesheetActivityEntries)
                    {
                        if (rowCounts.ContainsKey(entry.Date.Date))
                        {
                            rowCounts[entry.Date.Date]++;
                            if (rowCounts[entry.Date.Date] > maxRows)
                            {
                                maxRows = rowCounts[entry.Date.Date];
                            }
                        }
                        else
                        {
                            rowCounts.Add(entry.Date.Date, 1);
                            if (rowCounts[entry.Date.Date] > maxRows)
                            {
                                maxRows = rowCounts[entry.Date.Date];
                            }
                        }
                    }
                }

                return maxRows;
            }
        }

        /// <summary>
        /// Gets the start date of the timesheet.
        /// </summary>
        public DateTime StartDate
        {
            get
            {
                return this.timesheet.WeekEndDate.Date.AddDays(-6);
            }
        }

        /// <summary>
        /// Gets the the number of in/outs that occurred in the day.
        /// </summary>
        /// <param name="date">Date to search for.</param>
        /// <returns>The number of in/outs that have occurred in the current date.</returns>
        public int GetInOutsInDay(DateTime date)
        {
            int count = 0;
            if (this.timesheetAttendanceEntries != null)
            {
                foreach (TimesheetAttendanceEntry timesheetAttendanceEntry in this.timesheetAttendanceEntries)
                {
                    if (timesheetAttendanceEntry.StartDateTime.Date == date.Date)
                    {
                        count++;
                    }
                }
            }

            return count;
        }

        /// <summary>
        /// Gets the matching Timesheet Attendance Entry that match the date and sequence Number.
        /// </summary>
        /// <param name="date">Date to search for.</param>
        /// <param name="sequenceIndex">Index of the entry in the timesheet sequence.</param>
        /// <returns>Matching entry.</returns>
        public TimesheetAttendanceEntry GetTimesheetAttendanceEntry(DateTime date, int sequenceIndex)
        {
            List<DateTime> times = new List<DateTime>();
            foreach (TimesheetAttendanceEntry timesheetAttendanceEntry in this.timesheetAttendanceEntries)
            {
                if (timesheetAttendanceEntry.StartDateTime.Date == date.Date)
                {
                    times.Add(timesheetAttendanceEntry.StartDateTime);
                }
            }

            if (times.Count - 1 >= sequenceIndex)
            {
                times.Sort();

                foreach (TimesheetAttendanceEntry timesheetAttendanceEntry in this.timesheetAttendanceEntries)
                {
                    if (timesheetAttendanceEntry.StartDateTime == times[sequenceIndex])
                    {
                        return timesheetAttendanceEntry;
                    }
                }
            }

            return null;
        }
    }
}
