//-----------------------------------------------------------------------
// <copyright file="TimesheetManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage timesheets.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using System.Collections.Generic; 
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer;

    /// <summary>
    /// Class used to manage timesheet.
    /// </summary>
    public sealed class TimesheetManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static TimesheetManager objects.</summary>
        private static volatile TimesheetManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the TimesheetManager class from being created.
        /// </summary>
        private TimesheetManager()
        {
        }

        #endregion Constructor

        #region Public Static Properties

        /// <summary>
        /// Gets instance to an TimesheetManager object. 
        /// </summary>
        public static TimesheetManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new TimesheetManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Properties

        #region Public Methods

        /// <summary>
        /// Adds new Timesheet.
        /// </summary>
        /// <param name="userId">id of user the timesheet is for.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="weekEndDate">The end date of the week. Must be a friday.</param>
        /// <returns>Id of new Timesheet.</returns>
        /// <exception cref="System.Exception">If week end date is not a friday.</exception>
        public long AddTimesheet(long userId, short timesheetStatusId, DateTime weekEndDate)
        {
            try
            {
                if (weekEndDate.Date.DayOfWeek != DayOfWeek.Friday)
                {
                    // Week end must be a friday
                    throw new Exception();
                }

                User user = UserManager.Instance.GetUser(userId);
                if (!user.OrganisationId.HasValue)
                {
                    // User must be in an organisation
                    throw new Exception();
                }

                return TimesheetDataObject.Instance.AddTimesheet(userId, timesheetStatusId, weekEndDate.Date);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Adds new Timesheet Attendance.
        /// </summary>
        /// <param name="timesheetId">id of the timesheet.</param>
        /// <param name="startDateTime">Start Date of the entry.</param>
        /// <param name="endDateTime">End Date of the entry.</param>
        /// <returns>Id of new entry.</returns>
        public long AddTimesheetAttendanceEntry(long timesheetId, DateTime startDateTime, DateTime endDateTime)
        {
            try
            {
                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);
                if (this.IsValidTimesheetAttendanceEntry(timesheet.WeekEndDate, startDateTime, endDateTime) == false)
                {
                    throw new Exception();
                }

                // Make sure that we dont overlap any other entry on this day.
                TimesheetAttendanceEntryFilter timesheetAttendanceEntryFilter = new TimesheetAttendanceEntryFilter();
                timesheetAttendanceEntryFilter.TimesheetId = timesheetId;
                timesheetAttendanceEntryFilter.StartDateTime = startDateTime;
                timesheetAttendanceEntryFilter.EndDateTime = endDateTime;
                if (TimesheetAttendanceEntryDataObject.Instance.GetTimesheetAttendanceEntries(timesheetAttendanceEntryFilter).Count > 0)
                {
                    // Cannot overlap an attendance entrance that already exists
                    throw new Exception();
                }

                return TimesheetAttendanceEntryDataObject.Instance.AddTimesheetAttendanceEntry(timesheetId, startDateTime, endDateTime);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Adds new Timesheet Activity entry.
        /// </summary>
        /// <param name="timesheetId">id of timesheet the entry is for.</param>
        /// <param name="activityId">Id of activity is for.</param>
        /// <param name="hours">Number od hours for entry.</param>
        /// <param name="description">Description of entry.</param>
        /// <param name="date">Date for entry.</param>
        /// <returns>Id of new entry.</returns>
        public long AddTimesheetActivityEntry(long timesheetId, long activityId, decimal hours, string description, DateTime date)
        {
            try
            {
                if (hours > 24 || hours <= 0)
                {
                    // hours must be greater then 0 and less than or equal to 24 
                    throw new Exception();
                }

                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);

                if (date.Date > timesheet.WeekEndDate.Date
                    || date.Date < timesheet.WeekEndDate.Date.AddDays(-7))
                {
                    // Date must be within the timesheets range of 7 days.
                    throw new Exception();
                }

                // Make sure that we dont overlap any other entry on this day.
                TimesheetActivityEntryFilter timesheetActivityEntryFilter = new TimesheetActivityEntryFilter();
                timesheetActivityEntryFilter.TimesheetId = timesheetId;
                timesheetActivityEntryFilter.ActivityId = activityId;
                timesheetActivityEntryFilter.StartDateTime = date;
                timesheetActivityEntryFilter.EndDateTime = date;
                if (TimesheetActivityEntryDataObject.Instance.GetTimesheetActivityEntries(timesheetActivityEntryFilter).Count > 0)
                {
                    // Cannot overlap an activity entrance that already exists
                    throw new Exception();
                }

                return TimesheetActivityEntryDataObject.Instance.AddTimesheetActivityEntry(timesheetId, activityId, hours, description, date);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Validates the entire timesheet.
        /// </summary>
        /// <param name="timesheetId">Timesheet to validate.</param>
        /// <returns>True on valid.</returns>
        public bool IsValidTimesheet(long timesheetId)
        {
            try
            {
                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);
                
                TimesheetAttendanceEntryFilter timesheetAttendanceEntryFilter = new TimesheetAttendanceEntryFilter();
                timesheetAttendanceEntryFilter.TimesheetId = timesheetId;
                TimesheetAttendanceEntryCollection timesheetAttendanceEntries = 
                    TimesheetAttendanceEntryDataObject.Instance.GetTimesheetAttendanceEntries(timesheetAttendanceEntryFilter);

                TimesheetActivityEntryFilter timesheetActivityEntryFilter = new TimesheetActivityEntryFilter();
                timesheetActivityEntryFilter.TimesheetId = timesheetId;
                TimesheetActivityEntryCollection timesheetActivityEntries =
                    TimesheetActivityEntryDataObject.Instance.GetTimesheetActivityEntries(timesheetActivityEntryFilter);

                return this.IsValidTimesheet(timesheet, timesheetAttendanceEntries, timesheetActivityEntries);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Saves the entire timesheet.
        /// </summary>
        /// <param name="timesheetStatusId">Status to save the timesheet as. Cannot be approved or declined.</param>
        /// <param name="timesheet">Timesheet to save.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to save.</param>
        /// <param name="timesheetActivityEntries">Related activities to save.</param>
        /// <returns>Id of timesheet.</returns>
        public long SaveTimesheet(
            short timesheetStatusId,
            BusinessObjects.Timesheet timesheet,
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            try
            {
                long timesheetId = -1;
                bool valid = true;

                if (timesheet == null)
                {
                    throw new ArgumentNullException("timesheet");
                }

                if (timesheetAttendanceEntries == null
                    || timesheetAttendanceEntries.Count == 0)
                {
                    throw new ArgumentNullException("timesheetAttendanceEntries");
                }

                if (timesheetActivityEntries == null
                    || timesheetActivityEntries.Count == 0)
                {
                    throw new ArgumentNullException("timesheetActivityEntries");
                }

                // Check if timesheet is valid only if the user has NOT selected save
                if (Convert.ToInt16(BusinessLogicLayer.TimesheetStatusManager.TimesheetStatuses.Save) != timesheetStatusId)
                {
                    valid = this.IsValidTimesheet(timesheet, timesheetAttendanceEntries, timesheetActivityEntries);
                }
                else
                {
                    valid = this.IsValidActivities(timesheetActivityEntries);
                }

                if (valid)
                {
                    if (timesheet.TimesheetId == 0)
                    {
                        // New timesheet
                        timesheetId = DataAccessLayer.TimesheetDataObject.Instance.AddTimesheet(
                            timesheet.UserId,
                            timesheetStatusId,
                            timesheet,
                            timesheetAttendanceEntries,
                            timesheetActivityEntries);
                    }
                    else
                    {
                        // Old timesheet
                        timesheetId = DataAccessLayer.TimesheetDataObject.Instance.UpdateTimesheet(
                            timesheet.UserId,
                            timesheetStatusId,
                            timesheet,
                            timesheetAttendanceEntries,
                            timesheetActivityEntries);
                    }
                }

                return timesheetId;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Validates the entire timesheet.
        /// </summary>
        /// <param name="timesheet">Timesheet to validate.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to validate.</param>
        /// <param name="timesheetActivityEntries">Related activities to validate.</param>
        /// <returns>True on valid.</returns>
        public bool IsValidTimesheet(
            BusinessObjects.Timesheet timesheet, 
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            try
            {
                if (timesheet == null)
                {
                    throw new ArgumentNullException("timesheet");
                }

                if (timesheetAttendanceEntries == null)
                {
                    throw new ArgumentNullException("timesheetAttendanceEntries");
                }

                if (timesheetActivityEntries == null)
                {
                    throw new ArgumentNullException("timesheetActivityEntries");
                }

                // Validate timesheet
                if (this.IsValidTimesheet(timesheet.WeekEndDate) == false)
                {
                    return false;
                }

                // Validate Attendence
                foreach (TimesheetAttendanceEntry timesheetAttendanceEntry in timesheetAttendanceEntries)
                {
                    if (this.IsValidTimesheetAttendanceEntry(timesheet.WeekEndDate, timesheetAttendanceEntry.StartDateTime, timesheetAttendanceEntry.EndDateTime) == false)
                    {
                        return false;
                    }

                    if (this.IsValidTimesheetAttendanceEntry(timesheetAttendanceEntry, timesheetAttendanceEntries) == false)
                    {
                        return false;
                    }
                }

                // Validate Activites
                foreach (TimesheetActivityEntry timesheetActivityEntry in timesheetActivityEntries)
                {
                    if (this.IsValidTimesheetActivityEntry(timesheet.WeekEndDate, timesheetActivityEntry.Date, timesheetActivityEntry.Hours) == false)
                    {
                        return false;
                    }

                    if (this.IsValidTimesheetActivityEntry(timesheetActivityEntry, timesheetActivityEntries) == false)
                    {
                        return false;
                    }
                }

                if (!this.IsValidActivities(timesheetActivityEntries))
                {
                    return false;
                }

                if (this.IsValidAttendanceVsActivity(timesheetAttendanceEntries, timesheetActivityEntries) == false)
                {
                    return false;
                }

                return true;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Validates the entire timesheet.
        /// </summary>
        /// <param name="timesheetActivityEntries">Activities to validate.</param>
        /// <returns>True on valid.</returns>
        public bool IsValidActivities(
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            foreach (TimesheetActivityEntry timesheetActivityEntry in timesheetActivityEntries)
            {
                foreach (TimesheetActivityEntry timesheetActivityEntry1 in timesheetActivityEntries)
                {
                    if (timesheetActivityEntry != timesheetActivityEntry1)
                    {
                        if (timesheetActivityEntry.ActivityId == timesheetActivityEntry1.ActivityId
                            && timesheetActivityEntry.Date.Date == timesheetActivityEntry1.Date.Date)
                        {
                            // Duplicate found
                            return false;
                        }
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Gets all timesheet information from the database.
        /// </summary>
        /// <param name="timesheetId">Id of timesheet.</param>
        /// <returns>All Timesheet information.</returns>
        public FullTimesheet GetFullTimesheet(long timesheetId)
        {
            try
            {
                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);

                TimesheetAttendanceEntryFilter timesheetAttendanceEntryFilter = new TimesheetAttendanceEntryFilter();
                timesheetAttendanceEntryFilter.TimesheetId = timesheetId;
                TimesheetAttendanceEntryCollection timesheetAttendanceEntries =
                    TimesheetAttendanceEntryDataObject.Instance.GetTimesheetAttendanceEntries(timesheetAttendanceEntryFilter);

                TimesheetActivityEntryFilter timesheetActivityEntryFilter = new TimesheetActivityEntryFilter();
                timesheetActivityEntryFilter.TimesheetId = timesheetId;
                TimesheetActivityEntryCollection timesheetActivityEntries =
                    TimesheetActivityEntryDataObject.Instance.GetTimesheetActivityEntries(timesheetActivityEntryFilter);

                return new FullTimesheet(timesheet, timesheetAttendanceEntries, timesheetActivityEntries);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets timesheet information from the database.
        /// </summary>
        /// <param name="timesheetId">Id of timesheet.</param>
        /// <returns>Timesheet information.</returns>
        public Timesheet GetTimesheet(long timesheetId)
        {
            try
            {
                return TimesheetDataObject.Instance.GetTimesheet(timesheetId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets all timesheet information from the database.
        /// </summary>
        /// <param name="userId">Id of timesheet.</param>
        /// <param name="weekEndDate">The date of the weekend.</param>
        /// <returns>All Timesheet information or null on error.</returns>
        public FullTimesheet GetFullTimesheet(long userId, DateTime weekEndDate)
        {
            try
            {
                if (weekEndDate.Date.DayOfWeek != DayOfWeek.Friday)
                {
                    // Week end must be a friday
                    throw new Exception();
                }

                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(userId, weekEndDate);

                TimesheetAttendanceEntryFilter timesheetAttendanceEntryFilter = new TimesheetAttendanceEntryFilter();
                timesheetAttendanceEntryFilter.TimesheetId = timesheet.TimesheetId;
                TimesheetAttendanceEntryCollection timesheetAttendanceEntries =
                    TimesheetAttendanceEntryDataObject.Instance.GetTimesheetAttendanceEntries(timesheetAttendanceEntryFilter);

                TimesheetActivityEntryFilter timesheetActivityEntryFilter = new TimesheetActivityEntryFilter();
                timesheetActivityEntryFilter.TimesheetId = timesheet.TimesheetId;
                TimesheetActivityEntryCollection timesheetActivityEntries =
                    TimesheetActivityEntryDataObject.Instance.GetTimesheetActivityEntries(timesheetActivityEntryFilter);

                return new FullTimesheet(timesheet, timesheetAttendanceEntries, timesheetActivityEntries);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Gets timesheet information from the database.
        /// </summary>
        /// <param name="userId">Id of timesheet.</param>
        /// <param name="weekEndDate">The date of the weekend.</param>
        /// <returns>Timesheet information or null on error.</returns>
        public Timesheet GetTimesheet(long userId, DateTime weekEndDate)
        {
            try
            {
                if (weekEndDate.Date.DayOfWeek != DayOfWeek.Friday)
                {
                    // Week end must be a friday
                    throw new Exception();
                }

                return TimesheetDataObject.Instance.GetTimesheet(userId, weekEndDate);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Gets a collection of Timesheets that match the filter.
        /// </summary>
        /// <param name="accessorUserId">Id of user attempting to access the timesheets.</param>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetCollection GetTimesheets(long accessorUserId, TimesheetFilter filter)
        {
            try
            {
                User accesorUser = UserManager.Instance.GetUser(accessorUserId);
                UserCollection userHierachy = UserDataObject.Instance.GetUserHierachyAboveUser(accessorUserId, false);
                TimesheetCollection timesheets = TimesheetDataObject.Instance.GetTimesheets(filter);

                for (int i = 0; i < timesheets.Count; i++)
                {
                    if (!this.CanAccessTimesheet(accesorUser, userHierachy, timesheets[i]))
                    {
                        timesheets.RemoveAt(i);
                        i--;
                    }
                }

                return timesheets;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Checks to see if the current user can access the timesheet
        /// </summary>
        /// <param name="accessorUserId">Id of the user trying to access the timeshhet.</param>
        /// <param name="timesheetId">Id of timesheet being accessed.</param>
        /// <returns>Whether or not the user can access the timesheet.</returns>
        public bool CanAccessTimesheet(long accessorUserId, long timesheetId)
        {
            try
            {
                bool result = false;
                Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);
                User accessedUser = UserManager.Instance.GetUser(timesheet.UserId);

                if (timesheet != null)
                {
                    if (accessedUser.UserId == accessorUserId)
                    {
                        // Ids match so it is users timesheet i.e owner
                        result = true;
                    }
                    else if (accessedUser.ApproverUserId.HasValue && accessedUser.ApproverUserId.Value == accessorUserId)
                    {
                        // Is approver
                        result = true;
                    }
                    else
                    {
                        // Check if the accessor user is someone above the approver in the hierachy.
                        UserCollection users = UserDataObject.Instance.GetUserHierachyAboveUser(accessorUserId, false);
                        foreach (User user in users)
                        {
                            if (user.UserId == accessorUserId)
                            {
                                result = true;
                            }
                        }
                    }
                }

                return result;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Checks to see if the user passed in is the approver of the timesheet.
        /// </summary>
        /// <param name="userId">Id of user to check.</param>
        /// <param name="timesheetId">Id of Timesheet to check against.</param>
        /// <returns>Whether or not the user is the approver of the timesheet</returns>
        public bool IsTimesheetApprover(long userId, long timesheetId)
        {
            try
            {
                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);
                User user = UserManager.Instance.GetUser(timesheet.UserId);
                if (user.ApproverUserId.HasValue && userId == user.ApproverUserId.Value)
                {
                    return true;
                }

                return false;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Checks to see if the user passed in is the owner of the timesheet.
        /// </summary>
        /// <param name="userId">Id of user to check.</param>
        /// <param name="timesheetId">Id of Timesheet to check against.</param>
        /// <returns>Whether or not the user is the owner of the timesheet</returns>
        public bool IsTimesheetOwner(long userId, long timesheetId)
        {
            try
            {
                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);
                if (timesheet.UserId == userId)
                {
                    return true;
                }

                return false;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Checks to see if the user passed only has readonly access to the timesheet.
        /// </summary>
        /// <param name="accessorUserId">Id of user to check.</param>
        /// <param name="timesheetId">Id of Timesheet to check against.</param>
        /// <returns>Whether or not the user has readonly access to the timesheet</returns>
        public bool IsTimesheetReadOnly(long accessorUserId, long timesheetId)
        {
            try
            {
                if (this.IsTimesheetOwner(accessorUserId, timesheetId))
                {
                    BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);
                    if (timesheet.TimesheetStatusId == (int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Approved
                        || timesheet.TimesheetStatusId == (int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Submitted)
                    {
                        // Readonly because its been approved or submitted.
                        return true;
                    }

                    return false;
                }

                if (this.CanAccessTimesheet(accessorUserId, timesheetId))
                {
                    // Readonly
                    return true;
                }

                // User has no access to it at all!!!
                throw new Exception();
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a list of valid statues that the user can change the timesheet to 
        /// depending on who the user is and the current status of the timesheet.
        /// </summary>
        /// <param name="userId">The id of user.</param>
        /// <param name="timesheetId">The id of the timesheet.</param>
        /// <returns>Collection of Timesheet status that the user can assin to the timesheet.</returns>
        public TimesheetStatusCollection GetValidTimesheetStatues(long userId, long? timesheetId)
        {
            TimesheetStatusCollection allStatuses = TimesheetStatusDataObject.Instance.GetTimesheetStatuses(null);
            TimesheetStatusCollection validStatuses = new TimesheetStatusCollection();

            if (!timesheetId.HasValue)
            {
                validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Save));
                validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Submit));
            }
            else
            {
                BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId.Value);

                if (this.IsTimesheetOwner(userId, timesheetId.Value))
                {
                    // User is owner of the timesheet
                    if (timesheet.TimesheetStatusId == (int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Saved)
                    {
                        // If curr status is saved, user can safe or submit
                        validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Save));
                        validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Submit));
                    }

                    if (timesheet.TimesheetStatusId == (int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Declined)
                    {
                        // If curr status is declined, user can only submit it.
                        validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Submit));
                    }
                }
                else if (this.IsTimesheetApprover(userId, timesheetId.Value))
                {
                    // User is approver
                    if (timesheet.TimesheetStatusId == (int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Submitted)
                    {
                        // If it is submitted then user can approve or decline
                        validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Approve));
                        validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Decline));
                    }

                    if (timesheet.TimesheetStatusId == (int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Approved)
                    {
                        validStatuses.Add(allStatuses.GetById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Decline));
                    }
                }
            }

            User user = UserDataObject.Instance.GetUser(userId);
            if (!user.ApproverUserId.HasValue)
            {
                // No one to submit to
                validStatuses.RemoveById((int)BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions.Submit);
            }

            return validStatuses;
        }

        /// <summary>
        /// Returns whether or not the owner can delete this timesheet.
        /// A timesheet can only be deleted by the owner if it has only been saved or declined.
        /// </summary>
        /// <param name="timesheetId">Id of timesheet to check.</param>
        /// <returns>True if timesheet can be deleted.</returns>
        public bool CanOwnerDeleteTimesheet(long timesheetId)
        {
            BusinessObjects.Timesheet timesheet = TimesheetDataObject.Instance.GetTimesheet(timesheetId);
            if (timesheet.TimesheetStatusId == (short)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Saved
                || timesheet.TimesheetStatusId == (short)BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Declined)
            {
                return true;
            }

            return false;
        }

        /// <summary>
        /// Deletes the timesheet and any related entries!!
        /// </summary>
        /// <param name="timesheetId">Id of timesheet to delete.</param>
        public void DeleteAllTimesheet(long timesheetId)
        {
            try
            {
                TimesheetActivityEntryDataObject.Instance.DeleteTimesheetActivityEntries(timesheetId);
                TimesheetAttendanceEntryDataObject.Instance.DeleteTimesheetAttendanceEntries(timesheetId);
                TimesheetDataObject.Instance.DeleteTimesheet(timesheetId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Sets the status of the timesheet.
        /// </summary>
        /// <param name="timesheetId">The id of the timesheet to update.</param>
        /// <param name="changerId">id of the user changing the status.</param>
        /// <param name="timesheetStatusId">The ewn status of the timesheet.</param>
        public void SetTimesheetStatus(long timesheetId, long changerId, short timesheetStatusId)
        {
            DataAccessLayer.TimesheetDataObject.Instance.ChangeTimesheetStatus(timesheetId, changerId, timesheetStatusId, null);
        }

        #endregion Public Methods

        #region Private Methods

        /// <summary>
        /// Validates the timesheets week end date.
        /// </summary>
        /// <param name="weekEndDate">Week enda date to validate.</param>
        /// <returns>True if valid</returns>
        private bool IsValidTimesheet(DateTime weekEndDate)
        {
            if (weekEndDate.Date.DayOfWeek != DayOfWeek.Friday)
            {
                // Week end must be a friday
                return false;
            }

            return true;
        }

        /// <summary>
        /// Validates the attendance entries times against the timesheet's week end time.
        /// </summary>
        /// <param name="timesheetWeekEndDate">Week end date of the timesheet.</param>
        /// <param name="startDateTime">Start time of the entry.</param>
        /// <param name="endDateTime">End time of the entry.</param>
        /// <returns>True if valid</returns>
        private bool IsValidTimesheetAttendanceEntry(DateTime timesheetWeekEndDate, DateTime startDateTime, DateTime endDateTime)
        {
            if (startDateTime >= endDateTime)
            {
                // Start must be less than end
                return false;
            }

            if (startDateTime.Date != endDateTime.Date)
            {
                // Start and end date must be on the same day
                return false;
            }

            if (startDateTime.Date > timesheetWeekEndDate.Date
                || startDateTime.Date < timesheetWeekEndDate.Date.AddDays(-7))
            {
                // Date must be within the timesheets range of 7 days.
                return false;
            }

            return true;
        }

        /// <summary>
        /// Validates timesheetAttendanceEntry against a collection.
        /// </summary>
        /// <param name="timesheetAttendanceEntry">Entry to validate.</param>
        /// <param name="timesheetAttendanceEntries">Entries to validate against.</param>
        /// <returns>True if valid.</returns>
        private bool IsValidTimesheetAttendanceEntry(TimesheetAttendanceEntry timesheetAttendanceEntry, TimesheetAttendanceEntryCollection timesheetAttendanceEntries)
        {
            foreach (TimesheetAttendanceEntry entry in timesheetAttendanceEntries)
            {
                if (timesheetAttendanceEntry.TimesheetAttendanceEntryId != entry.TimesheetAttendanceEntryId
                    && timesheetAttendanceEntry.TimesheetId == entry.TimesheetId)
                {
                    if (timesheetAttendanceEntry.StartDateTime >= entry.EndDateTime
                        || timesheetAttendanceEntry.EndDateTime <= entry.StartDateTime)
                    {
                        // Doesn't overlap
                    }
                    else
                    {
                        // Overlap found
                        return false;
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Validates TimesheetActivityEntry against timesheets weekendate.
        /// </summary>
        /// <param name="timesheetWeekEndDate">Week end date of the timesheet.</param>
        /// <param name="date">Date of activity.</param>
        /// <param name="hours">Hours done for activity.</param>
        /// <returns>True if valid.</returns>
        private bool IsValidTimesheetActivityEntry(DateTime timesheetWeekEndDate, DateTime date, decimal hours)
        {
            if (hours > 24 || hours <= 0)
            {
                // hours must be greater then 0 and less than or equal to 24 
                return false;
            }

            if (date.Date > timesheetWeekEndDate.Date
                || date.Date < timesheetWeekEndDate.Date.AddDays(-7))
            {
                // Date must be within the timesheets range of 7 days.
                return false;
            }

            return true;
        }

        /// <summary>
        /// Validates timesheet activity against a collection of activities
        /// </summary>
        /// <param name="timesheetActivityEntry">Activity to validate.</param>
        /// <param name="timesheetActivityEntries">Collection to validate against.</param>
        /// <returns>True if valid.</returns>
        private bool IsValidTimesheetActivityEntry(TimesheetActivityEntry timesheetActivityEntry, TimesheetActivityEntryCollection timesheetActivityEntries) 
        {
            decimal totalHours = timesheetActivityEntry.Hours;
            foreach (TimesheetActivityEntry entry in timesheetActivityEntries)
            {
                if (timesheetActivityEntry.TimesheetActivityEntryId != entry.TimesheetActivityEntryId
                    && timesheetActivityEntry.TimesheetId == entry.TimesheetId)
                {
                    if (timesheetActivityEntry.ActivityId == entry.ActivityId
                        && timesheetActivityEntry.Date.Date == entry.Date)
                    {
                        // Duplicate ativity found for same day.
                        return false;
                    }

                    if (timesheetActivityEntry.Date.Date == entry.Date)
                    {
                        totalHours += entry.Hours;
                    }
                }
            }

            if (totalHours > 24)
            {
                // More than 24 hours
                return false;
            }

            return true;
        }

        /// <summary>
        /// Validates a set of activity entries against a set of attendance entries.
        /// </summary>
        /// <param name="timesheetAttendanceEntries">Attendance entries to use.</param>
        /// <param name="timesheetActivityEntries">Activities to validate.</param>
        /// <returns>True if valid.</returns>
        private bool IsValidAttendanceVsActivity(TimesheetAttendanceEntryCollection timesheetAttendanceEntries, TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            Dictionary<DateTime, DateTotalHours> attendanceTotalHours = this.GetTimesheetAttendanceDateTotalHours(timesheetAttendanceEntries);
            Dictionary<DateTime, DateTotalHours> activityTotalHours = this.GetTimesheetActivityDateTotalHours(timesheetActivityEntries);

            if (attendanceTotalHours.Count != activityTotalHours.Count)
            {
                // Days must match
                return false;
            }

            foreach (DateTotalHours dateTotalHours in activityTotalHours.Values)
            {
                if (attendanceTotalHours.ContainsKey(dateTotalHours.Date.Date) == false)
                {
                    // attendence must exist for day in order to have activity
                    return false;
                }
                else if (attendanceTotalHours[dateTotalHours.Date.Date].TotalHours != dateTotalHours.TotalHours)
                {
                    // Hours must be equal
                    return false;
                }
            }

            return true;
        }

        /// <summary>
        /// Gets the total Attendance hours for each day.
        /// </summary>
        /// <param name="timesheetAttendanceEntries">Entries to calculate hours for.</param>
        /// <returns>Dictionary containg the Date and hours.</returns>
        private Dictionary<DateTime, DateTotalHours> GetTimesheetAttendanceDateTotalHours(TimesheetAttendanceEntryCollection timesheetAttendanceEntries)
        {
            Dictionary<DateTime, DateTotalHours> totalHours = new Dictionary<DateTime, DateTotalHours>();
            foreach (TimesheetAttendanceEntry timesheetAttendanceEntry in timesheetAttendanceEntries)
            {
                TimeSpan span = timesheetAttendanceEntry.EndDateTime - timesheetAttendanceEntry.StartDateTime;
                if (!totalHours.ContainsKey(timesheetAttendanceEntry.StartDateTime.Date))
                {
                    totalHours.Add(timesheetAttendanceEntry.StartDateTime.Date, new DateTotalHours(timesheetAttendanceEntry.StartDateTime.Date, (decimal)span.TotalHours));
                }
                else
                {
                    totalHours[timesheetAttendanceEntry.StartDateTime.Date].TotalHours += (decimal)span.TotalHours;
                }
            }

            return totalHours;
        }

        /// <summary>
        /// Gets the total Activity hours for each day.
        /// </summary>
        /// <param name="timesheetActivityEntries">Entries to calculate hours for.</param>
        /// <returns>Dictionary containg the Date and hours.</returns>
        private Dictionary<DateTime, DateTotalHours> GetTimesheetActivityDateTotalHours(TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            Dictionary<DateTime, DateTotalHours> totalHours = new Dictionary<DateTime, DateTotalHours>();
            foreach (TimesheetActivityEntry timesheetActivityEntry in timesheetActivityEntries)
            {
                if (!totalHours.ContainsKey(timesheetActivityEntry.Date.Date))
                {
                    totalHours.Add(timesheetActivityEntry.Date.Date, new DateTotalHours(timesheetActivityEntry.Date.Date, (decimal)timesheetActivityEntry.Hours));
                }
                else
                {
                    totalHours[timesheetActivityEntry.Date.Date].TotalHours += (decimal)timesheetActivityEntry.Hours;
                }
            }

            return totalHours;
        }

        /// <summary>
        /// Checks to see if the current user can access the timesheet
        /// </summary>
        /// <param name="accesorUser">user trying to access the timeshhet.</param>
        /// <param name="userHierachy">users who are above the user passed in in terms of approver hierachy.</param>
        /// <param name="timesheet">Timesheet being accessed.</param>
        /// <returns>Whether or not the user can access the timesheet.</returns>
        private bool CanAccessTimesheet(User accesorUser, UserCollection userHierachy, BusinessObjects.Timesheet timesheet)
        {
            try
            {
                if (accesorUser == null)
                {
                    throw new ArgumentNullException("accesorUser");
                }

                if (timesheet == null)
                {
                    throw new ArgumentNullException("timesheet");
                }

                User timesheetOwnerUser = UserManager.Instance.GetUser(timesheet.UserId);
                return this.CanAccessTimesheet(accesorUser, timesheetOwnerUser, userHierachy);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Checks to see if the current user can access the timesheet
        /// </summary>
        /// <param name="accesorUser">user trying to access the timeshhet.</param>
        /// <param name="timesheetOwnerUser">The owner of the Timesheet being accessed.</param>
        /// <param name="userHierachy">Users who are above the accessor user passed in in terms of approver hierachy.</param>
        /// <returns>Whether or not the user can access the timesheet.</returns>
        private bool CanAccessTimesheet(User accesorUser, User timesheetOwnerUser, UserCollection userHierachy)
        {
            try
            {
                if (accesorUser == null)
                {
                    throw new ArgumentNullException("accesorUser");
                }

                if (timesheetOwnerUser == null)
                {
                    throw new ArgumentNullException("timesheetOwnerUser");
                }

                bool result = false;

                if (timesheetOwnerUser.UserId == accesorUser.UserId)
                {
                    // Ids match so it is users timesheet
                    result = true;
                }
                else if (timesheetOwnerUser.ApproverUserId.HasValue 
                    && timesheetOwnerUser.ApproverUserId.Value == accesorUser.UserId)
                {
                    // Approver is trying to access the timesheet
                    result = true;
                }
                else
                {
                    // Check if an approver can access the timesheet
                    if (userHierachy != null)
                    {
                        foreach (User user in userHierachy)
                        {
                            if (user.UserId == accesorUser.UserId)
                            {
                                result = true;
                            }
                        }
                    }
                }

                return result;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        #endregion Private Methods

        #region Private Classes

        /// <summary>Holds Total hours for each date.</summary>
        private class DateTotalHours
        {
            /// <summary>
            /// Holds date of entry.
            /// </summary>
            private DateTime date;

            /// <summary>
            /// Holds total hours.
            /// </summary>
            private decimal totalHours;

            /// <summary>
            /// Initializes a new instance of the DateTotalHours class.
            /// </summary>
            /// <param name="date">Date of item.</param>
            /// <param name="totalHours">Numner of hoyrs.</param>
            public DateTotalHours(DateTime date, decimal totalHours)
            {
                this.Date = date;
                this.TotalHours = totalHours;
            }

            /// <summary>Gets or sets Date of entry.</summary>
            public DateTime Date
            {
                get { return this.date; }
                set { this.date = value; }
            }

            /// <summary>Gets or sets Total hours for date.</summary>
            public decimal TotalHours
            {
                get { return this.totalHours; }
                set { this.totalHours = value; }
            }
        }

        #endregion Private Classes
    }
}
