//-----------------------------------------------------------------------
// <copyright file="TimesheetStatusManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage timesheet statues.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer; 
    
    /// <summary>
    /// Class used to manage Timesheet Statuses.
    /// </summary>
    public sealed class TimesheetStatusManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static EntryStatusManager objects.</summary>
        private static volatile TimesheetStatusManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the TimesheetStatusManager class from being created.
        /// </summary>
        private TimesheetStatusManager()
        {
        }

        #endregion Constructor

        #region Public Enums

        /// <summary>
        /// Represents the constant Timesheet statuses available.
        /// </summary>
        public enum TimesheetStatuses : short
        {
            /// <summary>Timesheet is saved.</summary>
            Save = 1,

            /// <summary>Timesheet is waiting to be approved.</summary>
            AwaitingApproval = 2,

            /// <summary>Timesheet is declined.</summary>
            Declined = 3,

            /// <summary>Timesheet is Approved.</summary>
            Approved = 4
        }

        #endregion Public Enums

        #region Public Static Properties

        /// <summary>
        /// Gets instance to an EntryStatusManager object. 
        /// </summary>
        public static TimesheetStatusManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new TimesheetStatusManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Properties

        #region Public Methods

        /// <summary>
        /// Adds new Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of timesheet status.</param>
        /// <param name="name">Name of Timesheet Status.</param>
        /// <returns>Id of new Timesheet Status.</returns>
        public short AddTimesheetStatus(short timesheetStatusId, string name)
        {
            try
            {
                if (string.IsNullOrEmpty(name))
                {
                    throw new ArgumentNullException("name");
                }

                return TimesheetStatusDataObject.Instance.AddTimesheetStatus(timesheetStatusId, name);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatus">Timesheet Status to update.</param>
        public void UpdateTimesheetStatus(TimesheetStatus timesheetStatus)
        {
            try
            {
                TimesheetStatusDataObject.Instance.UpdateTimesheetStatus(timesheetStatus);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of Timesheet Status to delete.</param>
        public void DeleteTimesheetStatus(short timesheetStatusId)
        {
            try
            {
                TimesheetStatusDataObject.Instance.DeleteTimesheetStatus(timesheetStatusId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Timesheet Statuses that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetStatusCollection GetTimesheetStatuses(TimesheetStatusFilter filter)
        {
            try
            {
                return TimesheetStatusDataObject.Instance.GetTimesheetStatuses(filter);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a Timesheet Status that matches the id.
        /// </summary>
        /// <param name="timesheetStatusId">Id to use when retrieving data.</param>
        /// <returns>Matching Timesheet Status.</returns>
        public TimesheetStatus GetTimesheetStatus(short timesheetStatusId)
        {
            try
            {
                return TimesheetStatusDataObject.Instance.GetTimesheetStatus(timesheetStatusId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        #endregion Public Methods
    }
}
