//-----------------------------------------------------------------------
// <copyright file="UserManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage Users.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer
{
    using System;
    using System.Text;
    using BusinessLogicLayer.Utilities;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using DataAccessLayer; 

    /// <summary>
    /// Class used to manage Users.
    /// </summary>
    public sealed class UserManager : Manager
    {
        #region Private Static Members

        /// <summary>Holds static UserManager objects.</summary>
        private static volatile UserManager instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Static Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the UserManager class from being created.
        /// </summary>
        private UserManager()
        {
        }

        #endregion Constructor

        #region Public Static Properties

        /// <summary>
        /// Gets instance to an UserManager object. 
        /// </summary>
        public static UserManager Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new UserManager();
                        }
                    }
                }

                return instance;
            }
        }

        #endregion Public Static Properties

        #region Public Static Methods

        /// <summary>
        /// Creates and returns a new password.
        /// </summary>
        /// <returns>New password.</returns>
        public static string GetNewPassword()
        {
            StringBuilder builder = new StringBuilder();
            Random random = new Random();
            char ch;
            for (int i = 0; i < CommonConstants.MinPasswordLength; i++)
            {
                ch = Convert.ToChar(Convert.ToInt32(Math.Floor((26 * random.NextDouble()) + 65)));
                builder.Append(ch);
            }

            return builder.ToString();
        }

        #endregion Public Static Methods

        #region Public Methods

        /// <summary>
        /// Adds new User.
        /// </summary>
        /// <param name="employeeId">Employee id of user.</param>
        /// <param name="genderId">Gender id of the user.</param>
        /// <param name="roleId">Role of user.</param>
        /// <param name="firstname">Firstname of user.</param>
        /// <param name="surname">Surname of user.</param>
        /// <param name="email">Email of user.</param>
        /// <param name="password">password of user.</param>
        /// <param name="organisationId">Id of organisation the user belongs to.</param>
        /// <param name="approverUserId">The user who will approve this users timesheet entries.</param>
        /// <returns>Id of user.</returns>
        public long AddUser(string employeeId, byte genderId, short roleId, string firstname, string surname, string email, string password, int? organisationId, long? approverUserId)
        {
            try
            {
                if (string.IsNullOrEmpty(employeeId))
                {
                    throw new ArgumentNullException("employeeId");
                }

                if (!Validators.IsValidName(firstname))
                {
                    throw new ArgumentException("firstname");
                }

                if (!Validators.IsValidName(surname))
                {
                    throw new ArgumentException("surname");
                }

                if (!Validators.IsValidEmail(email))
                {
                    throw new ArgumentException("email");
                }

                if (!Validators.IsValidPassword(password))
                {
                    throw new ArgumentException("password");
                }

                string encryptedPassword = Cryptographer.Encrypt(password, this.PasswordEncryptionType);
                return UserDataObject.Instance.AddUser(employeeId, genderId, roleId, firstname, surname, email, encryptedPassword, organisationId, approverUserId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Adds new User.
        /// </summary>
        /// <param name="employeeId">Employee id of user.</param>
        /// <param name="genderId">Gender id of the user.</param>
        /// <param name="roleId">Role of user.</param>
        /// <param name="firstname">Firstname of user.</param>
        /// <param name="surname">Surname of user.</param>
        /// <param name="email">Email of user.</param>
        /// <param name="organisationId">Id of organisation the user belongs to.</param>
        /// <param name="approverUserId">The user who will approve this users timesheet entries.</param>
        /// <returns>Id of user.</returns>
        public long AddUser(string employeeId, byte genderId, short roleId, string firstname, string surname, string email, int? organisationId, long? approverUserId)
        {
            string password = GetNewPassword();
            return this.AddUser(
                employeeId,
                genderId,
                roleId,
                firstname,
                surname,
                email,
                password,
                organisationId,
                approverUserId);
        }

        /// <summary>
        /// Updates existing user.
        /// </summary>
        /// <param name="user">User to update.</param>
        public void UpdateUser(User user)
        {
            try
            {
                if (!Validators.IsValidName(user.Firstname))
                {
                    throw new ArgumentException("Firstname");
                }

                if (!Validators.IsValidName(user.Surname))
                {
                    throw new ArgumentException("Surname");
                }

                if (!Validators.IsValidEmail(user.Email))
                {
                    throw new ArgumentException("Email");
                }

                UserDataObject.Instance.UpdateUser(user);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing User.
        /// </summary>
        /// <param name="userId">Id of User to delete.</param>
        public void DeleteUser(long userId)
        {
            try
            {
                UserDataObject.Instance.DeleteUser(userId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Users that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public UserCollection GetUsers(UserFilter filter)
        {
            try
            {
                return UserDataObject.Instance.GetUsers(filter);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disables existing user.
        /// </summary>
        /// <param name="userId">Id of User to disable.</param>
        public void DisableUser(long userId)
        {
            try
            {
                UserDataObject.Instance.DisableUser(userId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing user.
        /// </summary>
        /// <param name="userId">Id of User to enable.</param>
        public void EnableUser(long userId)
        {
            try
            {
                UserDataObject.Instance.EnableUser(userId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Activates existing user.
        /// </summary>
        /// <param name="userId">Id of User to activate.</param>
        public void ActivateUser(long userId)
        {
            try
            {
                UserDataObject.Instance.ActivateUser(userId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deactivates existing user.
        /// </summary>
        /// <param name="userId">Id of User to Deactivate.</param>
        public void DeactivateUser(long userId)
        {
            try
            {
                UserDataObject.Instance.DeactivateUser(userId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates user's password.
        /// </summary>
        /// <param name="userId">Id of User to change password for.</param>
        /// <param name="password">The users new password.</param>
        public void SetUserPassword(long userId, string password)
        {
            try
            {
                UserDataObject.Instance.SetUserPassword(userId, Cryptographer.Encrypt(password, this.PasswordEncryptionType));
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Checks to see if the login information is correct.
        /// </summary>
        /// <param name="email">The user's Email.</param>
        /// <param name="password">The user's password.</param>
        /// <returns>True is values are authentic.</returns>
        public bool AuthenticateUser(string email, string password)
        {
            try
            {
                User user = UserDataObject.Instance.GetUser(email);

                if (user != null &&
                    !user.DisabledDateTime.HasValue && 
                    user.ActivationDateTime.HasValue && 
                    user.Password == Cryptographer.Encrypt(password, this.PasswordEncryptionType))
                {
                    // User is not disabled, is activated and the password is correct.
                    return true;
                }
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
            }

            return false;
        }

        /// <summary>
        /// Gets the user with matching email.
        /// </summary>
        /// <param name="email">The user's Email.</param>
        /// <returns>Matching user.</returns>
        public User GetUser(string email)
        {
            try
            {
                return UserDataObject.Instance.GetUser(email);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets the user with user id.
        /// </summary>
        /// <param name="userId">The user's id.</param>
        /// <returns>Matching user.</returns>
        public User GetUser(long userId)
        {
            try
            {
                return UserDataObject.Instance.GetUser(userId);
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets whether or not the passed in user is an approver.
        /// </summary>
        /// <param name="userId">Id of user to check.</param>
        /// <returns>True if user is approver.</returns>
        public bool IsUserApprover(long userId)
        {
            try
            {
                UserFilter filter = new UserFilter();
                filter.ApproverUserId = userId;
                return UserDataObject.Instance.GetUsers(filter).Count > 0;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets whether or not the passed in user has an approver.
        /// </summary>
        /// <param name="userId">Id of user to check.</param>
        /// <returns>True if user has approver.</returns>
        public bool HasApprover(long userId)
        {
            try
            {
                User user = UserDataObject.Instance.GetUser(userId);
                return user.ApproverUserId.HasValue;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Resets the users password.
        /// </summary>
        /// <param name="email">Email of the user's password to reset.</param>
        /// <returns>The new password.</returns>
        public string ResetPassword(string email)
        {
            try
            {
                string newPassword = GetNewPassword();
                User user = this.GetUser(email);
                this.SetUserPassword(user.UserId, newPassword);
                return newPassword;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                return null;
            }
        }

        /// <summary>
        /// Resets the users password.
        /// </summary>
        /// <param name="userId">Id of the user's password to reset.</param>
        /// <returns>The new password.</returns>
        public string ResetPassword(long userId)
        {
            try
            {
                string newPassword = GetNewPassword();
                this.SetUserPassword(userId, newPassword);
                return newPassword;
            }
            catch (Exception ex)
            {
                this.HandleException(ex);
                return null;
            }
        }

        #endregion Public Methods  
    }
}
