//-----------------------------------------------------------------------
// <copyright file="Cryptographer.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that is used to manage Cryptography.</summary>
//-----------------------------------------------------------------------
namespace BusinessLogicLayer.Utilities
{
    using System;
    using System.IO;
    using System.Security.Cryptography;

    /// <summary>
    /// Class used to encrypt and decrypt data. 
    /// Provides static Encrypt methods that expect encryption parameters to be passed.
    /// Provides Decrypt methods that expect decryption parameters to be passed.
    /// Provides Key/Vector Creation Helper methods to generate random key/vector combinations, which in turn can be used encrypt and decrypt
    /// data for certain protocols.
    /// </summary>
    public static class Cryptographer
    {
        #region Public Enums

        /// <summary>
        /// Holds the different encryption types.
        /// </summary>
        public enum EncryptionType
        {
            /// <summary>No Encryption Type.</summary>
            None,

            /// <summary>DES Encryption Type.</summary>
            DES,

            /// <summary>MD5 Encryption Type.</summary>
            MD5,

            /// <summary>RC2 Encryption Type.</summary>
            RC2,

            /// <summary>RSA Encryption Type.</summary>
            RSA,

            /// <summary>SHA1 Encryption Type.</summary>
            SHA1,

            /// <summary>TripleDES Encryption Type.</summary>
            TripleDES
        }

        #endregion Public Enums

        #region Public Methods

        /// <summary>
        /// Encrypts passed in data to encryption type provided. Will not work for DES, RC2 and TripleDES as they require key/vector combo
        /// </summary>
        /// <param name="data">Data to encrypt.</param>
        /// <param name="encryptionType">Encryption type to use.</param>
        /// <returns>Encrypted data.</returns>
        public static string Encrypt(string data, EncryptionType encryptionType)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            string result = null;
            System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
            byte[] tempByteResult = null;

            switch (encryptionType)
            {
                case EncryptionType.None:
                    result = data;
                    break;

                case EncryptionType.DES:
                    throw new InvalidOperationException("DES needs key and vector");

                case EncryptionType.MD5:
                    tempByteResult = EncryptMD5(System.Text.Encoding.UTF8.GetBytes(data));
                    result = Convert.ToBase64String(tempByteResult);
                    break;

                case EncryptionType.RC2:
                    throw new InvalidOperationException("RC2 needs key and vector");

                case EncryptionType.RSA:
                    throw new InvalidOperationException("RSA needs key");

                case EncryptionType.SHA1:
                    tempByteResult = EncryptSHA1(System.Text.Encoding.UTF8.GetBytes(data));
                    result = Convert.ToBase64String(tempByteResult);
                    break;

                case EncryptionType.TripleDES:
                    throw new InvalidOperationException("TripleDES needs key and vector");

                default:
                    break;
            }

            return result;
        }

        /// <summary>
        /// Encrypts passed in data to encryption type provided.
        /// </summary>
        /// <param name="data">Data to encrypt.</param>
        /// <param name="encryptionType">Encryption type to use.</param>
        /// <param name="key">Secret key to use in encryption. Required for DES, RC2 and TripleDES.</param>
        /// <param name="vector">vector to use in encryption. Required for DES, RC2 and TripleDES.</param>
        /// <returns>Encrypted data.</returns>
        public static string Encrypt(string data, EncryptionType encryptionType, byte[] key, byte[] vector)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            string result = null;
            System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
            byte[] tempByteResult = null;

            switch (encryptionType)
            {
                case EncryptionType.None:
                    result = data;
                    break;

                case EncryptionType.DES:
                    result = EncryptDES(data, key, vector);
                    break;

                case EncryptionType.MD5:
                    tempByteResult = EncryptMD5(System.Text.Encoding.UTF8.GetBytes(data));
                    result = Convert.ToBase64String(tempByteResult);
                    break;

                case EncryptionType.RC2:
                    result = EncryptRC2(data, key, vector);
                    break;

                case EncryptionType.RSA:
                    tempByteResult = EncryptRSA(System.Text.Encoding.UTF8.GetBytes(data), System.Text.Encoding.UTF8.GetString(key));
                    result = Convert.ToBase64String(tempByteResult);
                    break;

                case EncryptionType.SHA1:
                    tempByteResult = EncryptSHA1(System.Text.Encoding.UTF8.GetBytes(data));
                    result = Convert.ToBase64String(tempByteResult);
                    break;

                case EncryptionType.TripleDES:
                    result = EncryptTripleDES(data, key, vector);
                    break;

                default:
                    break;
            }

            return result;
        }

        /// <summary>
        /// Encrypts passed in data to encryption type provided.
        /// </summary>
        /// <param name="data">Data to encrypt.</param>
        /// <param name="encryptionType">Encryption type to use.</param>
        /// <param name="key">Secret key to use in encryption. Required for DES, RC2 and TripleDES.</param>
        /// <param name="vector">vector to use in encryption. Required for DES, RC2 and TripleDES.</param>
        /// <returns>Encrypted data.</returns>
        public static string Encrypt(string data, EncryptionType encryptionType, string key, string vector)
        {
            byte[] keyBytes = null;
            if (!string.IsNullOrEmpty(key))
            {
                keyBytes = System.Text.Encoding.UTF8.GetBytes(key);
            }

            byte[] vectorBytes = null;
            if (!string.IsNullOrEmpty(vector))
            {
                vectorBytes = System.Text.Encoding.UTF8.GetBytes(vector);
            }

            return Encrypt(data, encryptionType, keyBytes, vectorBytes);
        }

        /// <summary>
        /// Decrypts passed in data using encryption type provided. Will not work for MD5 or SHA1 as they cannot be decrypted.
        /// </summary>
        /// <param name="data">Data to Decrypt.</param>
        /// <param name="encryptionType">Encryption type to use.</param>
        /// <param name="key">Secret key to use in Decryption. Required for DES, RC2 and TripleDES.</param>
        /// <param name="vector">vector to use in Decryption. Required for DES, RC2 and TripleDES.</param>
        /// <returns>Decrypted data.</returns>
        public static string Decrypt(string data, EncryptionType encryptionType, byte[] key, byte[] vector)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            string result = null;
            System.Text.ASCIIEncoding encoding = new System.Text.ASCIIEncoding();
            byte[] tempByteResult = null;

            switch (encryptionType)
            {
                case EncryptionType.None:
                    result = data;
                    break;

                case EncryptionType.DES:
                    result = DecryptDES(data, key, vector);
                    break;

                case EncryptionType.MD5:
                    throw new InvalidOperationException("Cannot Decrypt MD5");

                case EncryptionType.RC2:
                    result = DecryptRC2(data, key, vector);
                    break;

                case EncryptionType.RSA:
                    tempByteResult = DecryptRSA(Convert.FromBase64String(data), System.Text.Encoding.UTF8.GetString(key));
                    result = System.Text.Encoding.UTF8.GetString(tempByteResult);
                    break;

                case EncryptionType.SHA1:
                    throw new InvalidOperationException("Cannot Decrypt SHA1");

                case EncryptionType.TripleDES:
                    result = DecryptTripleDES(data, key, vector);
                    break;

                default:
                    break;
            }

            return result;
        }

        /// <summary>
        /// Decrypts passed in data using encryption type provided. Will not work for MD5 or SHA1 as they cannot be decrypted.
        /// </summary>
        /// <param name="data">Data to Decrypt.</param>
        /// <param name="encryptionType">Encryption type to use.</param>
        /// <param name="key">Secret key to use in Decryption. Required for DES, RC2 and TripleDES.</param>
        /// <param name="vector">vector to use in Decryption. Required for DES, RC2 and TripleDES.</param>
        /// <returns>Decrypted data.</returns>
        public static string Decrypt(string data, EncryptionType encryptionType, string key, string vector)
        {
            byte[] keyBytes = null;
            if (!string.IsNullOrEmpty(key))
            {
                keyBytes = System.Text.Encoding.UTF8.GetBytes(key);
            }

            byte[] vectorBytes = null;
            if (!string.IsNullOrEmpty(vector))
            {
                vectorBytes = System.Text.Encoding.UTF8.GetBytes(vector);
            }

            return Decrypt(data, encryptionType, keyBytes, vectorBytes);
        }

        /// <summary>
        /// Gets a valid key and vector for DES.
        /// </summary>
        /// <param name="key">Valid DES Key.</param>
        /// <param name="vector">Valid DES Vecyor.</param>
        public static void CreateDESKeyVector(out byte[] key, out byte[] vector)
        {
            DESCryptoServiceProvider cryptoProvider = new DESCryptoServiceProvider();
            key = cryptoProvider.Key;
            vector = cryptoProvider.IV;
        }

        /// <summary>
        /// Gets a valid key and vector for RC2.
        /// </summary>
        /// <param name="key">Valid RC2 Key.</param>
        /// <param name="vector">Valid RC2 Vecyor.</param>
        public static void CreateRC2KeyVector(out byte[] key, out byte[] vector)
        {
            RC2CryptoServiceProvider cryptoProvider = new RC2CryptoServiceProvider();
            key = cryptoProvider.Key;
            vector = cryptoProvider.IV;
        }

        /// <summary>
        /// Gets a public and private key pair for use with RSA.
        /// </summary>
        /// <param name="privateKey">Private key.</param>
        /// <param name="publicKey">Public key.</param>
        public static void CreateRSAPrivatePublicKeys(out string privateKey, out string publicKey)
        {
            RSACryptoServiceProvider cryptoProvider = new RSACryptoServiceProvider();
            privateKey = cryptoProvider.ToXmlString(true);
            publicKey = cryptoProvider.ToXmlString(false);
        }

        /// <summary>
        /// Gets a valid key and vector for TripleDES.
        /// </summary>
        /// <param name="key">Valid TripleDES Key.</param>
        /// <param name="vector">Valid TripleDES Vecyor.</param>
        public static void CreateTripleDESKeyVector(out byte[] key, out byte[] vector)
        {
            TripleDESCryptoServiceProvider cryptoProvider = new TripleDESCryptoServiceProvider();
            key = cryptoProvider.Key;
            vector = cryptoProvider.IV;
        }

        #endregion Public Methods

        #region Private Methods

        /// <summary>
        /// Encrypts passed in string using DES.
        /// </summary>
        /// <param name="data">Data to encrypt.</param>
        /// <param name="key">Secret key to use in encryption.</param>
        /// <param name="vector">vector to use in encryption.</param>
        /// <returns>Encrypted result.</returns>
        private static string EncryptDES(string data, byte[] key, byte[] vector)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            if (key == null)
            {
                throw new ArgumentNullException("key");
            }

            if (vector == null)
            {
                throw new ArgumentNullException("vector");
            }

            DESCryptoServiceProvider cryptoProvider = new DESCryptoServiceProvider();
            MemoryStream memoryStream = new MemoryStream();
            CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoProvider.CreateEncryptor(key, vector), CryptoStreamMode.Write);
            StreamWriter writer = new StreamWriter(cryptoStream);
            writer.Write(data);
            writer.Flush();
            cryptoStream.FlushFinalBlock();
            writer.Flush();
            return Convert.ToBase64String(memoryStream.GetBuffer(), 0, (int)memoryStream.Length);
        }

        /// <summary>
        /// Decrypt a crypted DES string.
        /// </summary>
        /// <param name="cryptedString">The crypted string.</param>
        /// <param name="key">Secret key to use in encryption.</param>
        /// <param name="vector">vector to use in encryption.</param>
        /// <returns>The decrypted result.</returns>
        private static string DecryptDES(string cryptedString, byte[] key, byte[] vector)
        {
            if (cryptedString == null)
            {
                throw new ArgumentNullException("cryptedString");
            }

            if (key == null)
            {
                throw new ArgumentNullException("key");
            }

            if (vector == null)
            {
                throw new ArgumentNullException("vector");
            }

            DESCryptoServiceProvider cryptoProvider = new DESCryptoServiceProvider();
            MemoryStream memoryStream = new MemoryStream(Convert.FromBase64String(cryptedString));
            CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoProvider.CreateDecryptor(key, vector), CryptoStreamMode.Read);
            StreamReader reader = new StreamReader(cryptoStream);
            return reader.ReadToEnd();
        }

        /// <summary>
        /// Encrypt a string.
        /// </summary>
        /// <param name="data">The data to encrypt.</param>
        /// <returns>The encrypted result.</returns>
        private static byte[] EncryptSHA1(byte[] data)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            SHA1 sha = new SHA1CryptoServiceProvider();
            return sha.ComputeHash(data);
        }

        /// <summary>
        /// Encrypt a string using md5.
        /// </summary>
        /// <param name="data">The data to encrypt.</param>
        /// <returns>The encrypted result.</returns>
        private static byte[] EncryptMD5(byte[] data)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            MD5 md5 = new MD5CryptoServiceProvider();
            return md5.ComputeHash(data);
        }

        /// <summary>
        /// Encrypt a string using RC2.
        /// </summary>
        /// <param name="data">The data to encrypt.</param>
        /// <param name="key">Secret key to use in encryption.</param>
        /// <param name="vector">vector to use in encryption.</param>
        /// <returns>The encrypted result.</returns>
        private static string EncryptRC2(string data, byte[] key, byte[] vector)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            if (key == null)
            {
                throw new ArgumentNullException("key");
            }

            if (vector == null)
            {
                throw new ArgumentNullException("vector");
            }

            RC2CryptoServiceProvider cryptoProvider = new RC2CryptoServiceProvider();
            MemoryStream memoryStream = new MemoryStream();
            CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoProvider.CreateEncryptor(key, vector), CryptoStreamMode.Write);
            StreamWriter writer = new StreamWriter(cryptoStream);
            writer.Write(data);
            writer.Flush();
            cryptoStream.FlushFinalBlock();
            writer.Flush();
            return Convert.ToBase64String(memoryStream.GetBuffer(), 0, (int)memoryStream.Length);
        }

        /// <summary>
        /// Decrypt a crypted RC2 string.
        /// </summary>
        /// <param name="cryptedString">The crypted string.</param>
        /// <param name="key">Secret key to use in encryption.</param>
        /// <param name="vector">vector to use in encryption.</param>
        /// <returns>The decrypted result.</returns>
        private static string DecryptRC2(string cryptedString, byte[] key, byte[] vector)
        {
            if (cryptedString == null)
            {
                throw new ArgumentNullException("cryptedString");
            }

            if (key == null)
            {
                throw new ArgumentNullException("key");
            }

            if (vector == null)
            {
                throw new ArgumentNullException("vector");
            }

            RC2CryptoServiceProvider cryptoProvider = new RC2CryptoServiceProvider();
            MemoryStream memoryStream = new MemoryStream(Convert.FromBase64String(cryptedString));
            CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoProvider.CreateDecryptor(key, vector), CryptoStreamMode.Read);
            StreamReader reader = new StreamReader(cryptoStream);
            return reader.ReadToEnd();
        }

        /// <summary>
        /// Encrypt a string using RSA.
        /// </summary>
        /// <param name="data">The data to encrypt.</param>
        /// <param name="publicKey">Public key for encryption.</param>
        /// <returns>The encrypted result.</returns>
        private static byte[] EncryptRSA(byte[] data, string publicKey)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            if (publicKey == null)
            {
                throw new ArgumentNullException("publicKey");
            }

            RSACryptoServiceProvider cryptoProvider = new RSACryptoServiceProvider();
            cryptoProvider.FromXmlString(publicKey);
            return cryptoProvider.Encrypt(data, false);
        }

        /// <summary>
        /// Decrypt a crypted RSA string.
        /// </summary>
        /// <param name="cryptedData">The crypted string.</param>
        /// <param name="privateKey">Private key for decryption.</param>
        /// <returns>The decrypted result.</returns>
        private static byte[] DecryptRSA(byte[] cryptedData, string privateKey)
        {
            if (cryptedData == null)
            {
                throw new ArgumentNullException("cryptedData");
            }

            if (privateKey == null)
            {
                throw new ArgumentNullException("privateKey");
            }

            RSACryptoServiceProvider cryptoProvider = new RSACryptoServiceProvider();
            cryptoProvider.FromXmlString(privateKey);
            return cryptoProvider.Decrypt(cryptedData, false);
        }

        /// <summary>
        /// Encrypts passed in string using TripleDES.
        /// </summary>
        /// <param name="data">Data to encrypt.</param>
        /// <param name="key">Secret key to use in encryption.</param>
        /// <param name="vector">vector to use in encryption.</param>
        /// <returns>Encrypted result.</returns>
        private static string EncryptTripleDES(string data, byte[] key, byte[] vector)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }

            if (key == null)
            {
                throw new ArgumentNullException("key");
            }

            if (vector == null)
            {
                throw new ArgumentNullException("vector");
            }

            TripleDESCryptoServiceProvider cryptoProvider = new TripleDESCryptoServiceProvider();
            KeySizes[] s = cryptoProvider.LegalKeySizes;
            MemoryStream memoryStream = new MemoryStream();
            CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoProvider.CreateEncryptor(key, vector), CryptoStreamMode.Write);
            StreamWriter writer = new StreamWriter(cryptoStream);
            writer.Write(data);
            writer.Flush();
            cryptoStream.FlushFinalBlock();
            writer.Flush();
            return Convert.ToBase64String(memoryStream.GetBuffer(), 0, (int)memoryStream.Length);
        }

        /// <summary>
        /// Decrypt a crypted TripleDES string.
        /// </summary>
        /// <param name="cryptedString">The crypted data.</param>
        /// <param name="key">Secret key to use in encryption.</param>
        /// <param name="vector">vector to use in encryption.</param>
        /// <returns>The decrypted data.</returns>
        private static string DecryptTripleDES(string cryptedString, byte[] key, byte[] vector)
        {
            if (cryptedString == null)
            {
                throw new ArgumentNullException("cryptedString");
            }

            if (key == null)
            {
                throw new ArgumentNullException("key");
            }

            if (vector == null)
            {
                throw new ArgumentNullException("vector");
            }

            TripleDESCryptoServiceProvider cryptoProvider = new TripleDESCryptoServiceProvider();
            MemoryStream memoryStream = new MemoryStream(Convert.FromBase64String(cryptedString));
            CryptoStream cryptoStream = new CryptoStream(memoryStream, cryptoProvider.CreateDecryptor(key, vector), CryptoStreamMode.Read);
            StreamReader reader = new StreamReader(cryptoStream);
            return reader.ReadToEnd();
        }

        #endregion Private Methods
    }
}
