//-----------------------------------------------------------------------
// <copyright file="TimesheetActivityEntryCollection.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a TimesheetActivityEntryCollection.</summary>
//-----------------------------------------------------------------------
namespace BusinessObjects.Collections
{
    using System;
    using System.Collections.Generic;

    /// <summary>
    /// Class that represents a collection of Timesheet Activity Entries.
    /// </summary>
    [Serializable]
    public class TimesheetActivityEntryCollection : List<TimesheetActivityEntry>
    {
        /// <summary>
        /// Gets all entries for the date specified.
        /// </summary>
        /// <param name="date">Date to search for.</param>
        /// <returns>Collection with matching dates.</returns>
        public TimesheetActivityEntryCollection GetDateEntries(DateTime date)
        {
            TimesheetActivityEntryCollection collection = new TimesheetActivityEntryCollection();
            foreach (TimesheetActivityEntry entry in this)
            {
                if (entry.Date.Date == date.Date)
                {
                    collection.Add(entry);
                }
            }

            return collection;
        }

        /// <summary>
        /// Gets all entries for the row index specified.
        /// </summary>
        /// <param name="index">Row Index to search for.</param>
        /// <returns>Collection with matching dates.</returns>
        public TimesheetActivityEntryCollection GetRowIndexEntries(int index)
        {
            TimesheetActivityEntryCollection collection = new TimesheetActivityEntryCollection();

            // Get a list of all the uniquw activity ids.
            List<long> activityIds = new List<long>();
            foreach (TimesheetActivityEntry entry in this)
            {
                if (activityIds.Contains(entry.ActivityId) == false)
                {
                    activityIds.Add(entry.ActivityId);
                }
            }

            if (activityIds.Count > index)
            {
                foreach (TimesheetActivityEntry entry in this)
                {
                    if (entry.ActivityId == activityIds[index])
                    {
                        collection.Add(entry);
                    }
                }
            }

            return collection;
        }

        /// <summary>
        /// Gets an entry with matching date and the correct index, if it exists.
        /// </summary>
        /// <param name="date">Date to search for.</param>
        /// <param name="index">Index to return.</param>
        /// <returns>Matching Entry ot null.</returns>
        public TimesheetActivityEntry GetEntry(DateTime date, int index)
        {
            TimesheetActivityEntryCollection collection = this.GetDateEntries(date);
            if (collection.Count > index)
            {
                return collection[index];
            }

            return null;
        }

        /// <summary>
        /// Gets an entry with matching date and the correct activityId, if it exists.
        /// </summary>
        /// <param name="date">Date to search for.</param>
        /// <param name="activityId">The activity Id.</param>
        /// <returns>Matching Entry ot null.</returns>
        public TimesheetActivityEntry GetEntry(DateTime date, long activityId)
        {
            TimesheetActivityEntryCollection collection = this.GetDateEntries(date);
            foreach (TimesheetActivityEntry entry in collection)
            {
                if (entry.ActivityId == activityId)
                {
                    return entry;
                }
            }

            return null;
        }

        /// <summary>
        /// Gets the activity id of the row.
        /// </summary>
        /// <param name="index">Index of the row.</param>
        /// <returns>Activity id.</returns>
        public long GetActivityId(int index)
        {
            // Get a list of all the uniquw activity ids.
            List<long> activityIds = new List<long>();
            foreach (TimesheetActivityEntry entry in this)
            {
                if (activityIds.Contains(entry.ActivityId) == false)
                {
                    activityIds.Add(entry.ActivityId);
                }
            }

            return activityIds[index];
        }
    }
}