//-----------------------------------------------------------------------
// <copyright file="CommunicationManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that manages communications.</summary>
//-----------------------------------------------------------------------
namespace CommunicationManager
{
    using System;
    using System.Net.Mail;
    using System.Threading;

    /// <summary>
    /// Class that manages communication. Handles queued messages on a seperate thread to reduce overheads.
    /// </summary>
    public class CommunicationManager
    {
        #region Private Members

        /// <summary>Holds queue of messages that need to be sent.</summary>
        private MessageQueue messageQueue;

        /// <summary>Represents the lock for the message queue.</summary>
        private object messageQueueLock;

        /// <summary>Holds the message handling thread.</summary>
        private volatile Thread messageHandlingThread;

        /// <summary>Represents the lock for the message Handling Thread.</summary>
        private object messageHandlingThreadLock;

        /// <summary>Holds whether or not the manager has started sending messages.</summary>
        private bool startedHandlingMessages = false;

        /// <summary>Holds the handler for message processing messages.</summary>
        private MessageStatusHandler messageStatusHandler;

        /// <summary>Holds the configuration information.</summary>
        private CommunicationManagerConfiguration communicationManagerConfiguration;

        #endregion Private Members

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the CommunicationManager class.
        /// </summary>
        public CommunicationManager()
        {
            this.Initialise(null, null);
        }

        /// <summary>
        /// Initializes a new instance of the CommunicationManager class.
        /// </summary>
        /// <param name="config">The object containing the configuration information.</param>
        /// <param name="messageStatusHandler">The object that handles message success/errors.</param>
        public CommunicationManager(CommunicationManagerConfiguration config, MessageStatusHandler messageStatusHandler)
        {
            this.Initialise(config, messageStatusHandler);
        }

        #endregion Constructors

        #region Destructor

        /// <summary>
        /// Finalizes an instance of the CommunicationManager class.
        /// </summary>
        ~CommunicationManager()
        {
            this.messageHandlingThread.Abort();
        }

        #endregion Destructor

        #region Public Properties

        /// <summary>
        /// Gets the number of messages in the message queue.
        /// </summary>
        public int MessageCount
        {
            get
            {
                lock (this.messageQueueLock)
                {
                    return this.messageQueue.Count;
                }
            }
        }

        #endregion Piblic Properties

        #region Public Methods

        /// <summary>
        /// Adds a message to the Message queue.
        /// </summary>
        /// <param name="message">The message to add to the queue.</param>
        public void AddMessage(Message message)
        {
            if (message == null)
            {
                throw new ArgumentNullException("message");
            }

            // Lock queue
            lock (this.messageQueueLock)
            {
                // Add message to queue
                this.messageQueue.Enqueue(message);

                // Lock thread objects
                lock (this.messageHandlingThreadLock)
                {
                    // Check if thread isn't running
                    if (this.messageHandlingThread.ThreadState != ThreadState.Running)
                    {
                        // Thread is not running, check to see if we need to start it straight away.
                        if (this.communicationManagerConfiguration.StartOnMessageAdd
                            || this.startedHandlingMessages == true)
                        {
                            this.StartHandlingMessages();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Starts handling messages of the message queue.
        /// </summary>
        public void StartHandlingMessages()
        {
            lock (this.messageHandlingThreadLock)
            {
                try
                {
                    if (this.messageHandlingThread.ThreadState == ThreadState.Running)
                    {
                        // Thread already running.
                        return;
                    }

                    this.startedHandlingMessages = true;
                    this.messageHandlingThread = new Thread(HandleMessages);
                    this.messageHandlingThread.Start(this);
                }
                catch (Exception ex)
                {
                    if (this.messageStatusHandler != null)
                    {
                        this.messageStatusHandler.HandleError(ex);
                    }
                }
            }
        }

        /// <summary>
        /// Stops handling messages of the message queue.
        /// </summary>
        public void StopHandlingMessages()
        {
            lock (this.messageHandlingThreadLock)
            {
                try
                {
                    this.startedHandlingMessages = false;
                    this.messageHandlingThread.Abort();
                }
                catch (Exception ex)
                {
                    if (this.messageStatusHandler != null)
                    {
                        this.messageStatusHandler.HandleError(ex);
                    }
                }
            }
        }

        #endregion Public Methods

        #region Private Static Methods

        /// <summary>
        /// Thread method that handles messages of the message queue.
        /// </summary>
        /// <param name="obj">Handle to communication manager.</param>
        private static void HandleMessages(object obj)
        {
            // Get comm object
            CommunicationManager communicationManager = obj as CommunicationManager;

            // Loop for each message.
            while (communicationManager.messageQueue.Count > 0)
            {
                // check to see if we haven't stopped the thread.
                lock (communicationManager.messageHandlingThreadLock)
                {
                    if (!communicationManager.startedHandlingMessages)
                    {
                        break;
                    }
                }

                // get a message of the queue.
                Message message = null;
                lock (communicationManager.messageQueueLock)
                {
                    message = communicationManager.messageQueue.Dequeue();
                }

                if (message != null)
                {
                    try
                    {
                        message.HandleMessage();

                        if (communicationManager.messageStatusHandler != null)
                        {
                            try
                            {
                                communicationManager.messageStatusHandler.HandleSuccess(message);
                            }
                            catch
                            {
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        if (communicationManager.messageStatusHandler != null)
                        {
                            try
                            {
                                communicationManager.messageStatusHandler.HandleError(message, ex);
                            }
                            catch
                            {
                            }
                        }
                    }
                }
            }
        }

        #endregion Private Static Methods

        #region Private Methods

        /// <summary>
        /// Initialise the Communication Manager.
        /// </summary>
        /// <param name="config">The object containing the configuration information.</param>
        /// <param name="messageStatusHandler">The object that handles message success/errors.</param>
        private void Initialise(CommunicationManagerConfiguration config, MessageStatusHandler messageStatusHandler)
        {
            this.messageQueueLock = new object();
            this.messageHandlingThreadLock = new object();
            this.messageQueue = new MessageQueue();
            this.messageStatusHandler = messageStatusHandler;
            if (config == null)
            {
                this.communicationManagerConfiguration = new CommunicationManagerConfiguration();
            }
            else
            {
                this.communicationManagerConfiguration = config;
            }

            this.messageHandlingThread = new Thread(HandleMessages);
        }

        #endregion Private Methods
    }
}
