//-----------------------------------------------------------------------
// <copyright file="EmailMessage.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents EmailMessage.</summary>
//-----------------------------------------------------------------------
namespace CommunicationManager
{
    using System;
    using System.Net.Mail;

    /// <summary>
    /// Class that represents a Message.
    /// </summary>
    public class EmailMessage : Message
    {
        #region Private Members

        /// <summary>Holds the Host of the Smtp Client.</summary>
        private string host;

        /// <summary>Holds the Port of the Smtp Client.</summary>
        private int port;

        /// <summary>Holds the message data.</summary>
        private MailMessage message;

        /// <summary>Holds the Username of the Smtp Client.</summary>
        private string username;

        /// <summary>Holds the Password of the Smtp Client.</summary>
        private string password;

        /// <summary>Holds the Domain of the Smtp Client.</summary>
        private string domain;

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the EmailMessage class.
        /// </summary>
        /// <param name="message">The message data.</param>
        /// <param name="priority">The priority of the message.</param>
        public EmailMessage(MailMessage message, MessagePriority priority)
            : base(priority)
        {
            this.Intialise(null, 0, null, null, null, message);
        }

        /// <summary>
        /// Initializes a new instance of the EmailMessage class.
        /// </summary>
        /// <param name="host">The host of the smtp client.</param>
        /// <param name="port">The port of the smtp client.</param>
        /// <param name="message">The message data.</param>
        /// <param name="priority">The priority of the message.</param>
        public EmailMessage(string host, int port, MailMessage message, MessagePriority priority)
            : base(priority)
        {
            this.Intialise(host, port, null, null, null, message);
        }

        /// <summary>
        /// Initializes a new instance of the EmailMessage class.
        /// </summary>
        /// <param name="host">The host of the smtp client.</param>
        /// <param name="port">The port of the smtp client.</param>
        /// <param name="username">Username to access smtp server.</param>
        /// <param name="password">Password to access smtp server.</param>
        /// <param name="domain">Domain to access the smtp server.</param>
        /// <param name="message">The message data.</param>
        /// <param name="priority">The priority of the message.</param>
        public EmailMessage(string host, int port, string username, string password, string domain, MailMessage message, MessagePriority priority)
            : base(priority)
        {
            this.Intialise(host, port, username, password, domain, message);
        }

        #endregion Constructor

        #region Public Methods

        /// <summary>Methods that handles the message object.</summary>
        public override void HandleMessage()
        {
            SmtpClient client = new SmtpClient(this.host, this.port);
            if (!string.IsNullOrEmpty(this.username))
            {
                client.UseDefaultCredentials = false;
                client.Credentials = new System.Net.NetworkCredential(this.username, this.password, this.domain);
            }

            client.Send(this.message);
        }

        #endregion Public Methods

        #region Private Methods

        /// <summary>
        /// Initializes a the EmailMessage object.
        /// </summary>
        /// <param name="host">The host of the smtp client.</param>
        /// <param name="port">The port of the smtp client.</param>
        /// <param name="username">Username to access smtp server.</param>
        /// <param name="password">Password to access smtp server.</param>
        /// <param name="domain">Domain to access the smtp server.</param>
        /// <param name="message">The message data.</param>
        public void Intialise(string host, int port, string username, string password, string domain, MailMessage message)
        {
            if (message == null)
            {
                throw new ArgumentNullException("message");
            }

            this.host = host;
            this.port = port;
            this.message = message;
            this.username = username;
            this.password = password;
            this.domain = domain;
        }

        #endregion Private Methods
    }
}
