//-----------------------------------------------------------------------
// <copyright file="ActivityDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Activity Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows Activity Data to be manipultaed.
    /// </summary>
    public sealed class ActivityDataObject
    {
        #region Private Members

        /// <summary>Holds static ActivityDataObject objects.</summary>
        private static volatile ActivityDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the ActivityDataObject class from being created.
        /// </summary>
        private ActivityDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an ActivityDataObject object. 
        /// </summary>
        public static ActivityDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new ActivityDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new activity.
        /// </summary>
        /// <param name="jobCodeId">Job Code that the activity belongs to.</param>
        /// <param name="code">Code of activity.</param>
        /// <param name="name">Name of activity.</param>
        /// <returns>Id of new activity.</returns>
        public long AddActivity(long jobCodeId, string code, string name)
        {
            try
            {
                if (string.IsNullOrEmpty(name))
                {
                    throw new ArgumentNullException("name");
                }

                return DataObject.DataAccessor.AddActivity(jobCodeId, code, name);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing activity.
        /// </summary>
        /// <param name="activity">Activity to update.</param>
        public void UpdateActivity(Activity activity)
        {
            try
            {
                if (activity == null)
                {
                    throw new ArgumentNullException("activity");
                }

                DataObject.DataAccessor.UpdateActivity(activity);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to delete.</param>
        public void DeleteActivity(long activityId)
        {
            try
            {
                DataObject.DataAccessor.DeleteActivity(activityId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of activities that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public ActivityCollection GetActivities(ActivityFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetActivities(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets an activity that matches the id.
        /// </summary>
        /// <param name="activityId">Id to use when retrieving data.</param>
        /// <returns>Matching activity.</returns>
        public Activity GetActivity(long activityId)
        {
            try
            {
                return DataObject.DataAccessor.GetActivity(activityId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disables existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to disable.</param>
        public void DisableActivity(long activityId)
        {
            try
            {
                DataObject.DataAccessor.DisableActivity(activityId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to enable.</param>
        public void EnableActivity(long activityId)
        {
            try
            {
                DataObject.DataAccessor.EnableActivity(activityId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
