//-----------------------------------------------------------------------
// <copyright file="GenderDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Gender Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows Gender Data to be manipultaed.
    /// </summary>
    public class GenderDataObject
    {
        #region Private Members

        /// <summary>Holds static GenderDataObject objects.</summary>
        private static volatile GenderDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the GenderDataObject class from being created.
        /// </summary>
        private GenderDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an GenderDataObject object. 
        /// </summary>
        public static GenderDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new GenderDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Gender.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <param name="name">Name of Gender.</param>
        /// <returns>Id of new Gender.</returns>
        public short AddGender(byte genderId, string name)
        {
            try
            {
                return DataObject.DataAccessor.AddGender(genderId, name);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Gender.
        /// </summary>
        /// <param name="gender">Gender to update.</param>
        public void UpdateGender(Gender gender)
        {
            try
            {
                DataObject.DataAccessor.UpdateGender(gender);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Gender.
        /// </summary>
        /// <param name="genderId">Id of Gender to delete.</param>
        public void DeleteGender(byte genderId)
        {
            try
            {
                DataObject.DataAccessor.DeleteGender(genderId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a Gender that matches the id.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <returns>gender matching the id.</returns>
        public Gender GetGender(byte genderId)
        {
            try
            {
                return DataObject.DataAccessor.GetGender(genderId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Genders that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public GenderCollection GetGenders(GenderFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetGenders(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
