//-----------------------------------------------------------------------
// <copyright file="IDataAccessor.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds interface that represensts methods used to access and modify data.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Interface that represensts methods used to access and modify data from a data source.
    /// </summary>
    internal interface IDataAccessor
    {
        #region Common Methods

        /// <summary>
        /// Handles any exception that is thrown during data access and modification.
        /// </summary>
        /// <param name="ex">Exception to handle.</param>
        void HandleException(Exception ex);

        #endregion Common Methods

        #region Activity Methods

        /// <summary>
        /// Adds new activity.
        /// </summary>
        /// <param name="jobCodeId">Job Code that the activity belongs to.</param>
        /// <param name="code">Code of activity.</param>
        /// <param name="name">Name of activity.</param>
        /// <returns>Id of new activity.</returns>
        long AddActivity(long jobCodeId, string code, string name);

        /// <summary>
        /// Updates existing activity.
        /// </summary>
        /// <param name="activity">Activity to update.</param>
        void UpdateActivity(Activity activity);

        /// <summary>
        /// Deletes existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to delete.</param>
        void DeleteActivity(long activityId);

        /// <summary>
        /// Gets a collection of activities that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        ActivityCollection GetActivities(ActivityFilter filter);

        /// <summary>
        /// Gets an activity that matches the id.
        /// </summary>
        /// <param name="activityId">Id to use when retrieving data.</param>
        /// <returns>Matching activity.</returns>
        Activity GetActivity(long activityId);

        /// <summary>
        /// Disbales existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to disable.</param>
        void DisableActivity(long activityId);

        /// <summary>
        /// Enables existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to enable.</param>
        void EnableActivity(long activityId);

        #endregion Activity Methods

        #region Timesheet Methods

        /// <summary>
        /// Adds new Timesheet.
        /// </summary>
        /// <param name="userId">id of user the timesheet is for.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="weekEndDate">The end date of the week.</param>
        /// <returns>Id of new Timesheet.</returns>
        long AddTimesheet(long userId, short timesheetStatusId, DateTime weekEndDate);

        /// <summary>
        /// Adds new Timesheet.
        /// </summary>
        /// <param name="userId">id of user the timesheet is for.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="timesheet">Timesheet to save.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to save.</param>
        /// <param name="timesheetActivityEntries">Related activities to save.</param>
        /// <returns>Id of new Timesheet.</returns>
        long AddTimesheet(
            long userId,
            short timesheetStatusId,
            Timesheet timesheet,
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries);

        /// <summary>
        /// Updates existing Timesheet.
        /// </summary>
        /// <param name="userId">id of the udpater user.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="timesheet">Timesheet to save.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to save.</param>
        /// <param name="timesheetActivityEntries">Related activities to save.</param>
        /// <returns>Id of new Timesheet.</returns>
        long UpdateTimesheet(
            long userId,
            short timesheetStatusId,
            Timesheet timesheet,
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries);

        /// <summary>
        /// Deletes existing Timesheet.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to delete.</param>
        void DeleteTimesheet(long timesheetId);

        /// <summary>
        /// Gets a collection of Timesheets that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        TimesheetCollection GetTimesheets(TimesheetFilter filter);

        /// <summary>
        /// Gets the number of Timesheets that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>The number of Timesheets.</returns>
        long GetTimesheetsCount(TimesheetFilter filter);

        /// <summary>
        /// Gets a Timesheets that matches the id.
        /// </summary>
        /// <param name="timesheetId">Id of the timesheet.</param>
        /// <returns>Matching timesheet.</returns>
        Timesheet GetTimesheet(long timesheetId);

        /// <summary>
        /// Gets a timesheet that matches the information provided.
        /// </summary>
        /// <param name="userId">Id of timesheet.</param>
        /// <param name="weekEndDate">The date of the weekend.</param>
        /// <returns>Matching timesheet.</returns>
        Timesheet GetTimesheet(long userId, DateTime weekEndDate);

        /// <summary>
        /// Changes existing Timesheet's status.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to change.</param>
        /// <param name="statusChangerUserId">Id of user changing status.</param>
        /// <param name="timesheetStatusId">Id of timesheet's new status.</param>
        /// <param name="statusChangeDescription">Description related to change.</param>
        void ChangeTimesheetStatus(long timesheetId, long statusChangerUserId, short timesheetStatusId, string statusChangeDescription);

        #endregion Timesheet Methods

        #region Timesheet Activity Entry Methods

        /// <summary>
        /// Adds new Timesheet Activity entry.
        /// </summary>
        /// <param name="timesheetId">id of timesheet the entry is for.</param>
        /// <param name="activityId">Id of activity is for.</param>
        /// <param name="hours">Number od hours for entry.</param>
        /// <param name="description">Description of entry.</param>
        /// <param name="date">Date for entry.</param>
        /// <returns>Id of new entry.</returns>
        long AddTimesheetActivityEntry(long timesheetId, long activityId, decimal hours, string description, DateTime date);

        /// <summary>
        /// Updates existing TimesheetActivityEntry.
        /// </summary>
        /// <param name="timesheetActivityEntry">Timesheet Activity Entry to update.</param>
        void UpdateTimesheetActivityEntry(TimesheetActivityEntry timesheetActivityEntry);

        /// <summary>
        /// Deletes existing Timesheet Activity Entry.
        /// </summary>
        /// <param name="timesheetActivityEntryId">Id of Timesheet Activity Entry to delete.</param>
        void DeleteTimesheetActivityEntry(long timesheetActivityEntryId);

        /// <summary>
        /// Deletes existing Timesheet Activity Entries.
        /// </summary>
        /// <param name="timesheetId">Id of timesheet to delete entries for.</param>
        void DeleteTimesheetActivityEntries(long timesheetId);

        /// <summary>
        /// Gets a collection of Timesheet Activity Entries that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        TimesheetActivityEntryCollection GetTimesheetActivityEntries(TimesheetActivityEntryFilter filter);

        #endregion Timesheet Activity Entry Methods

        #region Timesheet Attendance Entry Methods

        /// <summary>
        /// Adds new Timesheet Attendance entry.
        /// </summary>
        /// <param name="timesheetId">id of timesheet the entry is for.</param>
        /// <param name="startDateTime">Start Date time of attendance.</param>
        /// <param name="endDateTime">End Date time of attendance.</param>
        /// <returns>Id of new entry.</returns>
        long AddTimesheetAttendanceEntry(long timesheetId, DateTime startDateTime, DateTime endDateTime);

        /// <summary>
        /// Updates existing TimesheetAttendanceEntry.
        /// </summary>
        /// <param name="timesheetAttendanceEntry">Timesheet Attendance Entry to update.</param>
        void UpdateTimesheetAttendanceEntry(TimesheetAttendanceEntry timesheetAttendanceEntry);

        /// <summary>
        /// Deletes existing Timesheet Attendance Entry.
        /// </summary>
        /// <param name="timesheetAttendanceEntryId">Id of Timesheet Attendance Entry to delete.</param>
        void DeleteTimesheetAttendanceEntry(long timesheetAttendanceEntryId);

        /// <summary>
        /// Deletes existing Timesheet Attendance Entries.
        /// </summary>
        /// <param name="timesheetId">Id of timesheet to delete entries for.</param>
        void DeleteTimesheetAttendanceEntries(long timesheetId);

        /// <summary>
        /// Gets a collection of Timesheet Attendance Entries that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        TimesheetAttendanceEntryCollection GetTimesheetAttendanceEntries(TimesheetAttendanceEntryFilter filter);

        #endregion Timesheet Attendance Entry Methods

        #region Timesheet Status Methods

        /// <summary>
        /// Adds new Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of timesheet status.</param>
        /// <param name="name">Name of Timesheet Status.</param>
        /// <returns>Id of new Timesheet Status.</returns>
        short AddTimesheetStatus(short timesheetStatusId, string name);

        /// <summary>
        /// Updates existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatus">Timesheet Status to update.</param>
        void UpdateTimesheetStatus(TimesheetStatus timesheetStatus);

        /// <summary>
        /// Deletes existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of Timesheet Status to delete.</param>
        void DeleteTimesheetStatus(short timesheetStatusId);

        /// <summary>
        /// Gets a collection of Timesheet Statuses that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        TimesheetStatusCollection GetTimesheetStatuses(TimesheetStatusFilter filter);

        /// <summary>
        /// Gets a Timesheet Status that matches the id.
        /// </summary>
        /// <param name="timesheetStatusId">Id to use when retrieving data.</param>
        /// <returns>Matching Timesheet Status.</returns>
        TimesheetStatus GetTimesheetStatus(short timesheetStatusId);

        #endregion Timesheet Status Methods

        #region Gender Methods

        /// <summary>
        /// Adds new Gender.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <param name="name">Name of Gender.</param>
        /// <returns>Id of new Gender.</returns>
        byte AddGender(byte genderId, string name);

        /// <summary>
        /// Updates existing Gender.
        /// </summary>
        /// <param name="gender">Gender to update.</param>
        void UpdateGender(Gender gender);

        /// <summary>
        /// Deletes existing Gender.
        /// </summary>
        /// <param name="genderId">Id of Gender to delete.</param>
        void DeleteGender(byte genderId);

        /// <summary>
        /// Gets a Gender that matches the id.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <returns>gender matching the id.</returns>
        Gender GetGender(byte genderId);

        /// <summary>
        /// Gets a collection of Genders that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        GenderCollection GetGenders(GenderFilter filter);

        #endregion Gender Methods

        #region Job Code Methods

        /// <summary>
        /// Adds new Job Code.
        /// </summary>
        /// <param name="code">The job code.</param>
        /// <param name="name">Name of Job Code.</param>
        /// <param name="organisationId">Organisation that the job code belongs to.</param>
        /// <returns>Id of new JobCode.</returns>
        long AddJobCode(string code, string name, int organisationId);

        /// <summary>
        /// Updates existing Job Code.
        /// </summary>
        /// <param name="jobCode">Job Code to update.</param>
        void UpdateJobCode(JobCode jobCode);

        /// <summary>
        /// Deletes existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to delete.</param>
        void DeleteJobCode(long jobCodeId);

        /// <summary>
        /// Gets a collection of Job Codes that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        JobCodeCollection GetJobCodes(JobCodeFilter filter);

        /// <summary>
        /// Disables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to disable.</param>
        void DisableJobCode(long jobCodeId);

        /// <summary>
        /// Enables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to enable.</param>
        void EnableJobCode(long jobCodeId);

        /// <summary>
        /// Gets existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to get.</param>
        /// <returns>matching Job Code.</returns>
        JobCode GetJobCode(long jobCodeId);

        #endregion Job Code Methods

        #region Organisation Methods

        /// <summary>
        /// Adds new Organisation.
        /// </summary>
        /// <param name="name">Name of Organisation.</param>
        /// <returns>Id of new Organisation.</returns>
        int AddOrganisation(string name);

        /// <summary>
        /// Updates existing Organisation.
        /// </summary>
        /// <param name="organisation">Organisation to update.</param>
        void UpdateOrganisation(Organisation organisation);

        /// <summary>
        /// Deletes existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to delete.</param>
        void DeleteOrganisation(int organisationId);

        /// <summary>
        /// Gets a collection of Organisations that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        OrganisationCollection GetOrganisations(OrganisationFilter filter);

        /// <summary>
        /// Gets an Organisations that matches the id.
        /// </summary>
        /// <param name="organisationId">Id to use when retrieving data.</param>
        /// <returns>Elements that match the id.</returns>
        Organisation GetOrganisation(int organisationId);

        /// <summary>
        /// Disbales existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to disbale.</param>
        void DisableOrganisation(int organisationId);

        /// <summary>
        /// Enables existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to enable.</param>
        void EnableOrganisation(int organisationId);

        #endregion Organisation Methods

        #region Role Methods

        /// <summary>
        /// Adds new Role.
        /// </summary>
        /// <param name="roleId">id of role.</param>
        /// <param name="name">Name of Role.</param>
        /// <returns>Id of new Role.</returns>
        short AddRole(short roleId, string name);

        /// <summary>
        /// Updates existing Role.
        /// </summary>
        /// <param name="role">Role to update.</param>
        void UpdateRole(Role role);

        /// <summary>
        /// Deletes existing Role.
        /// </summary>
        /// <param name="roleId">Id of Role to delete.</param>
        void DeleteRole(short roleId);

        /// <summary>
        /// Gets a Role that matches the id.
        /// </summary>
        /// <param name="roleId">Id of role.</param>
        /// <returns>Matching role.</returns>
        Role GetRole(short roleId);

        /// <summary>
        /// Gets a collection of Roles that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        RoleCollection GetRoles(RoleFilter filter);

        #endregion Role Methods

        #region User Methods

        /// <summary>
        /// Adds new User.
        /// </summary>
        /// <param name="employeeId">Employee id of user.</param>
        /// <param name="genderId">Gender id of the user.</param>
        /// <param name="roleId">Role of user.</param>
        /// <param name="firstname">Firstname of user.</param>
        /// <param name="surname">Surname of user.</param>
        /// <param name="email">Email of user.</param>
        /// <param name="password">password of user.</param>
        /// <param name="organisationId">Id of organisation the user belongs to.</param>
        /// <param name="approverUserId">The user who will approve this users timesheet entries.</param>
        /// <returns>Id of user.</returns>
        long AddUser(string employeeId, byte genderId, short roleId, string firstname, string surname, string email, string password, int? organisationId, long? approverUserId);
        
        /// <summary>
        /// Updates existing user.
        /// </summary>
        /// <param name="user">User to update.</param>
        void UpdateUser(User user);

        /// <summary>
        /// Deletes existing User.
        /// </summary>
        /// <param name="userId">Id of User to delete.</param>
        void DeleteUser(long userId);

        /// <summary>
        /// Gets the User that matches the id.
        /// </summary>
        /// <param name="userId">id of user.</param>
        /// <returns>User information.</returns>
        User GetUser(long userId);

        /// <summary>
        /// Gets the User that matches the email.
        /// </summary>
        /// <param name="email">email of user.</param>
        /// <returns>User information.</returns>
        User GetUser(string email);

        /// <summary>
        /// Gets a collection of Users that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        UserCollection GetUsers(UserFilter filter);

        /// <summary>
        /// Gets the number of Users that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>The number of users.</returns>
        long GetUsersCount(UserFilter filter);

        /// <summary>
        /// Disables existing user.
        /// </summary>
        /// <param name="userId">Id of User to disable.</param>
        void DisableUser(long userId);

        /// <summary>
        /// Enables existing user.
        /// </summary>
        /// <param name="userId">Id of User to enable.</param>
        void EnableUser(long userId);

        /// <summary>
        /// Activates existing user.
        /// </summary>
        /// <param name="userId">Id of User to activate.</param>
        void ActivateUser(long userId);

        /// <summary>
        /// Deactivates existing user.
        /// </summary>
        /// <param name="userId">Id of User to Deactivate.</param>
        void DeactivateUser(long userId);

        /// <summary>
        /// Updates user's password.
        /// </summary>
        /// <param name="userId">Id of User to change password for.</param>
        /// <param name="password">The users new password.</param>
        void SetUserPassword(long userId, string password);

        /// <summary>
        /// Gets the collection of users who are above the passed in user in terms of approver structure.
        /// </summary>
        /// <param name="userId">Id of user to find hierachy for.</param>
        /// <param name="inclusive">Whether or not to include the passed in user in the result.</param>
        /// <returns>Users that are above the passed in user in terms of heirachy</returns>
        UserCollection GetUserHierachyAboveUser(long userId, bool inclusive);

        /// <summary>
        /// Gets the collection of users who are below the passed in user in terms of approver structure.
        /// </summary>
        /// <param name="userId">Id of user to find hierachy for.</param>
        /// <param name="inclusive">Whether or not to include the passed in user in the result.</param>
        /// <returns>Users that are below the passed in user in terms of heirachy</returns>
        UserCollection GetUserHierachyBelowUser(long userId, bool inclusive);

        #endregion User Methods
    }
}
