//-----------------------------------------------------------------------
// <copyright file="JobCodeDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Job Code Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows Job Code Data to be manipultaed.
    /// </summary>
    public class JobCodeDataObject
    {
        #region Private Members

        /// <summary>Holds static JobCodeDataObject objects.</summary>
        private static volatile JobCodeDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the JobCodeDataObject class from being created.
        /// </summary>
        private JobCodeDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an JobCodeDataObject object. 
        /// </summary>
        public static JobCodeDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new JobCodeDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Job Code.
        /// </summary>
        /// <param name="code">The job code.</param>
        /// <param name="name">Name of Job Code.</param>
        /// <param name="organisationId">Organisation that the job code belongs to.</param>
        /// <returns>Id of new JobCode.</returns>
        public long AddJobCode(string code, string name, int organisationId)
        {
            try
            {
                return DataObject.DataAccessor.AddJobCode(code, name, organisationId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Job Code.
        /// </summary>
        /// <param name="jobCode">Job Code to update.</param>
        public void UpdateJobCode(JobCode jobCode)
        {
            try
            {
                DataObject.DataAccessor.UpdateJobCode(jobCode);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to delete.</param>
        public void DeleteJobCode(long jobCodeId)
        {
            try
            {
                DataObject.DataAccessor.DeleteJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Job Codes that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public JobCodeCollection GetJobCodes(JobCodeFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetJobCodes(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to get.</param>
        /// <returns>matching Job Code.</returns>
        public JobCode GetJobCode(long jobCodeId)
        {
            try
            {
                return DataObject.DataAccessor.GetJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to disable.</param>
        public void DisableJobCode(long jobCodeId)
        {
            try
            {
                DataObject.DataAccessor.DisableJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to enable.</param>
        public void EnableJobCode(long jobCodeId)
        {
            try
            {
                DataObject.DataAccessor.EnableJobCode(jobCodeId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
