//-----------------------------------------------------------------------
// <copyright file="OrganisationDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Organisation Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows Organisation Data to be manipultaed.
    /// </summary>
    public class OrganisationDataObject
    {
        #region Private Members

        /// <summary>Holds static ActivityDataObject objects.</summary>
        private static volatile OrganisationDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the OrganisationDataObject class from being created.
        /// </summary>
        private OrganisationDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an OrganisationDataObject object. 
        /// </summary>
        public static OrganisationDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new OrganisationDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Organisation.
        /// </summary>
        /// <param name="name">Name of Organisation.</param>
        /// <returns>Id of new Organisation.</returns>
        public int AddOrganisation(string name)
        {
            try
            {
                return DataObject.DataAccessor.AddOrganisation(name);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Organisation.
        /// </summary>
        /// <param name="organisation">Organisation to update.</param>
        public void UpdateOrganisation(Organisation organisation)
        {
            try
            {
                DataObject.DataAccessor.UpdateOrganisation(organisation);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to delete.</param>
        public void DeleteOrganisation(int organisationId)
        {
            try
            {
                DataObject.DataAccessor.DeleteOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Organisations that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public OrganisationCollection GetOrganisations(OrganisationFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetOrganisations(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets an Organisations that matches the id.
        /// </summary>
        /// <param name="organisationId">Id to use when retrieving data.</param>
        /// <returns>Elements that match the id.</returns>
        public Organisation GetOrganisation(int organisationId)
        {
            try
            {
                return DataObject.DataAccessor.GetOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disbales existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to disbale.</param>
        public void DisableOrganisation(int organisationId)
        {
            try
            {
                DataObject.DataAccessor.DisableOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to enable.</param>
        public void EnableOrganisation(int organisationId)
        {
            try
            {
                DataObject.DataAccessor.EnableOrganisation(organisationId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
