//-----------------------------------------------------------------------
// <copyright file="RoleDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Role Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows Role Data to be manipultaed.
    /// </summary>
    public class RoleDataObject
    {
        #region Private Members

        /// <summary>Holds static RoleDataObject objects.</summary>
        private static volatile RoleDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the RoleDataObject class from being created.
        /// </summary>
        private RoleDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an RoleDataObject object. 
        /// </summary>
        public static RoleDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new RoleDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Role.
        /// </summary>
        /// <param name="roleId">id of role.</param>
        /// <param name="name">Name of Role.</param>
        /// <returns>Id of new Role.</returns>
        public short AddRole(short roleId, string name)
        {
            try
            {
                return DataObject.DataAccessor.AddRole(roleId, name);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Role.
        /// </summary>
        /// <param name="role">Role to update.</param>
        public void UpdateRole(Role role)
        {
            try
            {
                DataObject.DataAccessor.UpdateRole(role);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Role.
        /// </summary>
        /// <param name="roleId">Id of Role to delete.</param>
        public void DeleteRole(short roleId)
        {
            try
            {
                DataObject.DataAccessor.DeleteRole(roleId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a Role that matches the id.
        /// </summary>
        /// <param name="roleId">Id of role.</param>
        /// <returns>Matching role.</returns>
        public Role GetRole(short roleId)
        {
            try
            {
                return DataObject.DataAccessor.GetRole(roleId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Roles that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public RoleCollection GetRoles(RoleFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetRoles(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
