//-----------------------------------------------------------------------
// <copyright file="SqlServerDataAccessor.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represensts methods used to access and modify data.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using System.Collections.Generic;
    using System.Data.SqlClient;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that represensts methods used to access and modify data from an SQL server data source.
    /// </summary>
    internal class SqlServerDataAccessor : IDataAccessor
    {
        #region Constructor

        /// <summary>
        /// Initializes a new instance of the SqlServerDataAccessor class.
        /// </summary>
        public SqlServerDataAccessor()
        {
        }

        #endregion Constructor

        #region Destructor

        /// <summary>
        /// Finalizes an instance of the SqlServerDataAccessor class.
        /// </summary>
        ~SqlServerDataAccessor()
        {
        }

        #endregion Destructor

        #region Properties

        /// <summary>Gets the maximum number of records that are returned from a result.</summary>
        public int MaxRecords
        {
            get { return 1000; }
        }

        #endregion Properties

        #region Common Methods

        /// <summary>
        /// Handles any exception that is thrown during data access and modification.
        /// </summary>
        /// <param name="ex">Exception to handle.</param>
        public void HandleException(Exception ex)
        {
            try
            {
                // Awaiting implementation
                throw ex;
            }
            catch
            {
                // Leave Blank
            }
        }

        #endregion Common Methods

        #region Activity Methods

        /// <summary>
        /// Adds new activity.
        /// </summary>
        /// <param name="jobCodeId">Job Code that the activity belongs to.</param>
        /// <param name="code">Code of activity.</param>
        /// <param name="name">Name of activity.</param>
        /// <returns>Id of new activity.</returns>
        public long AddActivity(long jobCodeId, string code, string name)
        {
            DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter();
            return Convert.ToInt64(ta.InsertQuery(jobCodeId, code, name));
        }

        /// <summary>
        /// Updates existing activity.
        /// </summary>
        /// <param name="activity">Activity to update.</param>
        public void UpdateActivity(Activity activity)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter();
            ta.FillByActivityId(timesheetDataSet.Activity, activity.ActivityId);
            timesheetDataSet.Activity[0].Name = activity.Name;
            ta.Update(timesheetDataSet.Activity);
        }

        /// <summary>
        /// Deletes existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to delete.</param>
        public void DeleteActivity(long activityId)
        {
            DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter();
            ta.Delete(activityId);
        }

        /// <summary>
        /// Gets a collection of activities that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public ActivityCollection GetActivities(ActivityFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter();

            if (filter != null)
            {
                ta.FillByParameters(timesheetDataSet.Activity, filter.JobCodeId, filter.Enabled);
            }
            else
            {
                ta.Fill(timesheetDataSet.Activity);
            }

            ActivityCollection activityCollection = new ActivityCollection();
            foreach (DataSets.TimesheetDataSet.ActivityRow row in timesheetDataSet.Activity.Rows)
            {
                activityCollection.Add(this.ConvertDataRowToActivity(row));
            }

            return activityCollection;
        }

        /// <summary>
        /// Gets an activity that matches the id.
        /// </summary>
        /// <param name="activityId">Id to use when retrieving data.</param>
        /// <returns>Matching activity.</returns>
        public Activity GetActivity(long activityId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter();
            ta.FillByActivityId(timesheetDataSet.Activity, activityId);
            return this.ConvertDataRowToActivity(timesheetDataSet.Activity.Rows[0] as DataSets.TimesheetDataSet.ActivityRow);
        }

        /// <summary>
        /// Disables existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to disable.</param>
        public void DisableActivity(long activityId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter();
            ta.FillByActivityId(timesheetDataSet.Activity, activityId);
            timesheetDataSet.Activity[0].DisabledDateTime = DateTime.Now;
            ta.Update(timesheetDataSet.Activity);
        }

        /// <summary>
        /// Enables existing activity.
        /// </summary>
        /// <param name="activityId">Id of Activity to enable.</param>
        public void EnableActivity(long activityId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.ActivityTableAdapter();
            ta.FillByActivityId(timesheetDataSet.Activity, activityId);
            timesheetDataSet.Activity[0][timesheetDataSet.Activity.DisabledDateTimeColumn] = DBNull.Value;
            ta.Update(timesheetDataSet.Activity);
        }

        #endregion Activity Methods

        #region Timesheet Methods

        /// <summary>
        /// Adds new Timesheet.
        /// </summary>
        /// <param name="userId">id of user the timesheet is for.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="weekEndDate">The end date of the week.</param>
        /// <returns>Id of new Timesheet.</returns>
        public long AddTimesheet(long userId, short timesheetStatusId, DateTime weekEndDate)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter();
            return Convert.ToInt64(ta.InsertQuery(userId, timesheetStatusId, weekEndDate));
        }

        /// <summary>
        /// Adds new Timesheet.
        /// </summary>
        /// <param name="userId">id of user the timesheet is for.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="timesheet">Timesheet to save.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to save.</param>
        /// <param name="timesheetActivityEntries">Related activities to save.</param>
        /// <returns>Id of new Timesheet.</returns>
        public long AddTimesheet(
            long userId,
            short timesheetStatusId,
            Timesheet timesheet,
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            long timesheetId = this.AddTimesheet(userId, timesheetStatusId, timesheet.WeekEndDate);

            foreach (TimesheetAttendanceEntry entry in timesheetAttendanceEntries)
            {
                this.AddTimesheetAttendanceEntry(timesheetId, entry.StartDateTime, entry.EndDateTime);
            }

            foreach (TimesheetActivityEntry entry in timesheetActivityEntries)
            {
                this.AddTimesheetActivityEntry(timesheetId, entry.ActivityId, entry.Hours, entry.Description, entry.Date);
            }

            return timesheetId;
        }

        /// <summary>
        /// Updates existing Timesheet.
        /// </summary>
        /// <param name="userId">id of the udpater user.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="timesheet">Timesheet to save.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to save.</param>
        /// <param name="timesheetActivityEntries">Related activities to save.</param>
        /// <returns>Id of new Timesheet.</returns>
        public long UpdateTimesheet(
            long userId,
            short timesheetStatusId,
            Timesheet timesheet,
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            this.ChangeTimesheetStatus(timesheet.TimesheetId, userId, timesheetStatusId, null);

            // Delete Attendances and activities
            this.DeleteTimesheetAttendanceEntries(timesheet.TimesheetId);
            this.DeleteTimesheetActivityEntries(timesheet.TimesheetId);

            foreach (TimesheetAttendanceEntry entry in timesheetAttendanceEntries)
            {
                this.AddTimesheetAttendanceEntry(timesheet.TimesheetId, entry.StartDateTime, entry.EndDateTime);
            }

            foreach (TimesheetActivityEntry entry in timesheetActivityEntries)
            {
                this.AddTimesheetActivityEntry(timesheet.TimesheetId, entry.ActivityId, entry.Hours, entry.Description, entry.Date);
            }

            return timesheet.TimesheetId;
        }

        /// <summary>
        /// Deletes existing Timesheet.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to delete.</param>
        public void DeleteTimesheet(long timesheetId)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter();
            ta.Delete(timesheetId);
        }

        /// <summary>
        /// Gets a collection of Timesheets that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetCollection GetTimesheets(TimesheetFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter();

            if (filter != null)
            {
                ta.FillByParameters(
                    timesheetDataSet.Timesheet,
                    filter.MaxRecords.HasValue ? filter.MaxRecords.Value : this.GetTimesheetsCount(filter),
                    filter.UserId,
                    filter.ApproverUserId,
                    filter.OrganisationId,
                    filter.TimesheetStatusId,
                    filter.StatusChangerUserId,
                    filter.StartDateTime,
                    filter.EndDateTime);
            }
            else
            {
                ta.Fill(timesheetDataSet.Timesheet);
            }

            TimesheetCollection timesheetCollection = new TimesheetCollection();
            foreach (DataSets.TimesheetDataSet.TimesheetRow row in timesheetDataSet.Timesheet.Rows)
            {
                timesheetCollection.Add(this.ConvertDataRowToTimesheet(row));
            }

            return timesheetCollection;
        }

        /// <summary>
        /// Gets the number of Timesheets that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>The number of Timesheets.</returns>
        public long GetTimesheetsCount(TimesheetFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter();
           
            object result = null;
            if (filter != null)
            {
                result = ta.GetTimesheetsCount(
                     filter.UserId,
                     filter.ApproverUserId,
                     filter.OrganisationId,
                     filter.TimesheetStatusId,
                     filter.StatusChangerUserId,
                     filter.StartDateTime,
                     filter.EndDateTime);
            }
            else
            {
                result = ta.GetTimesheetsCount(null, null, null, null, null, null, null);
            }

            return Convert.ToInt64(result);
        }

        /// <summary>
        /// Gets a Timesheets that matches the id.
        /// </summary>
        /// <param name="timesheetId">Id of the timesheet.</param>
        /// <returns>Matching timesheet.</returns>
        public Timesheet GetTimesheet(long timesheetId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter();
            ta.FillByTimesheetId(timesheetDataSet.Timesheet, timesheetId);
            return this.ConvertDataRowToTimesheet(timesheetDataSet.Timesheet[0]);
        }

        /// <summary>
        /// Gets a timesheet that matches the information provided.
        /// </summary>
        /// <param name="userId">Id of timesheet.</param>
        /// <param name="weekEndDate">The date of the weekend.</param>
        /// <returns>Matching timesheet.</returns>
        public Timesheet GetTimesheet(long userId, DateTime weekEndDate)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter();
            ta.FillByUserIdAndWeekEndDate(timesheetDataSet.Timesheet, userId, weekEndDate);
            return this.ConvertDataRowToTimesheet(timesheetDataSet.Timesheet[0]);
        }

        /// <summary>
        /// Changes existing Timesheet's status.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to change.</param>
        /// <param name="statusChangerUserId">Id of user changing status.</param>
        /// <param name="timesheetStatusId">Id of timesheet's new status.</param>
        /// <param name="statusChangeDescription">Description related to change.</param>
        public void ChangeTimesheetStatus(long timesheetId, long statusChangerUserId, short timesheetStatusId, string statusChangeDescription)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetTableAdapter();
            ta.FillByTimesheetId(timesheetDataSet.Timesheet, timesheetId);
            timesheetDataSet.Timesheet[0].StatusChangerUserId = statusChangerUserId;
            timesheetDataSet.Timesheet[0].TimesheetStatusId = timesheetStatusId;
            timesheetDataSet.Timesheet[0].StatusChangeDateTime = DateTime.Now;
            timesheetDataSet.Timesheet[0].StatusChangeDescription = statusChangeDescription;
            ta.Update(timesheetDataSet.Timesheet);
        }

        #endregion Timesheet Methods

        #region Timesheet Activity Entry Methods

        /// <summary>
        /// Adds new Timesheet Activity entry.
        /// </summary>
        /// <param name="timesheetId">id of timesheet the entry is for.</param>
        /// <param name="activityId">Id of activity is for.</param>
        /// <param name="hours">Number od hours for entry.</param>
        /// <param name="description">Description of entry.</param>
        /// <param name="date">Date for entry.</param>
        /// <returns>Id of new entry.</returns>
        public long AddTimesheetActivityEntry(long timesheetId, long activityId, decimal hours, string description, DateTime date)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter();
            return Convert.ToInt64(ta.InsertQuery(timesheetId, activityId, hours, description, date));
        }

        /// <summary>
        /// Updates existing TimesheetActivityEntry.
        /// </summary>
        /// <param name="timesheetActivityEntry">Timesheet Activity Entry to update.</param>
        public void UpdateTimesheetActivityEntry(TimesheetActivityEntry timesheetActivityEntry)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter();
            ta.FillByTimesheetActivityEntryId(timesheetDataSet.TimesheetActivityEntry, timesheetActivityEntry.TimesheetActivityEntryId);
            timesheetDataSet.TimesheetActivityEntry[0].ActivityId = timesheetActivityEntry.ActivityId;
            timesheetDataSet.TimesheetActivityEntry[0].Date = timesheetActivityEntry.Date;
            timesheetDataSet.TimesheetActivityEntry[0].Hours = timesheetActivityEntry.Hours;
            ta.Update(timesheetDataSet.TimesheetActivityEntry);
        }

        /// <summary>
        /// Deletes existing Timesheet Activity Entry.
        /// </summary>
        /// <param name="timesheetActivityEntryId">Id of Timesheet Activity Entry to delete.</param>
        public void DeleteTimesheetActivityEntry(long timesheetActivityEntryId)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter();
            ta.Delete(timesheetActivityEntryId);
        }

        /// <summary>
        /// Deletes existing Timesheet Activity Entries.
        /// </summary>
        /// <param name="timesheetId">Id of timesheet to delete entries for.</param>
        public void DeleteTimesheetActivityEntries(long timesheetId)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter();
            ta.DeleteEntries(timesheetId);
        }

        /// <summary>
        /// Gets a collection of Timesheet Activity Entries that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetActivityEntryCollection GetTimesheetActivityEntries(TimesheetActivityEntryFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetActivityEntryTableAdapter();

            if (filter != null)
            {
                ta.FillByParameters(timesheetDataSet.TimesheetActivityEntry, filter.TimesheetId, filter.ActivityId, filter.StartDateTime, filter.EndDateTime);
            }
            else
            {
                ta.Fill(timesheetDataSet.TimesheetActivityEntry);
            }

            TimesheetActivityEntryCollection timesheetActivityEntryCollection = new TimesheetActivityEntryCollection();
            foreach (DataSets.TimesheetDataSet.TimesheetActivityEntryRow row in timesheetDataSet.TimesheetActivityEntry.Rows)
            {
                timesheetActivityEntryCollection.Add(this.ConvertDataRowToTimesheetActivityEntry(row));
            }

            return timesheetActivityEntryCollection;
        }

        #endregion Timesheet Activity Entry Methods

        #region Timesheet Attendance Entry Methods

        /// <summary>
        /// Adds new Timesheet Attendance entry.
        /// </summary>
        /// <param name="timesheetId">id of timesheet the entry is for.</param>
        /// <param name="startDateTime">Start Date time of attendance.</param>
        /// <param name="endDateTime">End Date time of attendance.</param>
        /// <returns>Id of new entry.</returns>
        public long AddTimesheetAttendanceEntry(long timesheetId, DateTime startDateTime, DateTime endDateTime)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter();
            return Convert.ToInt64(ta.InsertQuery(timesheetId, startDateTime, endDateTime));
        }

        /// <summary>
        /// Updates existing TimesheetAttendanceEntry.
        /// </summary>
        /// <param name="timesheetAttendanceEntry">Timesheet Attendance Entry to update.</param>
        public void UpdateTimesheetAttendanceEntry(TimesheetAttendanceEntry timesheetAttendanceEntry)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter();
            ta.FillByTimesheetAttendanceEntryId(timesheetDataSet.TimesheetAttendanceEntry, timesheetAttendanceEntry.TimesheetAttendanceEntryId);
            timesheetDataSet.TimesheetAttendanceEntry[0].StartDateTime = timesheetAttendanceEntry.StartDateTime;
            timesheetDataSet.TimesheetAttendanceEntry[0].EndDateTime = timesheetAttendanceEntry.EndDateTime;
            ta.Update(timesheetDataSet.TimesheetAttendanceEntry);
        }

        /// <summary>
        /// Deletes existing Timesheet Attendance Entry.
        /// </summary>
        /// <param name="timesheetAttendanceEntryId">Id of Timesheet Attendance Entry to delete.</param>
        public void DeleteTimesheetAttendanceEntry(long timesheetAttendanceEntryId)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter();
            ta.Delete(timesheetAttendanceEntryId);
        }

        /// <summary>
        /// Deletes existing Timesheet Attendance Entries.
        /// </summary>
        /// <param name="timesheetId">Id of timesheet to delete entries for.</param>
        public void DeleteTimesheetAttendanceEntries(long timesheetId)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter();
            ta.DeleteEntries(timesheetId);
        }

        /// <summary>
        /// Gets a collection of Timesheet Attendance Entries that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetAttendanceEntryCollection GetTimesheetAttendanceEntries(TimesheetAttendanceEntryFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetAttendanceEntryTableAdapter();

            if (filter != null)
            {
                ta.FillByParameters(timesheetDataSet.TimesheetAttendanceEntry, filter.TimesheetId, filter.StartDateTime, filter.EndDateTime);
            }
            else
            {
                ta.Fill(timesheetDataSet.TimesheetAttendanceEntry);
            }

            TimesheetAttendanceEntryCollection timesheetAttendanceEntryCollection = new TimesheetAttendanceEntryCollection();
            foreach (DataSets.TimesheetDataSet.TimesheetAttendanceEntryRow row in timesheetDataSet.TimesheetAttendanceEntry.Rows)
            {
                timesheetAttendanceEntryCollection.Add(this.ConvertDataRowToTimesheetAttendanceEntry(row));
            }

            return timesheetAttendanceEntryCollection;
        }

        #endregion Timesheet Attendance Entry Methods

        #region Timesheet Status Methods

        /// <summary>
        /// Adds new Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of timesheet status.</param>
        /// <param name="name">Name of Timesheet Status.</param>
        /// <returns>Id of new Timesheet Status.</returns>
        public short AddTimesheetStatus(short timesheetStatusId, string name)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter();
            ta.Insert(timesheetStatusId, name);
            return timesheetStatusId;
        }

        /// <summary>
        /// Updates existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatus">Timesheet Status to update.</param>
        public void UpdateTimesheetStatus(TimesheetStatus timesheetStatus)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter();
            ta.FillByTimesheetStatusId(timesheetDataSet.TimesheetStatus, timesheetStatus.TimesheetStatusId);
            timesheetDataSet.TimesheetStatus[0].Name = timesheetStatus.Name;
            ta.Update(timesheetDataSet.TimesheetStatus);
        }

        /// <summary>
        /// Deletes existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of Timesheet Status to delete.</param>
        public void DeleteTimesheetStatus(short timesheetStatusId)
        {
            DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter();
            ta.Delete(timesheetStatusId);
        }

        /// <summary>
        /// Gets a collection of Timesheet Statuses that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetStatusCollection GetTimesheetStatuses(TimesheetStatusFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter();
            ta.Fill(timesheetDataSet.TimesheetStatus);

            TimesheetStatusCollection timesheetStatusCollection = new TimesheetStatusCollection();
            foreach (DataSets.TimesheetDataSet.TimesheetStatusRow row in timesheetDataSet.TimesheetStatus.Rows)
            {
                timesheetStatusCollection.Add(this.ConvertDataRowToTimesheetStatus(row));
            }

            return timesheetStatusCollection;
        }

        /// <summary>
        /// Gets a Timesheet Status that matches the id.
        /// </summary>
        /// <param name="timesheetStatusId">Id to use when retrieving data.</param>
        /// <returns>Matching Timesheet Status.</returns>
        public TimesheetStatus GetTimesheetStatus(short timesheetStatusId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.TimesheetStatusTableAdapter();
            ta.FillByTimesheetStatusId(timesheetDataSet.TimesheetStatus, timesheetStatusId);
            return this.ConvertDataRowToTimesheetStatus(timesheetDataSet.TimesheetStatus.Rows[0] as DataSets.TimesheetDataSet.TimesheetStatusRow);
        }

        #endregion Timesheet Status Methods

        #region Gender Methods

        /// <summary>
        /// Adds new Gender.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <param name="name">Name of Gender.</param>
        /// <returns>Id of new Gender.</returns>
        public byte AddGender(byte genderId, string name)
        {
            DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter();
            ta.Insert(genderId, name);
            return genderId;
        }

        /// <summary>
        /// Updates existing Gender.
        /// </summary>
        /// <param name="gender">Gender to update.</param>
        public void UpdateGender(Gender gender)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter();
            ta.FillByGenderId(timesheetDataSet.Gender, gender.GenderId);
            timesheetDataSet.Gender[0].Name = gender.Name;
            ta.Update(timesheetDataSet.Gender);
        }

        /// <summary>
        /// Deletes existing Gender.
        /// </summary>
        /// <param name="genderId">Id of Gender to delete.</param>
        public void DeleteGender(byte genderId)
        {
            DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter();
            ta.Delete(genderId);
        }

        /// <summary>
        /// Gets a Gender that matches the id.
        /// </summary>
        /// <param name="genderId">Id of gender.</param>
        /// <returns>gender matching the id.</returns>
        public Gender GetGender(byte genderId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter();
            ta.FillByGenderId(timesheetDataSet.Gender, genderId);
            return this.ConvertDataRowToGender(timesheetDataSet.Gender.Rows[0] as DataSets.TimesheetDataSet.GenderRow);
        }

        /// <summary>
        /// Gets a collection of Genders that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public GenderCollection GetGenders(GenderFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.GenderTableAdapter();
            ta.Fill(timesheetDataSet.Gender);

            GenderCollection genderCollection = new GenderCollection();
            foreach (DataSets.TimesheetDataSet.GenderRow row in timesheetDataSet.Gender.Rows)
            {
                genderCollection.Add(this.ConvertDataRowToGender(row));
            }

            return genderCollection;
        }

        #endregion Gender Methods

        #region Job Code Methods

        /// <summary>
        /// Adds new Job Code.
        /// </summary>
        /// <param name="code">The job code.</param>
        /// <param name="name">Name of Job Code.</param>
        /// <param name="organisationId">Organisation that the job code belongs to.</param>
        /// <returns>Id of new JobCode.</returns>
        public long AddJobCode(string code, string name, int organisationId)
        {
            DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter();
            return Convert.ToInt64(ta.InsertQuery(organisationId, code, name));
        }

        /// <summary>
        /// Updates existing Job Code.
        /// </summary>
        /// <param name="jobCode">Job Code to update.</param>
        public void UpdateJobCode(JobCode jobCode)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter();
            ta.FillByJobCodeId(timesheetDataSet.JobCode, jobCode.JobCodeId);
            timesheetDataSet.JobCode[0].Name = jobCode.Name;
            ta.Update(timesheetDataSet.JobCode);
        }

        /// <summary>
        /// Deletes existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to delete.</param>
        public void DeleteJobCode(long jobCodeId)
        {
            DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter();
            ta.Delete(jobCodeId);
        }

        /// <summary>
        /// Gets a collection of Job Codes that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public JobCodeCollection GetJobCodes(JobCodeFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter();

            if (filter != null)
            {
                ta.FillByParameters(timesheetDataSet.JobCode, filter.OrganisationId, filter.Enabled);
            }
            else
            {
                ta.Fill(timesheetDataSet.JobCode);
            }

            JobCodeCollection jobCodeCollection = new JobCodeCollection();
            foreach (DataSets.TimesheetDataSet.JobCodeRow row in timesheetDataSet.JobCode.Rows)
            {
                jobCodeCollection.Add(this.ConvertDataRowToJobCode(row));
            }

            return jobCodeCollection;
        }

        /// <summary>
        /// Disables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to disable.</param>
        public void DisableJobCode(long jobCodeId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter();
            ta.FillByJobCodeId(timesheetDataSet.JobCode, jobCodeId);
            timesheetDataSet.JobCode[0].DisabledDateTime = DateTime.Now;
            ta.Update(timesheetDataSet.JobCode);
        }

        /// <summary>
        /// Enables existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to enable.</param>
        public void EnableJobCode(long jobCodeId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter();
            ta.FillByJobCodeId(timesheetDataSet.JobCode, jobCodeId);
            timesheetDataSet.JobCode[0][timesheetDataSet.JobCode.DisabledDateTimeColumn] = DBNull.Value;
            ta.Update(timesheetDataSet.JobCode);
        }

        /// <summary>
        /// Gets existing Job Code.
        /// </summary>
        /// <param name="jobCodeId">Id of Job Code to get.</param>
        /// <returns>matching Job Code.</returns>
        public JobCode GetJobCode(long jobCodeId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.JobCodeTableAdapter();
            ta.FillByJobCodeId(timesheetDataSet.JobCode, jobCodeId);
            return this.ConvertDataRowToJobCode(timesheetDataSet.JobCode.Rows[0] as DataSets.TimesheetDataSet.JobCodeRow);
        }

        #endregion Job Code Methods

        #region Organisation Methods

        /// <summary>
        /// Adds new Organisation.
        /// </summary>
        /// <param name="name">Name of Organisation.</param>
        /// <returns>Id of new Organisation.</returns>
        public int AddOrganisation(string name)
        {
            DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter();
            return Convert.ToInt32(ta.InsertQuery(name));
        }

        /// <summary>
        /// Updates existing Organisation.
        /// </summary>
        /// <param name="organisation">Organisation to update.</param>
        public void UpdateOrganisation(Organisation organisation)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter();
            ta.FillByOrganisationId(timesheetDataSet.Organisation, organisation.OrganisationId);
            timesheetDataSet.Organisation[0].Name = organisation.Name;
            ta.Update(timesheetDataSet.Organisation);
        }

        /// <summary>
        /// Deletes existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to delete.</param>
        public void DeleteOrganisation(int organisationId)
        {
            DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter();
            ta.Delete(organisationId);
        }

        /// <summary>
        /// Gets a collection of Organisations that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public OrganisationCollection GetOrganisations(OrganisationFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter();
            ta.Fill(timesheetDataSet.Organisation);

            OrganisationCollection organisationCollection = new OrganisationCollection();
            foreach (DataSets.TimesheetDataSet.OrganisationRow row in timesheetDataSet.Organisation.Rows)
            {
                organisationCollection.Add(this.ConvertDataRowToOrganisation(row));
            }

            return organisationCollection;
        }

        /// <summary>
        /// Gets an Organisations that matches the id.
        /// </summary>
        /// <param name="organisationId">Id to use when retrieving data.</param>
        /// <returns>Elements that match the id.</returns>
        public Organisation GetOrganisation(int organisationId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter();
            ta.FillByOrganisationId(timesheetDataSet.Organisation, organisationId);
            return this.ConvertDataRowToOrganisation(timesheetDataSet.Organisation.Rows[0] as DataSets.TimesheetDataSet.OrganisationRow);
        }

        /// <summary>
        /// Disbales existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to disbale.</param>
        public void DisableOrganisation(int organisationId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter();
            ta.FillByOrganisationId(timesheetDataSet.Organisation, organisationId);
            timesheetDataSet.Organisation[0].DisabledDateTime = DateTime.Now;
            ta.Update(timesheetDataSet.Organisation);
        }

        /// <summary>
        /// Enables existing Organisation.
        /// </summary>
        /// <param name="organisationId">Id of Organisation to enable.</param>
        public void EnableOrganisation(int organisationId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.OrganisationTableAdapter();
            ta.FillByOrganisationId(timesheetDataSet.Organisation, organisationId);
            timesheetDataSet.Organisation[0][timesheetDataSet.Organisation.DisabledDateTimeColumn] = DBNull.Value;
            ta.Update(timesheetDataSet.Organisation);
        }

        #endregion Organisation Methods

        #region Role Methods

        /// <summary>
        /// Adds new Role.
        /// </summary>
        /// <param name="roleId">id of role.</param>
        /// <param name="name">Name of Role.</param>
        /// <returns>Id of new Role.</returns>
        public short AddRole(short roleId, string name)
        {
            DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter();
            ta.Insert(roleId, name);
            return roleId;
        }

        /// <summary>
        /// Updates existing Role.
        /// </summary>
        /// <param name="role">Role to update.</param>
        public void UpdateRole(Role role)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter();
            ta.FillByRoleId(timesheetDataSet.Role, role.RoleId);
            timesheetDataSet.Role[0].Name = role.Name;
            ta.Update(timesheetDataSet.Role);
        }

        /// <summary>
        /// Deletes existing Role.
        /// </summary>
        /// <param name="roleId">Id of Role to delete.</param>
        public void DeleteRole(short roleId)
        {
            DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter();
            ta.Delete(roleId);
        }

        /// <summary>
        /// Gets a Role that matches the id.
        /// </summary>
        /// <param name="roleId">Id of role.</param>
        /// <returns>Matching role.</returns>
        public Role GetRole(short roleId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter();
            ta.FillByRoleId(timesheetDataSet.Role, roleId);
            return this.ConvertDataRowToRole(timesheetDataSet.Role[0]);
        }

        /// <summary>
        /// Gets a collection of Roles that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public RoleCollection GetRoles(RoleFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.RoleTableAdapter();
            ta.Fill(timesheetDataSet.Role);

            RoleCollection roleCollection = new RoleCollection();
            foreach (DataSets.TimesheetDataSet.RoleRow row in timesheetDataSet.Role.Rows)
            {
                roleCollection.Add(this.ConvertDataRowToRole(row));
            }

            return roleCollection;
        }

        #endregion Role Methods

        #region User Methods

        /// <summary>
        /// Adds new User.
        /// </summary>
        /// <param name="employeeId">Employee id of user.</param>
        /// <param name="genderId">Gender id of the user.</param>
        /// <param name="roleId">Role of user.</param>
        /// <param name="firstname">Firstname of user.</param>
        /// <param name="surname">Surname of user.</param>
        /// <param name="email">Email of user.</param>
        /// <param name="password">password of user.</param>
        /// <param name="organisationId">Id of organisation the user belongs to.</param>
        /// <param name="approverUserId">The user who will approve this users timesheet entries.</param>
        /// <returns>Id of user.</returns>
        public long AddUser(string employeeId, byte genderId, short roleId, string firstname, string surname, string email, string password, int? organisationId, long? approverUserId)
        {
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            return Convert.ToInt64(ta.InsertQuery(employeeId, email, genderId, roleId, firstname, surname, password, organisationId, approverUserId)); 
        }

        /// <summary>
        /// Updates existing user.
        /// </summary>
        /// <param name="user">User to update.</param>
        public void UpdateUser(User user)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, user.UserId);
            timesheetDataSet.User[0].EmployeeId = user.EmployeeId;
            if (user.ApproverUserId.HasValue)
            {
                timesheetDataSet.User[0].ApproverUserId = user.ApproverUserId.Value;
            }
            else
            {
                timesheetDataSet.User[0].SetApproverUserIdNull();
            }

            timesheetDataSet.User[0].Email = user.Email;
            timesheetDataSet.User[0].Firstname = user.Firstname;
            timesheetDataSet.User[0].GenderId = user.GenderId;

            if (user.OrganisationId.HasValue)
            {
                timesheetDataSet.User[0].OrganisationId = user.OrganisationId.Value;
            }
            else
            {
                timesheetDataSet.User[0].SetOrganisationIdNull();
            }

            timesheetDataSet.User[0].RoleId = user.RoleId;
            timesheetDataSet.User[0].Surname = user.Surname;
            ta.Update(timesheetDataSet.User);
        }

        /// <summary>
        /// Deletes existing User.
        /// </summary>
        /// <param name="userId">Id of User to delete.</param>
        public void DeleteUser(long userId)
        {
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.Delete(userId);
        }

        /// <summary>
        /// Gets the User that matches the id.
        /// </summary>
        /// <param name="userId">id of user.</param>
        /// <returns>User information.</returns>
        public User GetUser(long userId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);
            return this.ConvertDataRowToUser(timesheetDataSet.User.Rows[0] as DataSets.TimesheetDataSet.UserRow);
        }

        /// <summary>
        /// Gets the User that matches the email.
        /// </summary>
        /// <param name="email">email of user.</param>
        /// <returns>User information.</returns>
        public User GetUser(string email)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByEmail(timesheetDataSet.User, email);
            return this.ConvertDataRowToUser(timesheetDataSet.User.Rows[0] as DataSets.TimesheetDataSet.UserRow);
        }

        /// <summary>
        /// Gets a collection of Users that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public UserCollection GetUsers(UserFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();

            if (filter != null)
            {
                ta.FillByParameters(
                    timesheetDataSet.User,
                    filter.MaxRecords.HasValue ? filter.MaxRecords.Value : this.GetUsersCount(filter),
                    filter.GenderId,
                    filter.RoleId,
                    filter.OrganisationId,
                    filter.ApproverUserId,
                    filter.Activated,
                    filter.Enabled,
                    string.IsNullOrEmpty(filter.User) ? null : filter.User,
                    string.IsNullOrEmpty(filter.Approver) ? null : filter.Approver);
            }
            else
            {
                ta.Fill(timesheetDataSet.User);
            }

            UserCollection userCollection = new UserCollection();
            foreach (DataSets.TimesheetDataSet.UserRow row in timesheetDataSet.User.Rows)
            {
                userCollection.Add(this.ConvertDataRowToUser(row));
            }

            return userCollection;
        }

        /// <summary>
        /// Gets the number of Users that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>The number of users.</returns>
        public long GetUsersCount(UserFilter filter)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            object result = null;

            if (filter != null)
            {
                result = ta.GetUsersCount(
                     filter.GenderId,
                     filter.RoleId,
                     filter.OrganisationId,
                     filter.ApproverUserId,
                     filter.Activated,
                     filter.Enabled,
                     string.IsNullOrEmpty(filter.User) ? null : filter.User,
                     string.IsNullOrEmpty(filter.Approver) ? null : filter.Approver);
            }
            else
            {
                result = ta.GetUsersCount(
                     null,
                     null,
                     null,
                     null,
                     null,
                     null,
                     null,
                     null);
            }

            return Convert.ToInt64(result);
        }

        /// <summary>
        /// Disables existing user.
        /// </summary>
        /// <param name="userId">Id of User to disable.</param>
        public void DisableUser(long userId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);
            timesheetDataSet.User[0].DisabledDateTime = DateTime.Now;
            ta.Update(timesheetDataSet);
        }

        /// <summary>
        /// Enables existing user.
        /// </summary>
        /// <param name="userId">Id of User to enable.</param>
        public void EnableUser(long userId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);
            timesheetDataSet.User[0][timesheetDataSet.User.DisabledDateTimeColumn] = DBNull.Value;
            ta.Update(timesheetDataSet);
        }

        /// <summary>
        /// Activates existing user.
        /// </summary>
        /// <param name="userId">Id of User to activate.</param>
        public void ActivateUser(long userId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);
            timesheetDataSet.User[0].ActivationDateTime = DateTime.Now;
            ta.Update(timesheetDataSet);
        }

        /// <summary>
        /// Deactivates existing user.
        /// </summary>
        /// <param name="userId">Id of User to Deactivate.</param>
        public void DeactivateUser(long userId)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);
            timesheetDataSet.User[0][timesheetDataSet.User.ActivationDateTimeColumn] = DBNull.Value;
            ta.Update(timesheetDataSet);
        }

        /// <summary>
        /// Updates user's password.
        /// </summary>
        /// <param name="userId">Id of User to change password for.</param>
        /// <param name="password">The users new password.</param>
        public void SetUserPassword(long userId, string password)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);
            timesheetDataSet.User[0].Password = password;
            ta.Update(timesheetDataSet);
        }

        /// <summary>
        /// Gets the collection of users who are above the passed in user in terms of approver structure.
        /// </summary>
        /// <param name="userId">Id of user to find hierachy for.</param>
        /// <param name="inclusive">Whether or not to include the passed in user in the result.</param>
        /// <returns>Users that are above the passed in user in terms of heirachy</returns>
        public UserCollection GetUserHierachyAboveUser(long userId, bool inclusive)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);

            Dictionary<long, User> users = new Dictionary<long, User>();

            // Get user information and decide whether or not it needs to be added to list
            User user = this.ConvertDataRowToUser(timesheetDataSet.User.Rows[0] as DataSets.TimesheetDataSet.UserRow);
            if (inclusive)
            {
                users.Add(user.UserId, user);
            }

            // Loop until all approvers have been received.
            while (user.ApproverUserId.HasValue)
            {
                ta.FillByUserId(timesheetDataSet.User, user.ApproverUserId.Value);
                user = this.ConvertDataRowToUser(timesheetDataSet.User.Rows[0] as DataSets.TimesheetDataSet.UserRow);

                if (!users.ContainsKey(user.UserId))
                {
                    users.Add(user.UserId, user);
                }
                else
                {
                    break;
                }
            }

            UserCollection usersCollection = new UserCollection();
            usersCollection.AddRange(users.Values);
            return usersCollection;
        }

        /// <summary>
        /// Gets the collection of users who are below the passed in user in terms of approver structure.
        /// </summary>
        /// <param name="userId">Id of user to find hierachy for.</param>
        /// <param name="inclusive">Whether or not to include the passed in user in the result.</param>
        /// <returns>Users that are below the passed in user in terms of heirachy</returns>
        public UserCollection GetUserHierachyBelowUser(long userId, bool inclusive)
        {
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByUserId(timesheetDataSet.User, userId);

            UserCollection usersCollection = new UserCollection();

            // Get user information and decide whether or not it needs to be added to list
            User user = this.ConvertDataRowToUser(timesheetDataSet.User.Rows[0] as DataSets.TimesheetDataSet.UserRow);
            if (inclusive)
            {
                usersCollection.Add(user);
            }

            usersCollection.AddRange(this.GetUserHierachyBelowUser(user));
            return usersCollection;
        }

        #endregion User Methods

        #region Private Methods

        /// <summary>
        /// Converts a ActivityRow Dat Row to an Activity Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private Activity ConvertDataRowToActivity(DataSets.TimesheetDataSet.ActivityRow row)
        {
            Activity activity = new Activity(
                row.ActivityId, 
                row.JobCodeId, 
                row.Code, 
                row.CreationDateTime, 
                row.IsDisabledDateTimeNull() ? (DateTime?)null : row.DisabledDateTime,
                row.Name);
            return activity;
        }

        /// <summary>
        /// Converts a TimesheetRow Dat Row to an Timesheet Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private Timesheet ConvertDataRowToTimesheet(DataSets.TimesheetDataSet.TimesheetRow row)
        {
            Timesheet timesheet = new Timesheet(
                row.TimesheetId, 
                row.UserId, 
                row.TimesheetStatusId, 
                row.StatusChangerUserId, 
                row.WeekEndDate, 
                row.IsStatusChangeDescriptionNull() ? null : row.StatusChangeDescription, 
                row.CreationDateTime, 
                row.StatusChangeDateTime);
            return timesheet;
        }

        /// <summary>
        /// Converts a TimesheetActivityEntryRow Dat Row to an TimesheetActivityEntry Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private TimesheetActivityEntry ConvertDataRowToTimesheetActivityEntry(DataSets.TimesheetDataSet.TimesheetActivityEntryRow row)
        {
            TimesheetActivityEntry timesheetActivityEntry = new TimesheetActivityEntry(
                row.TimesheetActivityEntryId, 
                row.TimesheetId, 
                row.ActivityId, 
                row.Hours, 
                row.Description,
                row.Date, 
                row.CreationDateTime);
            return timesheetActivityEntry;
        }

        /// <summary>
        /// Converts a TimesheetAttendanceEntryRow Dat Row to an TimesheetAttendanceEntry Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private TimesheetAttendanceEntry ConvertDataRowToTimesheetAttendanceEntry(DataSets.TimesheetDataSet.TimesheetAttendanceEntryRow row)
        {
            TimesheetAttendanceEntry timesheetAttendanceEntry = new TimesheetAttendanceEntry(row.TimesheetAttendanceEntryId, row.TimesheetId, row.StartDateTime, row.EndDateTime, row.CreationDateTime);
            return timesheetAttendanceEntry;
        }

        /// <summary>
        /// Converts a TimesheetStatusRow Dat Row to an TimesheetStatus Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private TimesheetStatus ConvertDataRowToTimesheetStatus(DataSets.TimesheetDataSet.TimesheetStatusRow row)
        {
            TimesheetStatus timesheetStatus = new TimesheetStatus(row.TimesheetStatusId, row.Name);
            return timesheetStatus;
        }

        /// <summary>
        /// Converts a GenderRow Dat Row to an Gender Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private Gender ConvertDataRowToGender(DataSets.TimesheetDataSet.GenderRow row)
        {
            Gender gender = new Gender(row.GenderId, row.Name);
            return gender;
        }

        /// <summary>
        /// Converts a JobCodeRow Dat Row to an JobCode Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private JobCode ConvertDataRowToJobCode(DataSets.TimesheetDataSet.JobCodeRow row)
        {
            JobCode jobCode = new JobCode(
                row.JobCodeId, 
                row.OrganisationId, 
                row.Code, 
                row.Name, 
                row.CreationDateTime,
                row.IsDisabledDateTimeNull() ? (DateTime?)null : row.DisabledDateTime);
            return jobCode;
        }

        /// <summary>
        /// Converts a OrganisationRow Dat Row to an Organisation Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private Organisation ConvertDataRowToOrganisation(DataSets.TimesheetDataSet.OrganisationRow row)
        {
            Organisation organisation = new Organisation(
                row.OrganisationId, 
                row.Name, 
                row.CreationDateTime,
                row.IsDisabledDateTimeNull() ? (DateTime?)null : row.DisabledDateTime);
            return organisation;
        }

        /// <summary>
        /// Converts a RoleRow Dat Row to an Role Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private Role ConvertDataRowToRole(DataSets.TimesheetDataSet.RoleRow row)
        {
            Role role = new Role(row.RoleId, row.Name);
            return role;
        }

        /// <summary>
        /// Converts a UserRow Dat Row to an User Business Object.
        /// </summary>
        /// <param name="row">Data row to convert.</param>
        /// <returns>Object created from data row.</returns>
        private User ConvertDataRowToUser(DataSets.TimesheetDataSet.UserRow row)
        {
            User user = new User(
                row.UserId,
                row.EmployeeId,
                row.Email, 
                row.GenderId, 
                row.RoleId, 
                row.Firstname, 
                row.Surname, 
                row.Password,
                row.IsOrganisationIdNull() ? (int?)null : row.OrganisationId,
                row.IsApproverUserIdNull() ? (long?)null : row.ApproverUserId,
                row.IsActivationDateTimeNull() ? (DateTime?)null : row.ActivationDateTime,
                row.IsDisabledDateTimeNull() ? (DateTime?)null : row.DisabledDateTime, 
                row.CreationDateTime);
            return user;
        }

        /// <summary>
        /// Recursively looks for all users below the passed in user in terms of the tree structure.
        /// </summary>
        /// <param name="user">User to search children for.</param>
        /// <returns>Collection of users below the passed in user.</returns>
        private UserCollection GetUserHierachyBelowUser(User user)
        {
            long count = this.GetUsersCount(new UserFilter());
            return this.GetUserHierachyBelowUser(user, count);
        }

        /// <summary>
        /// Recursively looks for all users below the passed in user in terms of the tree structure.
        /// </summary>
        /// <param name="user">User to search children for.</param>
        /// <param name="totalUsers">the total users.</param>
        /// <returns>Collection of users below the passed in user.</returns>
        private UserCollection GetUserHierachyBelowUser(User user, long totalUsers)
        {
            UserCollection userCollection = new UserCollection();
            DataSets.TimesheetDataSet timesheetDataSet = new DataSets.TimesheetDataSet();
            DataSets.TimesheetDataSetTableAdapters.UserTableAdapter ta = new DataSets.TimesheetDataSetTableAdapters.UserTableAdapter();
            ta.FillByParameters(timesheetDataSet.User, totalUsers, null, null, null, user.UserId, null, null, null, null);

            foreach (DataSets.TimesheetDataSet.UserRow row in timesheetDataSet.User.Rows)
            {
                User childUser = this.ConvertDataRowToUser(row);
                userCollection.AddRange(this.GetUserHierachyBelowUser(childUser, totalUsers));
                userCollection.Add(childUser);
            }

            return userCollection;
        }

        #endregion Private Methods
    }
}
