//-----------------------------------------------------------------------
// <copyright file="TimesheetActivityEntryDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Timesheet Activity Entry Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows entry Data to be manipultaed.
    /// </summary>
    public class TimesheetActivityEntryDataObject
    {
        #region Private Members

        /// <summary>Holds static TimesheetEntryDataObject objects.</summary>
        private static volatile TimesheetActivityEntryDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the TimesheetActivityEntryDataObject class from being created.
        /// </summary>
        private TimesheetActivityEntryDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an TimesheetActivityEntryDataObject object. 
        /// </summary>
        public static TimesheetActivityEntryDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new TimesheetActivityEntryDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Timesheet Activity entry.
        /// </summary>
        /// <param name="timesheetId">id of timesheet the entry is for.</param>
        /// <param name="activityId">Id of activity is for.</param>
        /// <param name="hours">Number od hours for entry.</param>
        /// <param name="description">Description of entry.</param>
        /// <param name="date">Date for entry.</param>
        /// <returns>Id of new entry.</returns>
        public long AddTimesheetActivityEntry(long timesheetId, long activityId, decimal hours, string description, DateTime date)
        {
            try
            {
                return DataObject.DataAccessor.AddTimesheetActivityEntry(timesheetId, activityId, hours, description, date);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing TimesheetActivityEntry.
        /// </summary>
        /// <param name="timesheetActivityEntry">Timesheet Activity Entry to update.</param>
        public void UpdateTimesheetActivityEntry(TimesheetActivityEntry timesheetActivityEntry)
        {
            try
            {
                DataObject.DataAccessor.UpdateTimesheetActivityEntry(timesheetActivityEntry);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Timesheet Activity Entry.
        /// </summary>
        /// <param name="timesheetActivityEntryId">Id of Timesheet Activity Entry to delete.</param>
        public void DeleteTimesheetActivityEntry(long timesheetActivityEntryId)
        {
            try
            {
                DataObject.DataAccessor.DeleteTimesheetActivityEntry(timesheetActivityEntryId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Timesheet Activity Entries for a timesheet.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to delete.</param>
        public void DeleteTimesheetActivityEntries(long timesheetId)
        {
            try
            {
                DataObject.DataAccessor.DeleteTimesheetActivityEntries(timesheetId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Timesheet Activity Entries that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetActivityEntryCollection GetTimesheetActivityEntries(TimesheetActivityEntryFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetTimesheetActivityEntries(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
