//-----------------------------------------------------------------------
// <copyright file="TimesheetAttendanceEntryDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Timesheet Attendance Entry Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows Attendance Data to be manipultaed.
    /// </summary>
    public class TimesheetAttendanceEntryDataObject
    {
        #region Private Members

        /// <summary>Holds static TimesheetAttendanceEntryDataObject objects.</summary>
        private static volatile TimesheetAttendanceEntryDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the TimesheetAttendanceEntryDataObject class from being created.
        /// </summary>
        private TimesheetAttendanceEntryDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an TimesheetAttendanceEntryDataObject object. 
        /// </summary>
        public static TimesheetAttendanceEntryDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new TimesheetAttendanceEntryDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Timesheet Attendance entry.
        /// </summary>
        /// <param name="timesheetId">id of timesheet the entry is for.</param>
        /// <param name="startDateTime">Start Date time of attendance.</param>
        /// <param name="endDateTime">End Date time of attendance.</param>
        /// <returns>Id of new entry.</returns>
        public long AddTimesheetAttendanceEntry(long timesheetId, DateTime startDateTime, DateTime endDateTime)
        {
            try
            {
                return DataObject.DataAccessor.AddTimesheetAttendanceEntry(timesheetId, startDateTime, endDateTime);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing TimesheetAttendanceEntry.
        /// </summary>
        /// <param name="timesheetAttendanceEntry">Timesheet Attendance Entry to update.</param>
        public void UpdateTimesheetAttendanceEntry(TimesheetAttendanceEntry timesheetAttendanceEntry)
        {
            try
            {
                DataObject.DataAccessor.UpdateTimesheetAttendanceEntry(timesheetAttendanceEntry);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Timesheet Attendance Entry.
        /// </summary>
        /// <param name="timesheetAttendanceEntryId">Id of Timesheet Attendance Entry to delete.</param>
        public void DeleteTimesheetAttendanceEntry(long timesheetAttendanceEntryId)
        {
            try
            {
                DataObject.DataAccessor.DeleteTimesheetAttendanceEntry(timesheetAttendanceEntryId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Timesheet Attendance Entries for a timesheet.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to delete.</param>
        public void DeleteTimesheetAttendanceEntries(long timesheetId)
        {
            try
            {
                DataObject.DataAccessor.DeleteTimesheetAttendanceEntries(timesheetId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Timesheet Attendance Entries that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetAttendanceEntryCollection GetTimesheetAttendanceEntries(TimesheetAttendanceEntryFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetTimesheetAttendanceEntries(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
