//-----------------------------------------------------------------------
// <copyright file="TimesheetDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Timesheet Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows timesheet Data to be manipultaed.
    /// </summary>
    public class TimesheetDataObject
    {
        #region Private Members

        /// <summary>Holds static TimesheetDataObject objects.</summary>
        private static volatile TimesheetDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the TimesheetDataObject class from being created.
        /// </summary>
        private TimesheetDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an TimesheetDataObject object. 
        /// </summary>
        public static TimesheetDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new TimesheetDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Timesheet.
        /// </summary>
        /// <param name="userId">id of user the timesheet is for.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="weekEndDate">The end date of the week.</param>
        /// <returns>Id of new Timesheet.</returns>
        public long AddTimesheet(long userId, short timesheetStatusId, DateTime weekEndDate)
        {
            try
            {
                return DataObject.DataAccessor.AddTimesheet(userId, timesheetStatusId, weekEndDate);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Adds new Timesheet.
        /// </summary>
        /// <param name="userId">id of user the timesheet is for.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="timesheet">Timesheet to save.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to save.</param>
        /// <param name="timesheetActivityEntries">Related activities to save.</param>
        /// <returns>Id of new Timesheet.</returns>
        public long AddTimesheet(
            long userId, 
            short timesheetStatusId, 
            Timesheet timesheet, 
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            try
            {
                if (timesheet == null)
                {
                    throw new ArgumentNullException("timesheet");
                }

                if (timesheetAttendanceEntries == null)
                {
                    throw new ArgumentNullException("timesheetAttendanceEntries");
                }

                if (timesheetActivityEntries == null)
                {
                    throw new ArgumentNullException("timesheetActivityEntries");
                }

                return DataObject.DataAccessor.AddTimesheet(
                    userId, 
                    timesheetStatusId, 
                    timesheet, 
                    timesheetAttendanceEntries, 
                    timesheetActivityEntries);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Timesheet.
        /// </summary>
        /// <param name="userId">id of the udpater user.</param>
        /// <param name="timesheetStatusId">Id of timesheet Status.</param>
        /// <param name="timesheet">Timesheet to save.</param>
        /// <param name="timesheetAttendanceEntries">Related Attendances to save.</param>
        /// <param name="timesheetActivityEntries">Related activities to save.</param>
        /// <returns>Id of new Timesheet.</returns>
        public long UpdateTimesheet(
            long userId,
            short timesheetStatusId,
            Timesheet timesheet,
            TimesheetAttendanceEntryCollection timesheetAttendanceEntries,
            TimesheetActivityEntryCollection timesheetActivityEntries)
        {
            try
            {
                if (timesheet == null)
                {
                    throw new ArgumentNullException("timesheet");
                }

                if (timesheetAttendanceEntries == null)
                {
                    throw new ArgumentNullException("timesheetAttendanceEntries");
                }

                if (timesheetActivityEntries == null)
                {
                    throw new ArgumentNullException("timesheetActivityEntries");
                }

                return DataObject.DataAccessor.UpdateTimesheet(
                    userId,
                    timesheetStatusId,
                    timesheet,
                    timesheetAttendanceEntries,
                    timesheetActivityEntries);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Timesheet.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to delete.</param>
        public void DeleteTimesheet(long timesheetId)
        {
            try
            {
                DataObject.DataAccessor.DeleteTimesheet(timesheetId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Timesheets that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetCollection GetTimesheets(TimesheetFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetTimesheets(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a Timesheets that matches the id.
        /// </summary>
        /// <param name="timesheetId">Id of the timesheet.</param>
        /// <returns>Matching timesheet.</returns>
        public Timesheet GetTimesheet(long timesheetId)
        {
            try
            {
                return DataObject.DataAccessor.GetTimesheet(timesheetId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a timesheet that matches the information provided.
        /// </summary>
        /// <param name="userId">Id of timesheet.</param>
        /// <param name="weekEndDate">The date of the weekend.</param>
        /// <returns>Matching timesheet.</returns>
        public Timesheet GetTimesheet(long userId, DateTime weekEndDate)
        {
            try
            {
                return DataObject.DataAccessor.GetTimesheet(userId, weekEndDate);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Changes existing Timesheet's status.
        /// </summary>
        /// <param name="timesheetId">Id of Timesheet to change.</param>
        /// <param name="statusChangerUserId">Id of user changing status.</param>
        /// <param name="timesheetStatusId">Id of timesheet's new status.</param>
        /// <param name="statusChangeDescription">Description related to change.</param>
        public void ChangeTimesheetStatus(long timesheetId, long statusChangerUserId, short timesheetStatusId, string statusChangeDescription)
        {
            try
            {
                DataObject.DataAccessor.ChangeTimesheetStatus(timesheetId, statusChangerUserId, timesheetStatusId, statusChangeDescription);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
