//-----------------------------------------------------------------------
// <copyright file="TimesheetStatusDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows Timesheet Status Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows Entry Status Data to be manipultaed.
    /// </summary>
    public class TimesheetStatusDataObject
    {
        #region Private Members

        /// <summary>Holds static TimesheetStatusDataObject objects.</summary>
        private static volatile TimesheetStatusDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the TimesheetStatusDataObject class from being created.
        /// </summary>
        private TimesheetStatusDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an TimesheetStatusDataObject object. 
        /// </summary>
        public static TimesheetStatusDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new TimesheetStatusDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of timesheet status.</param>
        /// <param name="name">Name of Timesheet Status.</param>
        /// <returns>Id of new Timesheet Status.</returns>
        public short AddTimesheetStatus(short timesheetStatusId, string name)
        {
            try
            {
                return DataObject.DataAccessor.AddTimesheetStatus(timesheetStatusId, name);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatus">Timesheet Status to update.</param>
        public void UpdateTimesheetStatus(TimesheetStatus timesheetStatus)
        {
            try
            {
                DataObject.DataAccessor.UpdateTimesheetStatus(timesheetStatus);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing Timesheet Status.
        /// </summary>
        /// <param name="timesheetStatusId">Id of Timesheet Status to delete.</param>
        public void DeleteTimesheetStatus(short timesheetStatusId)
        {
            try
            {
                DataObject.DataAccessor.DeleteTimesheetStatus(timesheetStatusId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Timesheet Statuses that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public TimesheetStatusCollection GetTimesheetStatuses(TimesheetStatusFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetTimesheetStatuses(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a Timesheet Status that matches the id.
        /// </summary>
        /// <param name="timesheetStatusId">Id to use when retrieving data.</param>
        /// <returns>Matching Timesheet Status.</returns>
        public TimesheetStatus GetTimesheetStatus(short timesheetStatusId)
        {
            try
            {
                return DataObject.DataAccessor.GetTimesheetStatus(timesheetStatusId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
