//-----------------------------------------------------------------------
// <copyright file="UserDataObject.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that allows User Data to be manipultaed.</summary>
//-----------------------------------------------------------------------
namespace DataAccessLayer
{
    using System;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that allows User Data to be manipultaed.
    /// </summary>
    public class UserDataObject
    {
        #region Private Members

        /// <summary>Holds static UserDataObject objects.</summary>
        private static volatile UserDataObject instance;

        /// <summary>Object used in synchronisation.</summary>
        private static object syncRoot = new object();

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Prevents a default instance of the UserDataObject class from being created.
        /// </summary>
        private UserDataObject()
        {
        }

        #endregion Constructor

        #region Public Static Methods

        /// <summary>
        /// Gets instance to an UserDataObject object. 
        /// </summary>
        public static UserDataObject Instance
        {
            get
            {
                if (instance == null)
                {
                    lock (syncRoot)
                    {
                        if (instance == null)
                        {
                            instance = new UserDataObject();
                        }
                    }
                }

                return instance;
            }
        }

        /// <summary>
        /// Adds new User.
        /// </summary>
        /// <param name="employeeId">Employee id of user.</param>
        /// <param name="genderId">Gender id of the user.</param>
        /// <param name="roleId">Role of user.</param>
        /// <param name="firstname">Firstname of user.</param>
        /// <param name="surname">Surname of user.</param>
        /// <param name="email">Email of user.</param>
        /// <param name="password">password of user.</param>
        /// <param name="organisationId">Id of organisation the user belongs to.</param>
        /// <param name="approverUserId">The user who will approve this users timesheet entries.</param>
        /// <returns>Id of user.</returns>
        public long AddUser(string employeeId, byte genderId, short roleId, string firstname, string surname, string email, string password, int? organisationId, long? approverUserId)
        {
            try
            {
                return DataObject.DataAccessor.AddUser(employeeId, genderId, roleId, firstname, surname, email, password, organisationId, approverUserId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates existing user.
        /// </summary>
        /// <param name="user">User to update.</param>
        public void UpdateUser(User user)
        {
            try
            {
                DataObject.DataAccessor.UpdateUser(user);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deletes existing User.
        /// </summary>
        /// <param name="userId">Id of User to delete.</param>
        public void DeleteUser(long userId)
        {
            try
            {
                DataObject.DataAccessor.DeleteUser(userId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets the User that matches the id.
        /// </summary>
        /// <param name="userId">id of user.</param>
        /// <returns>User information.</returns>
        public User GetUser(long userId)
        {
            try
            {
                return DataObject.DataAccessor.GetUser(userId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets the User that matches the email.
        /// </summary>
        /// <param name="email">email of user.</param>
        /// <returns>User information.</returns>
        public User GetUser(string email)
        {
            try
            {
                return DataObject.DataAccessor.GetUser(email);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets a collection of Users that match the filter.
        /// </summary>
        /// <param name="filter">Filter to use when retrieving data.</param>
        /// <returns>Collection containing elements that match the filter.</returns>
        public UserCollection GetUsers(UserFilter filter)
        {
            try
            {
                return DataObject.DataAccessor.GetUsers(filter);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Disables existing user.
        /// </summary>
        /// <param name="userId">Id of User to disable.</param>
        public void DisableUser(long userId)
        {
            try
            {
                DataObject.DataAccessor.DisableUser(userId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Enables existing user.
        /// </summary>
        /// <param name="userId">Id of User to enable.</param>
        public void EnableUser(long userId)
        {
            try
            {
                DataObject.DataAccessor.EnableUser(userId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Activates existing user.
        /// </summary>
        /// <param name="userId">Id of User to activate.</param>
        public void ActivateUser(long userId)
        {
            try
            {
                DataObject.DataAccessor.ActivateUser(userId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Deactivates existing user.
        /// </summary>
        /// <param name="userId">Id of User to Deactivate.</param>
        public void DeactivateUser(long userId)
        {
            try
            {
                DataObject.DataAccessor.DeactivateUser(userId);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Updates user's password.
        /// </summary>
        /// <param name="userId">Id of User to change password for.</param>
        /// <param name="encryptedPassword">The users new password.</param>
        public void SetUserPassword(long userId, string encryptedPassword)
        {
            try
            {
                DataObject.DataAccessor.SetUserPassword(userId, encryptedPassword);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets the collection of users who are above the passed in user in terms of approver structure.
        /// </summary>
        /// <param name="userId">Id of user to find hierachy for.</param>
        /// <param name="inclusive">Whether or not to include the passed in user in the result.</param>
        /// <returns>Users that are above the passed in user in terms of heirachy</returns>
        public UserCollection GetUserHierachyAboveUser(long userId, bool inclusive)
        {
            try
            {
                return DataObject.DataAccessor.GetUserHierachyAboveUser(userId, inclusive);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        /// <summary>
        /// Gets the collection of users who are below the passed in user in terms of approver structure.
        /// </summary>
        /// <param name="userId">Id of user to find hierachy for.</param>
        /// <param name="inclusive">Whether or not to include the passed in user in the result.</param>
        /// <returns>Users that are below the passed in user in terms of heirachy</returns>
        public UserCollection GetUserHierachyBelowUser(long userId, bool inclusive)
        {
            try
            {
                return DataObject.DataAccessor.GetUserHierachyBelowUser(userId, inclusive);
            }
            catch (Exception ex)
            {
                DataObject.DataAccessor.HandleException(ex);
                throw;
            }
        }

        #endregion Public Static Methods
    }
}
