//-----------------------------------------------------------------------
// <copyright file="BasePage.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a BasePage.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.App_Code
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    /// <summary>
    /// Class that represents a base page.
    /// </summary>
    public class BasePage : System.Web.UI.Page
    {
        #region Private Members

        /// <summary>Holds Query String Manager.</summary>
        private QueryStringManager queryStringManager = null;

        /// <summary>Holds View State Manager.</summary>
        private ViewStateManager viewStateManager = null;

        /// <summary>Holds a list of errors that have occurred for this page. Should only ever be one but list created just in case.</summary>
        private List<string> errors;

        /// <summary>Holds the success message to show if there are no errors.</summary>
        private string successMessage;

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the BasePage class.
        /// </summary>
        public BasePage()
        {
            this.PreInit += new EventHandler(this.BasePage_PreInit);
        }

        #endregion Constructor

        #region Static Properties

        /// <summary>Gets a null itme.</summary>
        public static ListItem NullItem
        {
            get { return new ListItem("---", "-1"); }
        }

        #endregion Statis Properties

        #region Public Properties

        /// <summary>Gets the query string manager.</summary>
        public QueryStringManager QueryStringManager
        {
            get { return this.queryStringManager; }
        }

        /// <summary>Gets the ViewState Manager.</summary>
        public ViewStateManager ViewStateManager
        {
            get { return this.viewStateManager; }
        }

        #endregion Public Properties

        #region Public Methods

        /// <summary>
        /// Adds error to the current page that is to be displayed.
        /// </summary>
        /// <param name="message">The message to show.</param>
        public void AddErrorMessage(string message)
        {
            if (string.IsNullOrEmpty(message))
            {
                throw new ArgumentNullException("message");
            }

            if (this.errors == null)
            {
                this.errors = new List<string>();
            }

            this.errors.Add(message);
        }

        /// <summary>
        /// Shows the specified success on the page.
        /// </summary>
        /// <param name="message">The message to show.</param>
        public void SuccessMessage(string message)
        {
            if (string.IsNullOrEmpty(message))
            {
                throw new ArgumentNullException("message");
            }

            this.successMessage = message;
        }

        #endregion Public Methods

        #region Protected Methods

        /// <summary>
        /// Event Handler for on pre render.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            StringBuilder alertMessage = new StringBuilder();
            if (this.errors != null && this.errors.Count > 0)
            {
                foreach (string error in this.errors)
                {
                    alertMessage.Append(error + "\\n");
                }
            }
            else if (!string.IsNullOrEmpty(this.successMessage))
            {
                alertMessage.Append(this.successMessage);
            }

            if (alertMessage.Length > 0)
            {
                ClientScript.RegisterStartupScript(this.GetType(), "alert", "alert('" + alertMessage.ToString() + "');", true);
            }

            this.Title = CommonConstants.Configuration.SiteName + " - " + this.Title;
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Event Handler for pre Init.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void BasePage_PreInit(object sender, EventArgs e)
        {
            this.queryStringManager = new QueryStringManager(this.Page.Request.QueryString);
            this.viewStateManager = new ViewStateManager(ViewState);
            this.Theme = "Default";
        }

        #endregion Private Methods
    }
}
