//-----------------------------------------------------------------------
// <copyright file="MailManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that manages MailManager.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.App_Code
{
    using System; 
    using System.Collections.Specialized;
    using System.Net.Mail;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;

    /// <summary>
    /// Class that managers. 
    /// </summary>
    public static class MailManager
    {
        #region Public Static Methods

        /// <summary>
        /// Sends the New Password Message.
        /// </summary>
        /// <param name="toUser">The recipient.</param>
        /// <param name="newPassword">The new password</param>
        /// <param name="control">The owner control creating the email.</param>
        public static void SendNewPasswordMessage(
            User toUser,
            string newPassword, 
            System.Web.UI.Control control)
        {
            ListDictionary replacements = new ListDictionary();
            replacements.Add("<%To%>", toUser.FullName);
            replacements.Add("<%From%>", CommonConstants.Configuration.SiteName);
            replacements.Add("<%Link%>", Global.GetDomain() + control.ResolveUrl(CommonConstants.Urls.Default));
            replacements.Add("<%Password%>", newPassword);
            SendMessage(
                toUser.Email, 
                CommonConstants.Configuration.AdminEmail,
                CommonConstants.Configuration.SiteName,
                Resources.Main.NewPasswordRequest, 
                CommonConstants.EmailBodyFiles.ForgottenPasswordEmail, 
                replacements, 
                control);
        }

        /// <summary>
        /// Sends the Timesheet Submitted.
        /// </summary>
        /// <param name="timesheetId">The id of the timesheet</param>
        /// <param name="control">The owner control creating the email.</param>
        public static void SendTimesheetStatusChangedMessage(long timesheetId, System.Web.UI.Control control)
        {
            Timesheet timesheet = TimesheetManager.Instance.GetTimesheet(timesheetId);

            if (UserManager.Instance.HasApprover(timesheet.UserId))
            {
                User owner = UserManager.Instance.GetUser(timesheet.UserId);
                User approver = UserManager.Instance.GetUser(owner.ApproverUserId.Value);

                BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses status =
                    (BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses)timesheet.TimesheetStatusId;

                ListDictionary replacements = new ListDictionary();
                replacements.Add("<%From%>", CommonConstants.Configuration.SiteName);
                QueryStringManager manager = new QueryStringManager();
                manager.Add(CommonConstants.QueryStringVariables.TimesheetId, timesheetId.ToString());
                string url = QueryStringManager.AddValueToUrl(
                    CommonConstants.Urls.Secure.Timesheet,
                    CommonConstants.QueryStringVariables.Request,
                    manager.Encrypt());
                url = control.ResolveUrl(url);
                replacements.Add("<%Link%>", Global.GetDomain() + url);
                replacements.Add("<%WeekEndDate%>", timesheet.WeekEndDate.ToString(CommonConstants.Configuration.DateFormat));

                switch (status)
                {
                    case BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Approved:
                        replacements.Add("<%To%>", owner.FullName);
                        SendMessage(
                            owner.Email,
                            CommonConstants.Configuration.AdminEmail,
                            CommonConstants.Configuration.SiteName,
                            Resources.Main.TimesheetApproved,
                            CommonConstants.EmailBodyFiles.TimesheetApprovedEmail,
                            replacements,
                            control);
                        break;

                    case BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Declined:
                        replacements.Add("<%To%>", owner.FullName);
                        SendMessage(
                            owner.Email,
                            CommonConstants.Configuration.AdminEmail,
                            CommonConstants.Configuration.SiteName,
                            Resources.Main.TimesheetDeclined,
                            CommonConstants.EmailBodyFiles.TimesheetDeclinedEmail,
                            replacements,
                            control);
                        break;

                    case BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses.Submitted:
                        replacements.Add("<%To%>", approver.FullName);
                        replacements.Add("<%Submitter%>", owner.FullName);
                        SendMessage(
                            approver.Email,
                            CommonConstants.Configuration.AdminEmail,
                            CommonConstants.Configuration.SiteName,
                            Resources.Main.TimesheetWaitingApproval,
                            CommonConstants.EmailBodyFiles.TimesheetSubmittedEmail,
                            replacements,
                            control);
                        break;

                    default:
                        break;
                }
            }
        }

        /// <summary>
        /// Sends the Contact Us message.
        /// </summary>
        /// <param name="subject">The subject of the message</param>
        /// <param name="message">The tex of the message.</param>
        /// <param name="fromEmail">The sender of the message.</param>
        /// <param name="control">The owner control creating the email.</param>
        public static void SendContactUsMessage(string subject, string message, string fromEmail, System.Web.UI.Control control)
        {
            MailMessage mailMessage = new MailMessage(fromEmail, CommonConstants.Configuration.ContactUsEmail, subject, message);
            mailMessage.IsBodyHtml = false;
            Global.SendEmail(mailMessage);
        }

        #endregion Public Static Methods

        #region Private Static Methods

        /// <summary>
        /// Sends an email Message.
        /// </summary>
        /// <param name="toEmail">The recipient's email.</param>
        /// <param name="fromEmail">The sender's email.</param>
        /// <param name="from">Display Name of from.</param>
        /// <param name="subject">The subject.</param>
        /// <param name="bodyFileName">The name of the file with the email body.</param>
        /// <param name="replacements">The dictionary contaning the string replacements.</param>
        /// <param name="control">The owner control creating the email.</param>
        private static void SendMessage(
            string toEmail, 
            string fromEmail, 
            string from,
            string subject,
            string bodyFileName,
            ListDictionary replacements,
            System.Web.UI.Control control)
        {
            MailDefinition mail = new MailDefinition();
            mail.BodyFileName = bodyFileName;
            mail.From = fromEmail;
            mail.Subject = subject;

            MailMessage message = mail.CreateMailMessage(toEmail, replacements, control);
            message.IsBodyHtml = false;
            message.Sender = new MailAddress(fromEmail, from);
            Global.SendEmail(message);
        }

        #endregion Private Static Methods
    }
}
