//-----------------------------------------------------------------------
// <copyright file="QueryStringManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that manages QueryString.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.App_Code
{
    using System;
    using System.Collections.Specialized;
    using System.Text;
    using System.Web;  
    using System.Web.SessionState;
    using BusinessLogicLayer.Utilities;

    /// <summary>
    /// Class that manages Session.
    /// </summary>
    public class QueryStringManager
    {
        #region Private Members

        /// <summary>Holds query String details.</summary>
        private NameValueCollection queryString;

        #endregion Private Members

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the QueryStringManager class.
        /// </summary>
        public QueryStringManager()
        {
            this.queryString = new NameValueCollection();
        }

        /// <summary>
        /// Initializes a new instance of the QueryStringManager class.
        /// </summary>
        /// <param name="queryString">NameValueCollection conating querystring.</param>
        public QueryStringManager(NameValueCollection queryString)
        {
            if (queryString == null)
            {
                throw new ArgumentNullException("queryString");
            }

            this.queryString = queryString;
        }

        /// <summary>
        /// Initializes a new instance of the QueryStringManager class.
        /// </summary>
        /// <param name="encryptedQueryString">Encrypted querystring value.</param>
        public QueryStringManager(string encryptedQueryString)
        {
            this.queryString = Decrypt(encryptedQueryString);
        }

        #endregion Constructors

        #region Public Properties

        /// <summary>
        /// Gets a value indicating whether or not a request is in the querystring.
        /// </summary>
        public bool HasRequest
        {
            get
            {
                if (this.queryString[CommonConstants.QueryStringVariables.Request] == null)
                {
                    return false;
                }
                else
                {
                    return true;
                }
            }
        }

        /// <summary>
        /// Gets the Timesheet id.
        /// </summary>
        public string Request
        {
            get { return this.queryString[CommonConstants.QueryStringVariables.Request]; }
        }

        /// <summary>
        /// Gets a value indicating whether or not the timesheet id is in the querystring.
        /// </summary>
        public bool HasTimesheetId
        {
            get
            {
                try
                {
                    if (this.queryString[CommonConstants.QueryStringVariables.TimesheetId] == null)
                    {
                        return false;
                    }

                    Convert.ToInt64(this.queryString[CommonConstants.QueryStringVariables.TimesheetId]);
                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets the Timesheet id.
        /// </summary>
        public long TimesheetId
        {
            get { return Convert.ToInt64(this.queryString[CommonConstants.QueryStringVariables.TimesheetId]); }
        }

        /// <summary>
        /// Gets a value indicating whether or not the Organisation id is in the querystring.
        /// </summary>
        public bool HasOrganisationId
        {
            get
            {
                try
                {
                    if (this.queryString[CommonConstants.QueryStringVariables.OrganisationId] == null)
                    {
                        return false;
                    }

                    Convert.ToInt32(this.queryString[CommonConstants.QueryStringVariables.OrganisationId]);
                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets the Organisation Id.
        /// </summary>
        public int OrganisationId
        {
            get { return Convert.ToInt32(this.queryString[CommonConstants.QueryStringVariables.OrganisationId]); }
        }

        /// <summary>
        /// Gets a value indicating whether or not the user id is in the querystring.
        /// </summary>
        public bool HasUserId
        {
            get
            {
                try
                {
                    if (this.queryString[CommonConstants.QueryStringVariables.UserId] == null)
                    {
                        return false;
                    }

                    Convert.ToInt64(this.queryString[CommonConstants.QueryStringVariables.UserId]);
                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets the User Id.
        /// </summary>
        public long UserId
        {
            get { return Convert.ToInt32(this.queryString[CommonConstants.QueryStringVariables.UserId]); }
        }

        /// <summary>
        /// Gets a value indicating whether or not the Timesheet View is in the querystring.
        /// </summary>
        public bool HasTimesheetView
        {
            get
            {
                try
                {
                    if (this.queryString[CommonConstants.QueryStringVariables.TimesheetView] == null)
                    {
                        return false;
                    }

                    Enum.Parse(typeof(CommonConstants.TimesheetView), this.queryString[CommonConstants.QueryStringVariables.TimesheetView]);
                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets the TimesheetView.
        /// </summary>
        public CommonConstants.TimesheetView TimesheetView
        {
            get { return (CommonConstants.TimesheetView)Enum.Parse(typeof(CommonConstants.TimesheetView), this.queryString[CommonConstants.QueryStringVariables.TimesheetView]); }
        }

        /// <summary>
        /// Gets a value indicating whether or not the Timesheet Status Id is in the querystring.
        /// </summary>
        public bool HasTimesheetStatusId
        {
            get
            {
                try
                {
                    if (this.queryString[CommonConstants.QueryStringVariables.TimesheetStatusId] == null)
                    {
                        return false;
                    }

                    Convert.ToInt16(this.queryString[CommonConstants.QueryStringVariables.TimesheetStatusId]);
                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets the Timesheet Status Id.
        /// </summary>
        public short TimesheetStatusId
        {
            get { return Convert.ToInt16(this.queryString[CommonConstants.QueryStringVariables.TimesheetStatusId]); }
        }

        #endregion Public Properties

        #region Public Methods

        /// <summary>
        /// Adds a querystring value to the url.
        /// </summary>
        /// <param name="url">Url to add to.</param>
        /// <param name="name">Name of querystring property.</param>
        /// <param name="value">Value of property.</param>
        /// <returns>New url with query string information.</returns>
        public static string AddValueToUrl(string url, string name, string value)
        {
            string newUrl = url;
            if (newUrl.Trim().EndsWith("?") || newUrl.Trim().EndsWith("&"))
            {
                newUrl += name + "=" + value + "&";
            }
            else
            {
                newUrl += "?" + name + "=" + value + "&";
            }

            return newUrl;
        }

        /// <summary>
        /// Decrypts the querystring.
        /// </summary>
        /// <param name="encryptedQueryString">QueryString to decrypt and turn into NameValueCollection.</param>
        /// <returns>NameValueCollection containing querystring.</returns>
        public static NameValueCollection Decrypt(string encryptedQueryString)
        {
            NameValueCollection result = new NameValueCollection();
            if (!string.IsNullOrEmpty(encryptedQueryString))
            {
                string decryptedQueryString = Cryptographer.Decrypt(
                    encryptedQueryString,
                    Cryptographer.EncryptionType.RSA,
                    CommonConstants.Configuration.PrivateEncryptionKey,
                    null);

                string[] split = decryptedQueryString.Split('&');
                foreach (string pair in split)
                {
                    string[] keyValue = pair.Split('=');
                    result.Add(keyValue[0], keyValue[1]);
                }
            }

            return result;
        }

        /// <summary>
        /// Encrypts the querystring.
        /// </summary>
        /// <returns>Encrypted querystring.</returns>
        public string Encrypt()
        {
            string result = this.ToString();
            if (!string.IsNullOrEmpty(result))
            {
                result = Cryptographer.Encrypt(
                    result,
                    Cryptographer.EncryptionType.RSA,
                    CommonConstants.Configuration.PublicEncryptionKey,
                    null);

                result = HttpUtility.UrlEncode(result);
            }

            return result;
        }

        /// <summary>
        /// Adds name value pair to querystring.
        /// </summary>
        /// <param name="name">Name to add.</param>
        /// <param name="value">Matching value.</param>
        public void Add(string name, string value)
        {
            if (string.IsNullOrEmpty(name))
            {
                throw new ArgumentNullException("name");
            }

            if (string.IsNullOrEmpty(value))
            {
                throw new ArgumentNullException("value");
            }

            this.queryString.Add(name, value);
        }

        /// <summary>
        /// Gets the matching value.
        /// </summary>
        /// <param name="name">Name to look for.</param>
        /// <returns>Matching valuye.</returns>
        public string GetValue(string name)
        {
            return this.queryString[name];
        }

        /// <summary>
        /// Gets string representation of the quesry string.
        /// </summary>
        /// <returns>String representation of the querystring.</returns>
        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            foreach (string key in this.queryString)
            {
                if (!string.IsNullOrEmpty(this.queryString[key]))
                {
                    sb.Append(key + "=" + this.queryString[key] + "&");
                }
            }

            if (sb.Length > 0)
            {
                sb.Remove(sb.Length - 1, 1);
            }

            return sb.ToString();
        }

        /// <summary>
        /// Clears the querstring values.
        /// </summary>
        public void Clear()
        {
            this.queryString.Clear();
        }

        #endregion Public Methods
    }
}
