//-----------------------------------------------------------------------
// <copyright file="SecurePage.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a SecurePage.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.App_Code
{
    using System;
    using BusinessLogicLayer;

    /// <summary>
    /// Class that represents a secure page.
    /// </summary>
    public class SecurePage : BasePage
    {
        #region Private Members

        /// <summary>Holds Session Manager.</summary>
        private SessionManager sessionManager = null;

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the SecurePage class.
        /// </summary>
        public SecurePage()
        {
            this.PreInit += new EventHandler(this.SecurePage_PreInit);
        }

        #endregion Constructor

        #region Public Properties

        /// <summary>Gets the session manager.</summary>
        public SessionManager SessionManager
        {
            get { return this.sessionManager; }
        }

        /// <summary>
        /// Gets the user id.
        /// </summary>
        public long UserId
        {
            get { return this.SessionManager.UserId; }
        }

        /// <summary>
        /// Gets a value indicating whether or not the currently authenticated user is in an organisation.
        /// </summary>
        public bool IsUserInOrganisation
        {
            get
            {
                return UserManager.Instance.GetUser(this.UserId).OrganisationId.HasValue;
            }
        }

        /// <summary>
        /// Gets the currently authenticated users organisation id.
        /// </summary>
        public int UserOrganisationId
        {
            get
            {
                return UserManager.Instance.GetUser(this.UserId).OrganisationId.Value;
            }
        }

        /// <summary>
        /// Gets a value indicating whether the current user is an administrator.
        /// </summary>
        public bool IsUserAdministrator
        {
            get
            {
                return this.User.IsInRole(BusinessLogicLayer.Utilities.CommonConstants.RoleNames.Administrator.ToLower());
            }
        }

        /// <summary>
        /// Gets a value indicating whether the current user is an manager.
        /// </summary>
        public bool IsUserManager
        {
            get
            {
                return this.User.IsInRole(BusinessLogicLayer.Utilities.CommonConstants.RoleNames.Manager.ToLower());
            }
        }

        /// <summary>
        /// Gets a value indicating whether the current user is a standard user.
        /// </summary>
        public bool IsUserStandard
        {
            get
            {
                return this.User.IsInRole(BusinessLogicLayer.Utilities.CommonConstants.RoleNames.Standard.ToLower());
            }
        }

        #endregion Public Properties

        #region Private Methods

        /// <summary>
        /// Event Handler for pre Init.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void SecurePage_PreInit(object sender, EventArgs e)
        {
            // Check is user is authenticated.
            if (this.User.Identity.IsAuthenticated == false)
            {
                Response.Redirect(CommonConstants.Urls.Default);
                return;
            }

            this.sessionManager = new SessionManager(this);
        }

        #endregion Private Methods
    }
}
