//-----------------------------------------------------------------------
// <copyright file="SessionManager.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that manages session.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.App_Code
{
    using System;
    using System.Web.SessionState;
    using System.Web.UI;
    using BusinessObjects.Filters;

    /// <summary>
    /// Class that manages Session.
    /// </summary>
    public class SessionManager
    {
        #region Private Members

        /// <summary>
        /// Holds the page details.
        /// </summary>
        private SecurePage page;

        #endregion Private Members

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the SessionManager class.
        /// </summary>
        /// <param name="page">Page object containing current page information.</param>
        public SessionManager(SecurePage page)
        {
            if (page == null)
            {
                throw new ArgumentNullException("page");
            }

            this.page = page;
        }

        #endregion Constructor

        #region Public Properties

        /// <summary>
        /// Gets or sets the user id.
        /// </summary>
        public long UserId
        {
            get 
            {
                // Check if user id already in session
                if (this.page.Session[CommonConstants.SessionVariables.UserId] != null)
                {
                    return Convert.ToInt64(this.page.Session[CommonConstants.SessionVariables.UserId]);
                }
                else
                {
                    // Get user id from database and save in session
                    if (this.page.User.Identity.IsAuthenticated)
                    {
                        this.UserId = BusinessLogicLayer.UserManager.Instance.GetUser(this.page.User.Identity.Name).UserId;
                        return this.UserId;
                    }
                    else
                    {
                        this.UserId = 0;
                        return this.UserId;
                    }
                }
            }

            set 
            { 
                this.page.Session[CommonConstants.SessionVariables.UserId] = value; 
            }
        }

        /// <summary>
        /// Gets or sets the Timesheets Filter.
        /// </summary>
        public TimesheetFilter TimesheetsFilter
        {
            get 
            {
                if (this.page.Session[CommonConstants.SessionVariables.TimesheetsFilter] == null)
                {
                    this.TimesheetsFilter = new TimesheetFilter();
                    this.TimesheetsFilter.StartDateTime = Utilities.GetWeekEnd(DateTime.Now.AddDays(-CommonConstants.Configuration.TimesheetsSearchDefaultDaysBuffer));
                    this.TimesheetsFilter.EndDateTime = Utilities.GetWeekEnd(DateTime.Now);
                    this.TimesheetsFilter.MaxRecords = CommonConstants.Configuration.MaxRowsReturned;

                    if (this.page.IsUserInOrganisation)
                    {
                        this.TimesheetsFilter.OrganisationId = this.page.UserOrganisationId;
                    }

                    if (!this.page.IsUserAdministrator
                        && !this.page.IsUserManager
                        && BusinessLogicLayer.UserManager.Instance.IsUserApprover(this.UserId))
                    {
                        // user is not admin, nor manager but is an approver
                        this.TimesheetsFilter.ApproverUserId = this.UserId;
                    }
                }

                return this.page.Session[CommonConstants.SessionVariables.TimesheetsFilter] as TimesheetFilter; 
            }

            set 
            { 
                this.page.Session[CommonConstants.SessionVariables.TimesheetsFilter] = value; 
            }
        }

        /// <summary>
        /// Gets or sets the Timesheets Filter.
        /// </summary>
        public TimesheetFilter MyTimesheetsFilter
        {
            get
            {
                if (this.page.Session[CommonConstants.SessionVariables.MyTimesheetsFilter] == null)
                {
                    this.MyTimesheetsFilter = new TimesheetFilter();
                    this.MyTimesheetsFilter.StartDateTime = Utilities.GetWeekEnd(DateTime.Now.AddDays(-CommonConstants.Configuration.TimesheetsSearchDefaultDaysBuffer));
                    this.MyTimesheetsFilter.EndDateTime = Utilities.GetWeekEnd(DateTime.Now);
                    this.MyTimesheetsFilter.UserId = this.page.UserId;
                    this.MyTimesheetsFilter.MaxRecords = CommonConstants.Configuration.MaxRowsReturned;
                }

                return this.page.Session[CommonConstants.SessionVariables.MyTimesheetsFilter] as TimesheetFilter;
            }

            set
            {
                this.page.Session[CommonConstants.SessionVariables.MyTimesheetsFilter] = value;
            }
        }

        #endregion Public Properties

        #region Public Methods

        /// <summary>
        /// Clears the search filter.
        /// </summary>
        public void ClearMyTimesheetsFilter()
        {
            this.MyTimesheetsFilter = new TimesheetFilter();
            this.MyTimesheetsFilter.UserId = this.page.UserId;
            this.MyTimesheetsFilter.MaxRecords = CommonConstants.Configuration.MaxRowsReturned;
        }

        /// <summary>
        /// Clears the search filter.
        /// </summary>
        public void ClearTimesheetsFilter()
        {
            this.TimesheetsFilter = new TimesheetFilter();
            this.TimesheetsFilter.MaxRecords = CommonConstants.Configuration.MaxRowsReturned;

            if (this.page.IsUserInOrganisation)
            {
                this.TimesheetsFilter.OrganisationId = this.page.UserOrganisationId;
            }

            if (!this.page.IsUserAdministrator
                        && !this.page.IsUserManager
                        && BusinessLogicLayer.UserManager.Instance.IsUserApprover(this.UserId))
            {
                // user is not admin, nor manager but is an approver
                this.TimesheetsFilter.ApproverUserId = this.UserId;
            }
        }

        #endregion Public Methods
    }
}
