//-----------------------------------------------------------------------
// <copyright file="TimesheetTable.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that Timesheet Table.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.App_Code
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    
    /// <summary>
    /// Class used to represent timesheet table. This is the most complex class in the presentation layer
    /// as it is used to create the timesheet table with the correct rows, columns, controls and access rights.
    /// It takes a html table and :
    /// - adds attendance rows (textboxes and buttons)
    /// - adds activity rows (textboxes, dropdonwlists, buttons)
    /// The controls are all dynamic because each timesheet may contain a different number of rows from each other.
    /// Also users can add and remove rows if and when they want to.
    /// </summary>
    [Serializable]
    public class TimesheetTable
    {
        #region Private Properties

        /// <summary>Holds the timesheet.</summary>
        private FullTimesheet fullTimesheet;

        /// <summary>Holds the start Date.</summary>
        private DateTime startDate;

        /// <summary>The id of the organisation the current user belongs to.</summary>
        private int? organisationId;

        /// <summary>Holds whether or not the timesheet is readonly.</summary>
        private bool readOnly;

        /// <summary>Holds the attendances data.</summary>
        private DataTable attendanceDataTable;

        /// <summary>Holds the activities data.</summary>
        private DataTable activitiesDataTable;

        #endregion Private Properties

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the TimesheetTable class. Creates a timesheet from an existing timesheet.
        /// </summary>
        /// <param name="fullTimesheet">Timeshhet information.</param>
        /// <param name="readOnly">Whether the timesheet should be read only.</param>
        public TimesheetTable(FullTimesheet fullTimesheet, bool readOnly)
        {
            this.fullTimesheet = fullTimesheet;
            this.startDate = fullTimesheet.StartDate;
            this.organisationId = BusinessLogicLayer.UserManager.Instance.GetUser(fullTimesheet.TimeSheet.UserId).OrganisationId;
            this.readOnly = readOnly;
        }

        /// <summary>
        /// Initializes a new instance of the TimesheetTable class. Creates a blank timesheet for the user.
        /// </summary>
        /// <param name="userId">Id of the user.</param>
        /// <param name="date">Date of timesheet.</param>
        public TimesheetTable(long userId, DateTime date)
        {
            this.startDate = Utilities.GetWeekStart(date);
            this.organisationId = BusinessLogicLayer.UserManager.Instance.GetUser(userId).OrganisationId;
        }

        #endregion Constructors

        #region Public Properties

        /// <summary>
        /// Gets the start date.
        /// </summary>
        public DateTime StartDate
        {
            get { return this.startDate.Date; }
        }

        /// <summary>
        /// Gets the end date.
        /// </summary>
        public DateTime EndDate
        {
            get { return this.StartDate.AddDays(6); }
        }

        /// <summary>
        /// Gets the Timesheet.
        /// </summary>
        public FullTimesheet Timesheet
        {
            get { return this.fullTimesheet; }
        }

        /// <summary>
        /// Gets the Attendance DataTable
        /// </summary>
        public DataTable AttendanceDataTable
        {
            get
            {
                if (this.attendanceDataTable == null)
                {
                    return this.CreateAttendanceDataTableFromDataBase();
                }

                return this.attendanceDataTable;
            }
        }

        /// <summary>
        /// Gets the Attendance DataTable
        /// </summary>
        public DataTable ActivitiesDataTable
        {
            get
            {
                if (this.activitiesDataTable == null)
                {
                    return this.CreateActivitiesDataTableFromDataBase();
                }

                return this.activitiesDataTable;
            }
        }

        #endregion Public Properties

        #region Public Methods

        /// <summary>
        /// Clears the data tables.
        /// </summary>
        public void Clear()
        {
            if (this.attendanceDataTable != null)
            {
                this.attendanceDataTable.Clear();
                this.attendanceDataTable = null;
            }

            if (this.activitiesDataTable != null)
            {
                this.activitiesDataTable.Clear();
                this.activitiesDataTable = null;
            }
        }

        /// <summary>
        /// Gets contents from repeater and saves it to a datable.
        /// </summary>
        /// <param name="attendancesRepeater">The repeater to save.</param>
        /// <returns>The datatable that holds the data from the repeater.</returns>
        public DataTable SaveAttendancesRepeaterData(Repeater attendancesRepeater)
        {
            this.attendanceDataTable = new DataTable();
            this.attendanceDataTable.Columns.Add("Id");
            this.attendanceDataTable.Columns.Add("ReadOnly");

            this.attendanceDataTable.Columns.Add("SatIn");
            this.attendanceDataTable.Columns.Add("SunIn");
            this.attendanceDataTable.Columns.Add("MonIn");
            this.attendanceDataTable.Columns.Add("TueIn");
            this.attendanceDataTable.Columns.Add("WedIn");
            this.attendanceDataTable.Columns.Add("ThurIn");
            this.attendanceDataTable.Columns.Add("FriIn");
            this.attendanceDataTable.Columns.Add("SatOut");
            this.attendanceDataTable.Columns.Add("SunOut");
            this.attendanceDataTable.Columns.Add("MonOut");
            this.attendanceDataTable.Columns.Add("TueOut");
            this.attendanceDataTable.Columns.Add("WedOut");
            this.attendanceDataTable.Columns.Add("ThurOut");
            this.attendanceDataTable.Columns.Add("FriOut");

            this.attendanceDataTable.Columns.Add("SatInCssClass");
            this.attendanceDataTable.Columns.Add("SunInCssClass");
            this.attendanceDataTable.Columns.Add("MonInCssClass");
            this.attendanceDataTable.Columns.Add("TueInCssClass");
            this.attendanceDataTable.Columns.Add("WedInCssClass");
            this.attendanceDataTable.Columns.Add("ThurInCssClass");
            this.attendanceDataTable.Columns.Add("FriInCssClass");
            this.attendanceDataTable.Columns.Add("SatOutCssClass");
            this.attendanceDataTable.Columns.Add("SunOutCssClass");
            this.attendanceDataTable.Columns.Add("MonOutCssClass");
            this.attendanceDataTable.Columns.Add("TueOutCssClass");
            this.attendanceDataTable.Columns.Add("WedOutCssClass");
            this.attendanceDataTable.Columns.Add("ThurOutCssClass");
            this.attendanceDataTable.Columns.Add("FriOutCssClass");

            foreach (RepeaterItem item in attendancesRepeater.Items)
            {
                Label idLabel = item.FindControl("IdLabel") as Label;
                TextBox satInTextBox = item.FindControl("SatInTextBox") as TextBox;
                TextBox sunInTextBox = item.FindControl("SunInTextBox") as TextBox;
                TextBox monInTextBox = item.FindControl("MonInTextBox") as TextBox;
                TextBox tueInTextBox = item.FindControl("TueInTextBox") as TextBox;
                TextBox wedInTextBox = item.FindControl("WedInTextBox") as TextBox;
                TextBox thurInTextBox = item.FindControl("ThurInTextBox") as TextBox;
                TextBox friInTextBox = item.FindControl("FriInTextBox") as TextBox;
                TextBox satOutTextBox = item.FindControl("SatOutTextBox") as TextBox;
                TextBox sunOutTextBox = item.FindControl("SunOutTextBox") as TextBox;
                TextBox monOutTextBox = item.FindControl("MonOutTextBox") as TextBox;
                TextBox tueOutTextBox = item.FindControl("TueOutTextBox") as TextBox;
                TextBox wedOutTextBox = item.FindControl("WedOutTextBox") as TextBox;
                TextBox thurOutTextBox = item.FindControl("ThurOutTextBox") as TextBox;
                TextBox friOutTextBox = item.FindControl("FriOutTextBox") as TextBox;

                DataRow row = this.attendanceDataTable.NewRow();
                row["Id"] = idLabel.Text;
                row["ReadOnly"] = this.readOnly;

                string text = satInTextBox.Text;
                row["SatInCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutWeekendTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["SatIn"] = text;
                satInTextBox.Text = text;

                text = sunInTextBox.Text;
                row["SunInCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutWeekendTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["SunIn"] = text;
                sunInTextBox.Text = text;

                text = monInTextBox.Text;
                row["MonInCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["MonIn"] = text;
                monInTextBox.Text = text;

                text = tueInTextBox.Text;
                row["TueInCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["TueIn"] = text;
                tueInTextBox.Text = text;

                text = wedInTextBox.Text;
                row["WedInCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["WedIn"] = text;
                wedInTextBox.Text = text;

                text = thurInTextBox.Text;
                row["ThurInCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["ThurIn"] = text;
                thurInTextBox.Text = text;

                text = friInTextBox.Text;
                row["FriInCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["FriIn"] = text;
                friInTextBox.Text = text;

                text = satOutTextBox.Text;
                row["SatOutCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutWeekendTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["SatOut"] = text;
                satOutTextBox.Text = text;

                text = sunOutTextBox.Text;
                row["SunOutCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutWeekendTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["SunOut"] = text;
                sunOutTextBox.Text = text;

                text = monOutTextBox.Text;
                row["MonOutCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["MonOut"] = text;
                monOutTextBox.Text = text;

                text = tueOutTextBox.Text;
                row["TueOutCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["TueOut"] = text;
                tueOutTextBox.Text = text;

                text = wedOutTextBox.Text;
                row["WedOutCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["WedOut"] = text;
                wedOutTextBox.Text = text;

                text = thurOutTextBox.Text;
                row["ThurOutCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["ThurOut"] = text;
                thurOutTextBox.Text = text;

                text = friOutTextBox.Text;
                row["FriOutCssClass"] = this.ValidateInOutText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["FriOut"] = text;
                friOutTextBox.Text = text;
                this.attendanceDataTable.Rows.Add(row);
            }

            return this.attendanceDataTable;
        }

        /// <summary>
        /// Adds a new attendance row to the attendance datatable.
        /// </summary>
        /// <param name="rowId">Id of the row.</param>
        public void AddAttendanceRow(string rowId)
        {
            for (int i = 0; i < this.attendanceDataTable.Rows.Count; i++)
            {
                if (this.attendanceDataTable.Rows[i]["Id"].ToString() == rowId)
                {
                    DataRow row = this.attendanceDataTable.NewRow();
                    row["Id"] = Guid.NewGuid().ToString();
                    row["ReadOnly"] = this.readOnly;
                    row["SatInCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                    row["SunInCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                    row["MonInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["TueInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["WedInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["ThurInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["FriInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["SatOutCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                    row["SunOutCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                    row["MonOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["TueOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["WedOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["ThurOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["FriOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    this.attendanceDataTable.Rows.InsertAt(row, i + 1);
                    break;
                }
            }
        }

        /// <summary>
        /// Removes an attendance row from the attendance datatable.
        /// </summary>
        /// <param name="rowId">Id of the row.</param>
        public void RemoveAttendanceRow(string rowId)
        {
            if (this.attendanceDataTable.Rows.Count > 1)
            {
                for (int i = 0; i < this.attendanceDataTable.Rows.Count; i++)
                {
                    if (this.attendanceDataTable.Rows[i]["Id"].ToString() == rowId)
                    {
                        this.attendanceDataTable.Rows.RemoveAt(i);
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// Gets contents from repeater and saves it to a datable.
        /// </summary>
        /// <param name="activities">The repeater to save.</param>
        /// <returns>The datatable that holds the data from the repeater.</returns>
        public DataTable SaveActivitiesRepeaterData(Repeater activities)
        {
            this.activitiesDataTable = new DataTable();
            this.activitiesDataTable.Columns.Add("Id");
            this.activitiesDataTable.Columns.Add("ActivityId");
            this.activitiesDataTable.Columns.Add("ReadOnly");

            this.activitiesDataTable.Columns.Add("Sat");
            this.activitiesDataTable.Columns.Add("Sun");
            this.activitiesDataTable.Columns.Add("Mon");
            this.activitiesDataTable.Columns.Add("Tue");
            this.activitiesDataTable.Columns.Add("Wed");
            this.activitiesDataTable.Columns.Add("Thur");
            this.activitiesDataTable.Columns.Add("Fri");
            this.activitiesDataTable.Columns.Add("Description");

            this.activitiesDataTable.Columns.Add("SatCssClass");
            this.activitiesDataTable.Columns.Add("SunCssClass");
            this.activitiesDataTable.Columns.Add("MonCssClass");
            this.activitiesDataTable.Columns.Add("TueCssClass");
            this.activitiesDataTable.Columns.Add("WedCssClass");
            this.activitiesDataTable.Columns.Add("ThurCssClass");
            this.activitiesDataTable.Columns.Add("FriCssClass");

            foreach (RepeaterItem item in activities.Items)
            {
                Label idLabel = item.FindControl("IdLabel") as Label;
                DropDownList activityList = item.FindControl("ActivityList") as DropDownList;
                TextBox satTextBox = item.FindControl("SatTextBox") as TextBox;
                TextBox sunTextBox = item.FindControl("SunTextBox") as TextBox;
                TextBox monTextBox = item.FindControl("MonTextBox") as TextBox;
                TextBox tueTextBox = item.FindControl("TueTextBox") as TextBox;
                TextBox wedTextBox = item.FindControl("WedTextBox") as TextBox;
                TextBox thurTextBox = item.FindControl("ThurTextBox") as TextBox;
                TextBox friTextBox = item.FindControl("FriTextBox") as TextBox;
                TextBox descriptionTextBox = item.FindControl("DescriptionTextBox") as TextBox;

                DataRow row = this.activitiesDataTable.NewRow();
                row["Id"] = idLabel.Text;
                row["ActivityId"] = activityList.SelectedValue;
                row["ReadOnly"] = this.readOnly;

                string text = satTextBox.Text;
                row["SatCssClass"] = this.ValidateActivityText(ref text) ? CommonConstants.CssStyles.InOutWeekendTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["Sat"] = text;
                satTextBox.Text = text;

                text = sunTextBox.Text;
                row["SunCssClass"] = this.ValidateActivityText(ref text) ? CommonConstants.CssStyles.InOutWeekendTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["Sun"] = text;
                sunTextBox.Text = text;

                text = monTextBox.Text;
                row["MonCssClass"] = this.ValidateActivityText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["Mon"] = text;
                monTextBox.Text = text;

                text = tueTextBox.Text;
                row["TueCssClass"] = this.ValidateActivityText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["Tue"] = text;
                tueTextBox.Text = text;

                text = wedTextBox.Text;
                row["WedCssClass"] = this.ValidateActivityText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["Wed"] = text;
                wedTextBox.Text = text;

                text = thurTextBox.Text;
                row["ThurCssClass"] = this.ValidateActivityText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["Thur"] = text;
                thurTextBox.Text = text;

                text = friTextBox.Text;
                row["FriCssClass"] = this.ValidateActivityText(ref text) ? CommonConstants.CssStyles.InOutTextboxClassName : CommonConstants.CssStyles.InOutTextboxErrorClassName;
                row["Fri"] = text;
                friTextBox.Text = text;

                row["Description"] = descriptionTextBox.Text;

                this.activitiesDataTable.Rows.Add(row);
            }

            return this.activitiesDataTable;
        }

        /// <summary>
        /// Populates the job code list depending on the selected activity.
        /// </summary>
        /// <param name="jobCodeList">The list to populate.</param>
        /// <param name="activityId">The activity Id.</param>
        public void PopulateJobCodeList(DropDownList jobCodeList, long activityId)
        {
            jobCodeList.Items.Clear();
            JobCodeFilter filter = new JobCodeFilter();
            if (!this.readOnly)
            {
                filter.Enabled = true;
            }

            filter.OrganisationId = this.organisationId;
            JobCodeCollection jobCodes = JobCodeManager.Instance.GetJobCodes(filter);
            foreach (JobCode code in jobCodes)
            {
                ListItem item = new ListItem(code.Name, code.JobCodeId.ToString());
                jobCodeList.Items.Add(item);
            }

            if (activityId != -1)
            {
                Activity activity = ActivityManager.Instance.GetActivity(activityId);
                jobCodeList.SelectedValue = activity.JobCodeId.ToString();
            }
        }

        /// <summary>
        /// Populates the activity list depending on the selected job code.
        /// </summary>
        /// <param name="activityList">The list to populate.</param>
        /// <param name="jobCodeId">The job code.</param>
        public void PopulateActivityList(DropDownList activityList, long jobCodeId)
        {
            activityList.Items.Clear();
            ActivityFilter filter = new ActivityFilter();
            if (!this.readOnly)
            {
                filter.Enabled = true;
            }

            filter.JobCodeId = jobCodeId;

            ActivityCollection activities = ActivityManager.Instance.GetActivities(filter);
            foreach (Activity activity in activities)
            {
                ListItem item = new ListItem(activity.Name, activity.ActivityId.ToString());
                activityList.Items.Add(item);
            }
        }

        /// <summary>
        /// Gets the activity id.
        /// </summary>
        /// <param name="rowId">The row id.</param>
        /// <returns>The activity id.</returns>
        public long GetActivityId(string rowId)
        {
            foreach (DataRow row in this.ActivitiesDataTable.Rows)
            {
                if (row["Id"].ToString() == rowId)
                {
                    try
                    {
                        return Convert.ToInt64(row["ActivityId"]);
                    }
                    catch
                    {
                        return -1;
                    }
                }
            }

            return -1;
        }

        /// <summary>
        /// Sets the activity id to the job codes default.
        /// </summary>
        /// <param name="rowId">The row id.</param>
        /// <param name="jobCodeId">The job code id</param>
        public void SetActivityId(string rowId, long jobCodeId)
        {
            foreach (DataRow row in this.ActivitiesDataTable.Rows)
            {
                if (row["Id"].ToString() == rowId)
                {
                    ActivityFilter filter = new ActivityFilter();
                    if (!this.readOnly)
                    {
                        filter.Enabled = true;
                    }

                    filter.JobCodeId = jobCodeId;
                    ActivityCollection activities = ActivityManager.Instance.GetActivities(filter);
                    row["ActivityId"] = activities[0].ActivityId;
                    break;
                }
            }
        }

        /// <summary>
        /// Adds an activity row to the datatable.
        /// </summary>
        /// <param name="rowId">The row id.</param>
        public void AddActivityRow(string rowId)
        {
            for (int i = 0; i < this.activitiesDataTable.Rows.Count; i++)
            {
                if (this.activitiesDataTable.Rows[i]["Id"].ToString() == rowId)
                {
                    DataRow row = this.activitiesDataTable.NewRow();
                    row["Id"] = Guid.NewGuid().ToString();
                    row["ReadOnly"] = this.readOnly;
                    row["SatCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                    row["SunCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                    row["MonCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["TueCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["WedCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["ThurCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    row["FriCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                    this.activitiesDataTable.Rows.InsertAt(row, i + 1);
                    break;
                }
            }
        }

        /// <summary>
        /// Removes the activity row from the datatable.
        /// </summary>
        /// <param name="rowId">The row id.</param>
        public void RemoveActivityRow(string rowId)
        {
            if (this.activitiesDataTable.Rows.Count > 1)
            {
                for (int i = 0; i < this.activitiesDataTable.Rows.Count; i++)
                {
                    if (this.activitiesDataTable.Rows[i]["Id"].ToString() == rowId)
                    {
                        this.activitiesDataTable.Rows.RemoveAt(i);
                        break;
                    }
                }
            }
        }

        /// <summary>
        /// Gets the attendance sum for the date specified.
        /// </summary>
        /// <param name="dayOfWeek">The day of the week.</param>
        /// <returns>The sum of the attendance.</returns>
        public decimal GetAttendanceSum(DayOfWeek dayOfWeek)
        {
            decimal totalHours = 0;
            foreach (DataRow row in this.attendanceDataTable.Rows)
            {
                string inTime = null;
                string outTime = null;
                switch (dayOfWeek)
                {
                    case DayOfWeek.Friday:
                        inTime = row["FriIn"].ToString();
                        outTime = row["FriOut"].ToString();
                        break;

                    case DayOfWeek.Monday:
                        inTime = row["MonIn"].ToString();
                        outTime = row["MonOut"].ToString();
                        break;

                    case DayOfWeek.Saturday:
                        inTime = row["SatIn"].ToString();
                        outTime = row["SatOut"].ToString();
                        break;

                    case DayOfWeek.Sunday:
                        inTime = row["SunIn"].ToString();
                        outTime = row["SunOut"].ToString();
                        break;

                    case DayOfWeek.Thursday:
                        inTime = row["ThurIn"].ToString();
                        outTime = row["ThurOut"].ToString();
                        break;

                    case DayOfWeek.Tuesday:
                        inTime = row["TueIn"].ToString();
                        outTime = row["TueOut"].ToString();
                        break;

                    case DayOfWeek.Wednesday:
                        inTime = row["WedIn"].ToString();
                        outTime = row["WedOut"].ToString();
                        break;
                }

                DateTime? inDateTime = null;
                DateTime? outDateTime = null;
                try
                {
                    DateTime now = DateTime.Now;
                    int hour = Convert.ToInt32(inTime.Substring(0, 2));
                    int min = Convert.ToInt32(inTime.Substring(2, 2));
                    inDateTime = new DateTime(now.Year, now.Month, now.Day, hour, min, 0);

                    hour = Convert.ToInt32(outTime.Substring(0, 2));
                    min = Convert.ToInt32(outTime.Substring(2, 2));
                    outDateTime = new DateTime(now.Year, now.Month, now.Day, hour, min, 0);
                }
                catch
                {
                }

                if (inDateTime.HasValue && outDateTime.HasValue)
                {
                    TimeSpan span = outDateTime.Value - inDateTime.Value;
                    totalHours += (decimal)span.TotalHours;
                }
            }

            return totalHours;
        }

        /// <summary>
        /// Gets the activity sum for the date specified.
        /// </summary>
        /// <param name="dayOfWeek">The day of the week.</param>
        /// <returns>The sum of the activity.</returns>
        public decimal GetActivitySum(DayOfWeek dayOfWeek)
        {
            decimal total = 0;
            foreach (DataRow row in this.activitiesDataTable.Rows)
            {
                string val = null;
                switch (dayOfWeek)
                {
                    case DayOfWeek.Friday:
                        val = row["Fri"].ToString();
                        break;

                    case DayOfWeek.Monday:
                        val = row["Mon"].ToString();
                        break;

                    case DayOfWeek.Saturday:
                        val = row["Sat"].ToString();
                        break;

                    case DayOfWeek.Sunday:
                        val = row["Sun"].ToString();
                        break;

                    case DayOfWeek.Thursday:
                        val = row["Thur"].ToString();
                        break;

                    case DayOfWeek.Tuesday:
                        val = row["Tue"].ToString();
                        break;

                    case DayOfWeek.Wednesday:
                        val = row["Wed"].ToString();
                        break;
                }

                try
                {
                    total += Convert.ToDecimal(val);
                }
                catch
                {
                }
            }

            return total;
        }

        /// <summary>
        /// Gets the data in the datatables and assigns them to the collections.
        /// </summary>
        /// <param name="userId">The id of the user.</param>
        /// <param name="timesheet">The timesheet returned.</param>
        /// <param name="attendances">The attendances returned.</param>
        /// <param name="activities">The activities returned.</param>
        /// <returns>True on success.</returns>
        public bool GetTimesheet(
            long userId,
            out BusinessObjects.Timesheet timesheet,
            out TimesheetAttendanceEntryCollection attendances,
            out TimesheetActivityEntryCollection activities)
        {
            timesheet = new BusinessObjects.Timesheet(
                this.fullTimesheet == null ? 0 : this.fullTimesheet.TimeSheet.TimesheetId,
                userId,
                this.EndDate);
            attendances = new TimesheetAttendanceEntryCollection();
            activities = new TimesheetActivityEntryCollection();

            DateTime currDate = this.StartDate;
            foreach (DataRow attendanceRow in this.attendanceDataTable.Rows)
            {
                attendances.AddRange(this.GetAttendancesFromRow(this.StartDate, attendanceRow));
            }

            foreach (DataRow activityRow in this.activitiesDataTable.Rows)
            {
                activities.AddRange(this.GetActivitiesFromRow(this.StartDate, activityRow));
            }

            if (attendances.Count == 0
                || activities.Count == 0)
            {
                return false;
            }

            return true;
        }

        #endregion Public Methods

        #region Private Methods

        /// <summary>
        /// Creates the Attendances datatable from data that was retrieved from the database.
        /// </summary>
        /// <returns>The datatable.</returns>
        private DataTable CreateAttendanceDataTableFromDataBase()
        {
            this.attendanceDataTable = new DataTable();
            this.attendanceDataTable.Columns.Add("Id");
            this.attendanceDataTable.Columns.Add("ReadOnly");

            this.attendanceDataTable.Columns.Add("SatIn");
            this.attendanceDataTable.Columns.Add("SunIn");
            this.attendanceDataTable.Columns.Add("MonIn");
            this.attendanceDataTable.Columns.Add("TueIn");
            this.attendanceDataTable.Columns.Add("WedIn");
            this.attendanceDataTable.Columns.Add("ThurIn");
            this.attendanceDataTable.Columns.Add("FriIn");
            this.attendanceDataTable.Columns.Add("SatOut");
            this.attendanceDataTable.Columns.Add("SunOut");
            this.attendanceDataTable.Columns.Add("MonOut");
            this.attendanceDataTable.Columns.Add("TueOut");
            this.attendanceDataTable.Columns.Add("WedOut");
            this.attendanceDataTable.Columns.Add("ThurOut");
            this.attendanceDataTable.Columns.Add("FriOut");

            this.attendanceDataTable.Columns.Add("SatInCssClass");
            this.attendanceDataTable.Columns.Add("SunInCssClass");
            this.attendanceDataTable.Columns.Add("MonInCssClass");
            this.attendanceDataTable.Columns.Add("TueInCssClass");
            this.attendanceDataTable.Columns.Add("WedInCssClass");
            this.attendanceDataTable.Columns.Add("ThurInCssClass");
            this.attendanceDataTable.Columns.Add("FriInCssClass");
            this.attendanceDataTable.Columns.Add("SatOutCssClass");
            this.attendanceDataTable.Columns.Add("SunOutCssClass");
            this.attendanceDataTable.Columns.Add("MonOutCssClass");
            this.attendanceDataTable.Columns.Add("TueOutCssClass");
            this.attendanceDataTable.Columns.Add("WedOutCssClass");
            this.attendanceDataTable.Columns.Add("ThurOutCssClass");
            this.attendanceDataTable.Columns.Add("FriOutCssClass");

            int rowCount = this.fullTimesheet == null ? 1 : this.fullTimesheet.MaxInOutsInDay;
            if (rowCount == 0)
            {
                rowCount = 1;
            }

            for (int i = 0; i < rowCount; i++)
            {
                DataRow row = this.attendanceDataTable.NewRow();
                row["Id"] = Guid.NewGuid().ToString();
                row["ReadOnly"] = this.readOnly;
                row["SatIn"] = this.GetAttendanceText(this.StartDate, i, true);
                row["SunIn"] = this.GetAttendanceText(this.StartDate.AddDays(1), i, true);
                row["MonIn"] = this.GetAttendanceText(this.StartDate.AddDays(2), i, true);
                row["TueIn"] = this.GetAttendanceText(this.StartDate.AddDays(3), i, true);
                row["WedIn"] = this.GetAttendanceText(this.StartDate.AddDays(4), i, true);
                row["ThurIn"] = this.GetAttendanceText(this.StartDate.AddDays(5), i, true);
                row["FriIn"] = this.GetAttendanceText(this.StartDate.AddDays(6), i, true);
                row["SatOut"] = this.GetAttendanceText(this.StartDate, i, false);
                row["SunOut"] = this.GetAttendanceText(this.StartDate.AddDays(1), i, false);
                row["MonOut"] = this.GetAttendanceText(this.StartDate.AddDays(2), i, false);
                row["TueOut"] = this.GetAttendanceText(this.StartDate.AddDays(3), i, false);
                row["WedOut"] = this.GetAttendanceText(this.StartDate.AddDays(4), i, false);
                row["ThurOut"] = this.GetAttendanceText(this.StartDate.AddDays(5), i, false);
                row["FriOut"] = this.GetAttendanceText(this.StartDate.AddDays(6), i, false);

                row["SatInCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                row["SunInCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                row["MonInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["TueInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["WedInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["ThurInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["FriInCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["SatOutCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                row["SunOutCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                row["MonOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["TueOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["WedOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["ThurOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["FriOutCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;

                this.attendanceDataTable.Rows.Add(row);
            }

            return this.attendanceDataTable;
        }

        /// <summary>
        /// Gets the attendance text to show.
        /// </summary>
        /// <param name="date">The date of the attendance,</param>
        /// <param name="index">The index of the row.</param>
        /// <param name="isIn">Wteher it is an "in" row.</param>
        /// <returns>The text to display.</returns>
        private string GetAttendanceText(DateTime date, int index, bool isIn)
        {
            try
            {
                TimesheetAttendanceEntry entry = this.fullTimesheet.TimesheetAttendanceEntries.GetEntry(date, index);
                if (entry != null)
                {
                    if (isIn)
                    {
                        return entry.StartDateTime.ToString("HHmm");
                    }
                    else
                    {
                        return entry.EndDateTime.ToString("HHmm");
                    }
                }
            }
            catch
            {
            }

            return string.Empty;
        }

        /// <summary>
        /// Creates the Activities datatable from data that was retrieved from the database.
        /// </summary>
        /// <returns>The datatable.</returns>
        private DataTable CreateActivitiesDataTableFromDataBase()
        {
            this.activitiesDataTable = new DataTable();
            this.activitiesDataTable.Columns.Add("Id");
            this.activitiesDataTable.Columns.Add("ActivityId");
            this.activitiesDataTable.Columns.Add("ReadOnly");

            this.activitiesDataTable.Columns.Add("Sat");
            this.activitiesDataTable.Columns.Add("Sun");
            this.activitiesDataTable.Columns.Add("Mon");
            this.activitiesDataTable.Columns.Add("Tue");
            this.activitiesDataTable.Columns.Add("Wed");
            this.activitiesDataTable.Columns.Add("Thur");
            this.activitiesDataTable.Columns.Add("Fri");
            this.activitiesDataTable.Columns.Add("Description");

            this.activitiesDataTable.Columns.Add("SatCssClass");
            this.activitiesDataTable.Columns.Add("SunCssClass");
            this.activitiesDataTable.Columns.Add("MonCssClass");
            this.activitiesDataTable.Columns.Add("TueCssClass");
            this.activitiesDataTable.Columns.Add("WedCssClass");
            this.activitiesDataTable.Columns.Add("ThurCssClass");
            this.activitiesDataTable.Columns.Add("FriCssClass");

            int rowCount = this.fullTimesheet == null ? 1 : this.fullTimesheet.TotalActivityRows;
            if (rowCount == 0)
            {
                rowCount = 1;
            }

            for (int i = 0; i < rowCount; i++)
            {
                DataRow row = this.activitiesDataTable.NewRow();
                long activityId = -1;
                if (this.fullTimesheet != null)
                {
                    try
                    {
                        activityId = this.fullTimesheet.TimesheetActivityEntries.GetActivityId(i);
                    }
                    catch
                    {
                    }
                }

                row["Id"] = Guid.NewGuid().ToString();
                row["ActivityId"] = activityId;
                row["ReadOnly"] = this.readOnly;
                row["Sat"] = this.GetActivityText(this.StartDate, activityId, false);
                row["Sun"] = this.GetActivityText(this.StartDate.AddDays(1), activityId, false);
                row["Mon"] = this.GetActivityText(this.StartDate.AddDays(2), activityId, false);
                row["Tue"] = this.GetActivityText(this.StartDate.AddDays(3), activityId, false);
                row["Wed"] = this.GetActivityText(this.StartDate.AddDays(4), activityId, false);
                row["Thur"] = this.GetActivityText(this.StartDate.AddDays(5), activityId, false);
                row["Fri"] = this.GetActivityText(this.StartDate.AddDays(6), activityId, false);
                row["Description"] = this.GetActivityDescription(activityId);

                row["SatCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                row["SunCssClass"] = CommonConstants.CssStyles.InOutWeekendTextboxClassName;
                row["MonCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["TueCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["WedCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["ThurCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;
                row["FriCssClass"] = CommonConstants.CssStyles.InOutTextboxClassName;

                this.activitiesDataTable.Rows.Add(row);
            }

            return this.activitiesDataTable;
        }

        /// <summary>
        /// Validates the activoty text.
        /// </summary>
        /// <param name="text">The text to validate. Returns the correct format of the text if the value is correct.</param>
        /// <returns>True if valid.</returns>
        private bool ValidateActivityText(ref string text)
        {
            bool result = true;

            try
            {
                if (!string.IsNullOrEmpty(text))
                {
                    decimal value = Convert.ToDecimal(text);

                    if (value < 0 || value > 24)
                    {
                        result = false;
                    }
                    else
                    {
                        // Change the text in the textbox to the nearest quarter of an hour
                        decimal min = Convert.ToDecimal(text);
                        /*min = min * 4;
                        min = Decimal.Round(min) / 4;*/
                        text = min.ToString("0.00");
                    }
                }
            }
            catch
            {
                result = false;
            }

            return result;
        }

        /// <summary>
        /// Validates the attendance text text.
        /// </summary>
        /// <param name="text">The text to validate. Returns the correct format of the text if the value is correct.</param>
        /// <returns>True if valid.</returns>
        private bool ValidateInOutText(ref string text)
        {
            bool result = true;
            try
            {
                if (!string.IsNullOrEmpty(text))
                {
                    if (text.Trim().Length != 4)
                    {
                        result = false;
                    }
                    else
                    {
                        int value = Convert.ToInt32(text);

                        if (value < 0 || value >= 2400)
                        {
                            result = false;
                        }
                        else
                        {
                            // Change the text in the textbox to the nearest 15 min
                            int min = Convert.ToInt32(text.Substring(2, 2));
                            min = ((int)Math.Round(min / 15.0)) * 15;
                            text = text.Substring(0, 2) + string.Format("{0:00}", min);
                        }
                    }
                }
            }
            catch
            {
                result = false;
            }

            return result;
        }

        /// <summary>
        /// Gets the activity text to show.
        /// </summary>
        /// <param name="date">The date of the attendance,</param>
        /// <param name="activityId">The activity Id.</param>
        /// <param name="description">The description.</param>
        /// <returns>The text to display.</returns>
        private string GetActivityText(DateTime date, long activityId, bool description)
        {
            try
            {
                TimesheetActivityEntry entry = this.fullTimesheet.TimesheetActivityEntries.GetEntry(date, activityId);
                if (entry != null)
                {
                    if (description)
                    {
                        return entry.Description;
                    }
                    else
                    {
                        return entry.Hours.ToString("0.00");
                    }
                }
            }
            catch
            {
            }

            return string.Empty;
        }

        /// <summary>
        /// Gets the description for the activity row.
        /// </summary>
        /// <param name="activityId">The activity id.</param>
        /// <returns>The description of the row.</returns>
        private string GetActivityDescription(long activityId)
        {
            try
            {
                DateTime date = this.StartDate;
                while (date <= this.EndDate)
                {
                    TimesheetActivityEntry entry = this.fullTimesheet.TimesheetActivityEntries.GetEntry(date, activityId);
                    if (entry != null)
                    {
                        return entry.Description;
                    }
                    else
                    {
                        date = date.AddDays(1);
                    }
                }
            }
            catch
            {
            }

            return string.Empty;
        }

        /// <summary>
        /// Gets an TimesheetAttendanceEntryCollection from the datatable.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="row">The row to handle.</param>
        /// <returns>The collection with entries in it.</returns>
        private TimesheetAttendanceEntryCollection GetAttendancesFromRow(DateTime startDate, DataRow row)
        {
            TimesheetAttendanceEntryCollection collection = new TimesheetAttendanceEntryCollection();
            DateTime? inDateTime = this.GetDateTimeFromAttendanceText(startDate, row["SatIn"].ToString());
            DateTime? outDateTime = this.GetDateTimeFromAttendanceText(startDate, row["SatOut"].ToString());
            if (inDateTime.HasValue && outDateTime.HasValue)
            {
                collection.Add(new TimesheetAttendanceEntry(inDateTime.Value, outDateTime.Value));
            }

            inDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(1), row["SunIn"].ToString());
            outDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(1), row["SunOut"].ToString());
            if (inDateTime.HasValue && outDateTime.HasValue)
            {
                collection.Add(new TimesheetAttendanceEntry(inDateTime.Value, outDateTime.Value));
            }

            inDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(2), row["MonIn"].ToString());
            outDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(2), row["MonOut"].ToString());
            if (inDateTime.HasValue && outDateTime.HasValue)
            {
                collection.Add(new TimesheetAttendanceEntry(inDateTime.Value, outDateTime.Value));
            }

            inDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(3), row["TueIn"].ToString());
            outDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(3), row["TueOut"].ToString());
            if (inDateTime.HasValue && outDateTime.HasValue)
            {
                collection.Add(new TimesheetAttendanceEntry(inDateTime.Value, outDateTime.Value));
            }

            inDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(4), row["WedIn"].ToString());
            outDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(4), row["WedOut"].ToString());
            if (inDateTime.HasValue && outDateTime.HasValue)
            {
                collection.Add(new TimesheetAttendanceEntry(inDateTime.Value, outDateTime.Value));
            }

            inDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(5), row["ThurIn"].ToString());
            outDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(5), row["ThurOut"].ToString());
            if (inDateTime.HasValue && outDateTime.HasValue)
            {
                collection.Add(new TimesheetAttendanceEntry(inDateTime.Value, outDateTime.Value));
            }

            inDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(6), row["FriIn"].ToString());
            outDateTime = this.GetDateTimeFromAttendanceText(startDate.AddDays(6), row["FriOut"].ToString());
            if (inDateTime.HasValue && outDateTime.HasValue)
            {
                collection.Add(new TimesheetAttendanceEntry(inDateTime.Value, outDateTime.Value));
            }

            return collection;
        }

        /// <summary>
        /// Gets an TimesheetActivityEntryCollection from the datatable.
        /// </summary>
        /// <param name="startDate">The start date.</param>
        /// <param name="row">The row to handle.</param>
        /// <returns>The collection with entries in it.</returns>
        private TimesheetActivityEntryCollection GetActivitiesFromRow(DateTime startDate, DataRow row)
        {
            TimesheetActivityEntryCollection collection = new TimesheetActivityEntryCollection();
            long activityId = Convert.ToInt64(row["ActivityId"]);
            string desc = Convert.ToString(row["Description"]);
            string val = row["Sat"].ToString();
            if (!string.IsNullOrEmpty(val))
            {
                collection.Add(new TimesheetActivityEntry(activityId, Convert.ToDecimal(val), desc, startDate.Date));
            }

            val = row["Sun"].ToString();
            if (!string.IsNullOrEmpty(val))
            {
                collection.Add(new TimesheetActivityEntry(activityId, Convert.ToDecimal(val), desc, startDate.Date.AddDays(1)));
            }

            val = row["Mon"].ToString();
            if (!string.IsNullOrEmpty(val))
            {
                collection.Add(new TimesheetActivityEntry(activityId, Convert.ToDecimal(val), desc, startDate.Date.AddDays(2)));
            }

            val = row["Tue"].ToString();
            if (!string.IsNullOrEmpty(val))
            {
                collection.Add(new TimesheetActivityEntry(activityId, Convert.ToDecimal(val), desc, startDate.Date.AddDays(3)));
            }

            val = row["Wed"].ToString();
            if (!string.IsNullOrEmpty(val))
            {
                collection.Add(new TimesheetActivityEntry(activityId, Convert.ToDecimal(val), desc, startDate.Date.AddDays(4)));
            }

            val = row["Thur"].ToString();
            if (!string.IsNullOrEmpty(val))
            {
                collection.Add(new TimesheetActivityEntry(activityId, Convert.ToDecimal(val), desc, startDate.Date.AddDays(5)));
            }

            val = row["Fri"].ToString();
            if (!string.IsNullOrEmpty(val))
            {
                collection.Add(new TimesheetActivityEntry(activityId, Convert.ToDecimal(val), desc, startDate.Date.AddDays(6)));
            }

            return collection;
        }

        /// <summary>
        /// Gets the datetime using the text.
        /// </summary>
        /// <param name="currDate">The current date.</param>
        /// <param name="text">The text with the time in it.</param>
        /// <returns>The datetime.</returns>
        private DateTime? GetDateTimeFromAttendanceText(DateTime currDate, string text)
        {
            try
            {
                int hours = Convert.ToInt32(text.Substring(0, 2));
                int mins = Convert.ToInt32(text.Substring(2, 2));
                return new DateTime(currDate.Year, currDate.Month, currDate.Day, hours, mins, 0);
            }
            catch
            {
                return null;
            }
        }

        #endregion Private Methods
    }
}