//-----------------------------------------------------------------------
// <copyright file="Global.asax.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents the Global Applitcation.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication
{
    using System;
    using System.Net.Mail;
    using System.Security.Principal;
    using System.Web;
    using System.Web.Security;
    using System.Web.UI;
    using BusinessLogicLayer;
    using BusinessObjects;
    using CommunicationManager;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class that represents the global application.
    /// </summary>
    public class Global : System.Web.HttpApplication
    {
        #region Private Static Members

        /// <summary>Holds the communication manager.</summary>
        private static CommunicationManager commsManager;

        #endregion Private Static Members

        #region Public Static Methods

        /// <summary>
        /// Sends the mail message using the CommunicationManager.
        /// </summary>
        /// <param name="mailMessage">Message to send.</param>
        public static void SendEmail(MailMessage mailMessage)
        {
            if (commsManager == null)
            {
                CommunicationManagerConfiguration config = new CommunicationManagerConfiguration();
                config.StartOnMessageAdd = true;
                commsManager = new CommunicationManager(config, null);
            }

            EmailMessage message = new EmailMessage(
                CommonConstants.Configuration.SmtpHost,
                CommonConstants.Configuration.SmtpPort,
                mailMessage,
                Message.MessagePriority.Medium);
            commsManager.AddMessage(message);
        }

        /// <summary>
        /// Creates a forms authentication ticket for a validated user so that the user can access
        /// the secure pages. The authentication timeout value is also set so the user will be logged out
        /// afetr a certain amount of time.
        /// </summary>
        /// <param name="page">The page creating the ticket.</param>
        /// <param name="userName">The name of the user to login</param>
        /// <param name="stayLoggedIn">Whether or not we want the store the user login details</param>
        /// <param name="timeMin">The time in minutes for when the users authentication session will timeout</param>
        /// <returns>True on success.</returns>
        public static bool CreateTicket(Page page, string userName, bool stayLoggedIn, int timeMin)
        {
            try
            {
                page.Session.Clear();
                page.Session.Abandon();
                FormsAuthentication.Initialize();
                User user = UserManager.Instance.GetUser(userName);
                string role = RoleManager.Instance.GetRole(user.RoleId).Name.ToLower();

                // create the Authentication ticket with timeout info
                FormsAuthenticationTicket ticket = new FormsAuthenticationTicket(
                    1,
                    userName,
                    DateTime.Now,
                    DateTime.Now.AddMinutes(timeMin),
                    stayLoggedIn,
                    role,
                    FormsAuthentication.FormsCookiePath);

                string hash = FormsAuthentication.Encrypt(ticket);
                HttpCookie cookie = new HttpCookie(FormsAuthentication.FormsCookieName, hash);
                if (ticket.IsPersistent)
                {
                    cookie.Expires = ticket.Expiration;
                }

                HttpContext.Current.Response.Cookies.Add(cookie);
                return true;
            }
            catch
            {
                FormsAuthentication.SignOut();
                page.Session.Abandon();
                return false;
            }
        }

        /// <summary>
        /// Gets the full url with domain name.
        /// </summary>
        /// <returns>The domain name.</returns>
        public static string GetDomain()
        {
            return "http://" + HttpContext.Current.Request.ServerVariables["HTTP_HOST"];
        }

        #endregion Public Static Methods

        #region Protected Methods

        /// <summary>
        /// Event Handler for Application_Start.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Application_Start(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Event Handler for Application_End.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Application_End(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Event Handler for Application_OnAuthenticateRequest.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Application_OnAuthenticateRequest(object sender, EventArgs e)
        {
            if (HttpContext.Current.User != null)
            {
                if (HttpContext.Current.User.Identity.IsAuthenticated)
                {
                    if (HttpContext.Current.User.Identity is FormsIdentity)
                    {
                        // Get Forms Identity From Current User
                        FormsIdentity id = (FormsIdentity)
                        HttpContext.Current.User.Identity;

                        // Get Forms Ticket From Identity object
                        FormsAuthenticationTicket ticket = id.Ticket;

                        // Retrieve stored user-data (role information is assigned when the ticket
                        // is created, separate multiple roles with commas)
                        string userData = ticket.UserData;
                        string[] roles = userData.Split(',');

                        // Create a new Generic Principal Instance and assign to Current User
                        HttpContext.Current.User = new GenericPrincipal(id, roles);
                    }
                }
            }
        }

        /// <summary>
        /// Event Handler for Application_Error.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Application_Error(object sender, EventArgs e)
        {
            Exception objErr = Server.GetLastError();
            MailMessage message = new MailMessage(
                CommonConstants.Configuration.AdminEmail,
                CommonConstants.Configuration.AppErrorEmail,
                "Exception",
                objErr.ToString());
            SendEmail(message);
        }

        #endregion Protected Methods
    }
}