//-----------------------------------------------------------------------
// <copyright file="CreateActivity.aspx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a CreateActivity page.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.Secure.Admin
{
    using System;
    using System.Web.UI.WebControls;  
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class used to represent the Create activity page.
    /// </summary>
    public partial class CreateActivity : AdminPage
    {
        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                this.Populate();
            }
        }

        /// <summary>
        /// Event Handler for CreateActivityButton Click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CreateActivityButton_Clicked(object sender, EventArgs e)
        {
            try
            {
                long activityId = ActivityManager.Instance.AddActivity(
                    Convert.ToInt64(this.JobCodeList.SelectedValue),
                    this.CodeTextBox.Text,
                    this.NameTextBox.Text);
                this.SuccessMessage(Resources.Main.ActivityCreated);
            }
            catch
            {
                this.AddErrorMessage(Resources.Main.ActivityCreationError);
            }
        }

        /// <summary>
        /// Event Handler for CancelButton Click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CancelButton_Clicked(object sender, EventArgs e)
        {
            Response.Redirect(CommonConstants.Urls.Secure.Admin.Default);
        }

        /// <summary>
        /// Event Handler for OrganisationList Index changed.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void OrganisationList_SelectedIndexChanged(object sender, EventArgs e)
        {
            int orgId = Convert.ToInt32(this.OrganisationList.SelectedValue);
            this.PopulateJobCodeList(orgId);
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the current page.
        /// </summary>
        private void Populate()
        {
            this.CodeTextBox.Text = string.Empty;
            this.NameTextBox.Text = string.Empty;
            if (this.IsUserAdministrator)
            {
                this.OrganisationSection.Visible = true;
                this.PopulateOrganisationList();
            }
            else
            {
                if (!this.IsUserInOrganisation)
                {
                    this.OrganisationSection.Visible = true;
                    this.PopulateOrganisationList();
                }
                else
                {
                    this.OrganisationSection.Visible = false;
                    this.PopulateJobCodeList(this.UserOrganisationId);
                }
            }
        }

        /// <summary>
        /// Populates the Organisation List.
        /// </summary>
        private void PopulateOrganisationList()
        {
            OrganisationCollection orgs = OrganisationManager.Instance.GetOrganisations(null);
            this.OrganisationList.Items.Clear();
            this.OrganisationList.Items.Add(BasePage.NullItem);
            foreach (Organisation org in orgs)
            {
                this.OrganisationList.Items.Add(new ListItem(org.Name, org.OrganisationId.ToString()));
            }

            this.JobCodeList.Items.Clear();
            this.JobCodeList.Items.Add(BasePage.NullItem);
        }

        /// <summary>
        /// Populates the JobCode List.
        /// </summary>
        /// <param name="organisationId">Id of the organisation.</param>
        private void PopulateJobCodeList(int organisationId)
        {
            JobCodeFilter filter = new JobCodeFilter();
            filter.OrganisationId = organisationId;
            JobCodeCollection jobCodes = JobCodeManager.Instance.GetJobCodes(filter);
            this.JobCodeList.Items.Clear();
            this.JobCodeList.Items.Add(BasePage.NullItem);
            foreach (JobCode jobCode in jobCodes)
            {
                this.JobCodeList.Items.Add(new ListItem(jobCode.Name, jobCode.JobCodeId.ToString()));
            }
        }

        #endregion Private Methods
    }
}
