//-----------------------------------------------------------------------
// <copyright file="CreateUser.aspx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a CreateUser page.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.Secure.Admin
{
    using System;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class used to represent the CreateUser page.
    /// </summary>
    public partial class CreateUser : AdminPage
    {
        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                this.Populate();
            }
        }

        /// <summary>
        /// Event Handler for CreateUserButton Click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CreateUserButton_Clicked(object sender, EventArgs e)
        {
            try
            {
                long userId = UserManager.Instance.AddUser(
                    this.EmployeeIdTextBox.Text,
                    Convert.ToByte(this.GenderList.SelectedValue),
                    Convert.ToInt16(this.RoleList.SelectedValue),
                    this.FirstnameTextBox.Text,
                    this.SurnameTextBox.Text,
                    this.EmailTextBox.Text,
                    this.OrganisationList.SelectedIndex <= 0 ? (int?)null : Convert.ToInt32(this.OrganisationList.SelectedValue),
                    this.ApproverList.SelectedIndex <= 0 ? (long?)null : Convert.ToInt64(this.ApproverList.SelectedValue));
                this.SuccessMessage(Resources.Main.UserCreated);
            }
            catch
            {
                this.AddErrorMessage(Resources.Main.UserCreationError);
            }
        }

        /// <summary>
        /// Event Handler for CancelButton Click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CancelButton_Clicked(object sender, EventArgs e)
        {
            Response.Redirect(CommonConstants.Urls.Secure.Admin.Default);
        }

        /// <summary>
        /// Event Handler for OrganisationLis index change.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void OrganisationList_IndexChanged(object sender, EventArgs e)
        {
            this.PopulateApproverList(Convert.ToInt32(this.OrganisationList.SelectedValue));
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the current page.
        /// </summary>
        private void Populate()
        {
            this.PopulateGenderList();
            if (this.IsUserAdministrator)
            {
                this.OrganisationSection.Visible = true;
                this.PopulateOrganisationList();
            }
            else
            {
                if (!this.IsUserInOrganisation)
                {
                    this.OrganisationSection.Visible = true;
                    this.PopulateOrganisationList();
                }
                else
                {
                    this.OrganisationSection.Visible = false;
                    this.PopulateApproverList(this.UserOrganisationId);
                }
            }

            /*if (this.IsUserInOrganisation)
            {
                this.OrganisationSection.Visible = false;
                this.PopulateApproverList(this.UserOrganisationId);
            }
            else
            {
                this.OrganisationSection.Visible = true;
                this.PopulateOrganisationList();
            }*/

            this.PopulateRoleList();
        }

        /// <summary>
        /// Populates the Organisation List.
        /// </summary>
        private void PopulateOrganisationList()
        {
            OrganisationCollection orgs = OrganisationManager.Instance.GetOrganisations(null);
            this.OrganisationList.Items.Clear();
            this.OrganisationList.Items.Add(BasePage.NullItem);
            foreach (Organisation org in orgs)
            {
                this.OrganisationList.Items.Add(new ListItem(org.Name, org.OrganisationId.ToString()));
            }

            this.ApproverList.Items.Clear();
            this.ApproverList.Items.Add(BasePage.NullItem);
        }

        /// <summary>
        /// Populates the role List.
        /// </summary>
        private void PopulateRoleList()
        {
            RoleCollection roles = RoleManager.Instance.GetRoles(null);
            this.RoleList.Items.Clear();
            this.RoleList.Items.Add(BasePage.NullItem);
            foreach (Role role in roles)
            {
                if (role.RoleId == Convert.ToInt16(BusinessLogicLayer.Utilities.CommonConstants.Roles.Administrator)
                    && this.IsUserManager)
                {
                    // Managers cannot create administrators. Therefore dont add min role to list.
                }
                else
                {
                    this.RoleList.Items.Add(new ListItem(role.Name, role.RoleId.ToString()));
                }
            }
        }

        /// <summary>
        /// Populates the gender List.
        /// </summary>
        private void PopulateGenderList()
        {
            GenderCollection genders = GenderManager.Instance.GetGenders(null);
            this.GenderList.Items.Clear();
            this.GenderList.Items.Add(BasePage.NullItem);
            foreach (Gender gender in genders)
            {
                this.GenderList.Items.Add(new ListItem(gender.Name, gender.GenderId.ToString()));
            }
        }

        /// <summary>
        /// Populates the Approver List.
        /// </summary>
        /// <param name="organisationId">Id of the organisation.</param>
        private void PopulateApproverList(int organisationId)
        {
            BusinessObjects.Filters.UserFilter filter = new BusinessObjects.Filters.UserFilter();
            filter.OrganisationId = organisationId;
            UserCollection users = UserManager.Instance.GetUsers(filter);
            this.ApproverList.Items.Clear();
            this.ApproverList.Items.Add(BasePage.NullItem);
            foreach (BusinessObjects.User user in users)
            {
                this.ApproverList.Items.Add(new ListItem(user.FullName, user.UserId.ToString()));
            }
        }

        #endregion Private Methods
    }
}
