//-----------------------------------------------------------------------
// <copyright file="EditUser.aspx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a EditUser page.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.Secure.Admin
{
    using System;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class used to represent the EditUser page.
    /// </summary>
    public partial class EditUser : AdminPage
    {
        #region Private Members

        /// <summary>
        /// Gets the selected user.
        /// </summary>
        private BusinessObjects.User selectedUser;

        #endregion Private Members

        #region Private Properties

        /// <summary>Gets the id of the selected user.</summary>
        private long SelectedUserId
        {
            get
            {
                QueryStringManager manager = new QueryStringManager(this.Request.QueryString);
                if (manager.HasRequest)
                {
                    QueryStringManager decrypted = new QueryStringManager(manager.Request);
                    if (decrypted.HasUserId)
                    {
                        return decrypted.UserId;
                    }
                }

                return -1;
            }
        }

        #endregion Private Properties

        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Check if user has been selected
            if (this.SelectedUserId < 0)
            {
                Response.Redirect(CommonConstants.Urls.Secure.Admin.Users);
                return;
            }

            this.selectedUser = BusinessLogicLayer.UserManager.Instance.GetUser(this.SelectedUserId);
            if (!this.IsPostBack)
            {
                this.Populate();
            }
        }

        /// <summary>
        /// Event Handler for UpdateUserButton Click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void UpdateUserButton_Clicked(object sender, EventArgs e)
        {
            try
            {
                this.selectedUser.ApproverUserId = this.ApproverList.SelectedIndex <= 0 ? (long?)null : Convert.ToInt64(this.ApproverList.SelectedValue);
                this.selectedUser.EmployeeId = this.EmployeeIdTextBox.Text;
                this.selectedUser.Email = this.EmailTextBox.Text;
                this.selectedUser.Firstname = this.FirstnameTextBox.Text;
                this.selectedUser.GenderId = Convert.ToByte(this.GenderList.SelectedValue);
                this.selectedUser.OrganisationId = this.OrganisationList.SelectedIndex <= 0 ? (int?)null : Convert.ToInt32(this.OrganisationList.SelectedValue);
                this.selectedUser.RoleId = Convert.ToInt16(this.RoleList.SelectedValue);
                this.selectedUser.Surname = this.SurnameTextBox.Text;
                
                UserManager.Instance.UpdateUser(this.selectedUser);
                this.SuccessMessage(Resources.Main.UserUpdated);
            }
            catch
            {
                this.AddErrorMessage(Resources.Main.UserUpdateError);
            }
        }

        /// <summary>
        /// Event Handler for CancelButton Click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CancelButton_Clicked(object sender, EventArgs e)
        {
            Response.Redirect(CommonConstants.Urls.Secure.Admin.Default);
        }

        /// <summary>
        /// Event Handler for OrganisationLis index change.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void OrganisationList_IndexChanged(object sender, EventArgs e)
        {
            this.PopulateApproverList(Convert.ToInt32(this.OrganisationList.SelectedValue));
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the current page.
        /// </summary>
        private void Populate()
        {
            SecurePage page = this as SecurePage;
            this.PopulateGenderList();
            if (page.IsUserAdministrator)
            {
                // User is admin. Admin users can do anything
                this.PopulateRoleList();
                this.PopulateOrganisationList();
            }
            else
            {
                // User is not admin. Is Manager then

                // Hide role
                this.RoleSection.Visible = false;

                // Only show org list if user not in org
                if (!page.IsUserInOrganisation)
                {
                    this.PopulateOrganisationList();
                }
                else
                {
                    this.OrganisationSection.Visible = false;
                }
            }

            if (this.selectedUser.OrganisationId.HasValue)
            {
                this.PopulateApproverList(this.selectedUser.OrganisationId.Value);
            }

            /*if (this.IsUserInOrganisation)
            {
                this.OrganisationSection.Visible = false;
            }
            else
            {
                this.OrganisationSection.Visible = true;
                this.PopulateOrganisationList();
            }

            if (this.selectedUser.OrganisationId.HasValue)
            {
                this.PopulateApproverList(this.selectedUser.OrganisationId.Value);
            }

            if (this.IsUserManager)
            {
                this.RoleSection.Visible = false;
            }
            else
            {
                this.RoleSection.Visible = true;
                this.PopulateRoleList();
            }*/

            this.EmployeeIdTextBox.Text = this.selectedUser.EmployeeId;
            this.FirstnameTextBox.Text = this.selectedUser.Firstname;
            this.SurnameTextBox.Text = this.selectedUser.Surname;
            this.EmailTextBox.Text = this.selectedUser.Email;
        }

        /// <summary>
        /// Populates the Organisation List.
        /// </summary>
        private void PopulateOrganisationList()
        {
            OrganisationCollection orgs = OrganisationManager.Instance.GetOrganisations(null);
            this.OrganisationList.Items.Clear();
            this.OrganisationList.Items.Add(BasePage.NullItem);
            foreach (Organisation org in orgs)
            {
                this.OrganisationList.Items.Add(new ListItem(org.Name, org.OrganisationId.ToString()));
            }

            if (this.selectedUser.OrganisationId.HasValue)
            {
                this.OrganisationList.SelectedValue = this.selectedUser.OrganisationId.Value.ToString();
            }

            this.ApproverList.Items.Clear();
            this.ApproverList.Items.Add(BasePage.NullItem);
        }

        /// <summary>
        /// Populates the role List.
        /// </summary>
        private void PopulateRoleList()
        {
            RoleCollection roles = RoleManager.Instance.GetRoles(null);
            this.RoleList.Items.Clear();
            this.RoleList.Items.Add(BasePage.NullItem);
            foreach (Role role in roles)
            {
                this.RoleList.Items.Add(new ListItem(role.Name, role.RoleId.ToString()));
            }

            this.RoleList.SelectedValue = this.selectedUser.RoleId.ToString();
        }

        /// <summary>
        /// Populates the gender List.
        /// </summary>
        private void PopulateGenderList()
        {
            GenderCollection genders = GenderManager.Instance.GetGenders(null);
            this.GenderList.Items.Clear();
            this.GenderList.Items.Add(BasePage.NullItem);
            foreach (Gender gender in genders)
            {
                this.GenderList.Items.Add(new ListItem(gender.Name, gender.GenderId.ToString()));
            }

            this.GenderList.SelectedValue = this.selectedUser.GenderId.ToString();
        }

        /// <summary>
        /// Populates the Approver List.
        /// </summary>
        /// <param name="organisationId">Id of the organisation.</param>
        private void PopulateApproverList(int organisationId)
        {
            BusinessObjects.Filters.UserFilter filter = new BusinessObjects.Filters.UserFilter();
            filter.OrganisationId = organisationId;
            UserCollection users = UserManager.Instance.GetUsers(filter);
            this.ApproverList.Items.Clear();
            this.ApproverList.Items.Add(BasePage.NullItem);
            foreach (BusinessObjects.User user in users)
            {
                this.ApproverList.Items.Add(new ListItem(user.FullName, user.UserId.ToString()));
            }

            if (this.selectedUser.ApproverUserId.HasValue)
            {
                this.ApproverList.SelectedValue = this.selectedUser.ApproverUserId.Value.ToString();
            }
        }

        #endregion Private Methods
    }
}
