//-----------------------------------------------------------------------
// <copyright file="JobCodes.aspx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a JobCodes page.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.Secure.Admin
{
    using System;
    using System.Data;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects.Collections;
    using TimesheetWebApplication.App_Code;
    using TimesheetWebApplication.WebUserControls;

    /// <summary>
    /// Class used to represent the JobCodes page.
    /// </summary>
    public partial class JobCodes : AdminPage
    {
        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                if (this.IsUserInOrganisation && !this.IsUserAdministrator)
                {
                    // Non admin user who is in org
                    this.Filter.Visible = false;
                    this.PopulateJobCodesRepeater();
                }
                else
                {
                    if (this.QueryStringManager.HasRequest)
                    {
                        QueryStringManager unencryptedManager = new QueryStringManager(this.QueryStringManager.Request);
                        if (unencryptedManager.HasOrganisationId)
                        {
                            this.JobCodeFilterWebUserControl.OrganisationId = unencryptedManager.OrganisationId;
                            this.PopulateJobCodesRepeater();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Event Handler for on SearchButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void SearchButton_Clicked(object sender, EventArgs e)
        {
            this.PopulateJobCodesRepeater();
        }

        /// <summary>
        /// Event Handler for on DeleteJobCode click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void DeleteJobCode_Clicked(object sender, EventArgs e)
        {
            try
            {
                long id = Convert.ToInt64((sender as LinkButton).CommandArgument);
                BusinessLogicLayer.JobCodeManager.Instance.DeleteJobCode(id);
                this.PopulateJobCodesRepeater();
            }
            catch
            {
                this.AddErrorMessage(Resources.Main.JobCodeDeletionError);
            }
        }

        /// <summary>
        /// Event Handler for on DisableJobCode click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void DisableJobCode_Clicked(object sender, EventArgs e)
        {
            try
            {
                long id = Convert.ToInt64((sender as LinkButton).CommandArgument);
                BusinessLogicLayer.JobCodeManager.Instance.DisableJobCode(id);
                this.PopulateJobCodesRepeater();
            }
            catch
            {
            }
        }

        /// <summary>
        /// Event Handler for on EnableJobCode click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void EnableJobCode_Clicked(object sender, EventArgs e)
        {
            try
            {
                long id = Convert.ToInt64((sender as LinkButton).CommandArgument);
                BusinessLogicLayer.JobCodeManager.Instance.EnableJobCode(id);
                this.PopulateJobCodesRepeater();
            }
            catch
            {
            }
        }

        /// <summary>
        /// Event Handler for on CancelButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CancelButton_Clicked(object sender, EventArgs e)
        {
            Response.Redirect(CommonConstants.Urls.Secure.Admin.Default);
        }

        /// <summary>
        /// Event Handler for on CreateJobCodeButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CreateJobCodeButton_Clicked(object sender, EventArgs e)
        {
            Response.Redirect(CommonConstants.Urls.Secure.Admin.CreateJobCode);
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the Timesheets Tab Panel.
        /// </summary>
        private void PopulateJobCodesRepeater()
        {
            this.JobCodesRepeater.DataSource = this.CreateJobCodesDataTable();
            this.JobCodesRepeater.DataBind();
        }

        /// <summary>
        /// Creates a datatable for the timesheets repeater.
        /// </summary>
        /// <returns>Table with activity data.</returns>
        private DataTable CreateJobCodesDataTable()
        {
            DataTable result = new DataTable();
            result.Columns.Add("Job Code Id", typeof(int));
            result.Columns.Add("Organisation", typeof(string));
            result.Columns.Add("Code", typeof(string));
            result.Columns.Add("Name", typeof(string));
            result.Columns.Add("Creation Date", typeof(DateTime));
            result.Columns.Add("Disabled Date", typeof(DateTime));
            result.Columns.Add("Enabled", typeof(bool));

            BusinessObjects.Filters.JobCodeFilter filter = new BusinessObjects.Filters.JobCodeFilter();
            filter.OrganisationId = this.JobCodeFilterWebUserControl.OrganisationId;
            JobCodeCollection jobCodes = JobCodeManager.Instance.GetJobCodes(filter);

            foreach (BusinessObjects.JobCode jobCode in jobCodes)
            {
                DataRow row = result.NewRow();
                row["Job Code Id"] = jobCode.JobCodeId;
                row["Organisation"] = OrganisationManager.Instance.GetOrganisation(jobCode.OrganisationId).Name;
                row["Code"] = jobCode.Code;
                row["Name"] = jobCode.Name;
                row["Creation Date"] = jobCode.CreationDateTime;
                row["Enabled"] = !jobCode.DisabledDateTime.HasValue;
                if (jobCode.DisabledDateTime.HasValue)
                {
                    row["Disabled Date"] = jobCode.DisabledDateTime.Value;
                }

                result.Rows.Add(row);
            }

            return result;
        }

        #endregion Private Methods
    }
}
