//-----------------------------------------------------------------------
// <copyright file="Organisations.aspx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a Organisations page.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.Secure.Admin
{
    using System;
    using System.Data;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects.Collections;
    using TimesheetWebApplication.App_Code;
    using TimesheetWebApplication.WebUserControls;

    /// <summary>
    /// Class used to represent the Organisations page.
    /// </summary>
    public partial class Organisations : AdminPage
    {
        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                this.PopulateOrganisationsRepeater();
            }
        }

        /// <summary>
        /// Event Handler for on DeleteOrganisation click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void DeleteOrganisation_Clicked(object sender, EventArgs e)
        {
            try
            {
                int id = Convert.ToInt32((sender as LinkButton).CommandArgument);
                BusinessLogicLayer.OrganisationManager.Instance.DeleteOrganisation(id);
                this.PopulateOrganisationsRepeater();
            }
            catch
            {
                this.AddErrorMessage(Resources.Main.OrganisationDeletionError);
            }
        }

        /// <summary>
        /// Event Handler for on JobCodes click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void JobCodes_Clicked(object sender, EventArgs e)
        {
            try
            {
                int id = Convert.ToInt32((sender as LinkButton).CommandArgument);
                TimesheetWebApplication.App_Code.QueryStringManager manager = new QueryStringManager();
                manager.Add(CommonConstants.QueryStringVariables.OrganisationId, id.ToString());
                string url = TimesheetWebApplication.App_Code.QueryStringManager.AddValueToUrl(
                    CommonConstants.Urls.Secure.Admin.JobCodes,
                    CommonConstants.QueryStringVariables.Request,
                    manager.Encrypt());
                Response.Redirect(url);
            }
            catch
            {
            }
        }

        /// <summary>
        /// Event Handler for on DisableOrganisation click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void DisableOrganisation_Clicked(object sender, EventArgs e)
        {
            try
            {
                int id = Convert.ToInt32((sender as LinkButton).CommandArgument);
                BusinessLogicLayer.OrganisationManager.Instance.DisableOrganisation(id);
                this.PopulateOrganisationsRepeater();
            }
            catch
            {
            }
        }

        /// <summary>
        /// Event Handler for on EnableOrganisation click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void EnableOrganisation_Clicked(object sender, EventArgs e)
        {
            try
            {
                int id = Convert.ToInt32((sender as LinkButton).CommandArgument);
                BusinessLogicLayer.OrganisationManager.Instance.EnableOrganisation(id);
                this.PopulateOrganisationsRepeater();
            }
            catch
            {
            }
        }

        /// <summary>
        /// Event Handler for on CancelButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CancelButton_Clicked(object sender, EventArgs e)
        {
            Response.Redirect(CommonConstants.Urls.Secure.Admin.Default);
        }

        /// <summary>
        /// Event Handler for on CreateOrganisationButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CreateOrganisationButton_Clicked(object sender, EventArgs e)
        {
            Response.Redirect(CommonConstants.Urls.Secure.Admin.CreateOrganisation);
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the Organisations repeater.
        /// </summary>
        private void PopulateOrganisationsRepeater()
        {
            this.OrganisationsRepeater.DataSource = this.CreateOrganisationsDataTable();
            this.OrganisationsRepeater.DataBind();
        }

        /// <summary>
        /// Creates a datatable for the Organisations repeater.
        /// </summary>
        /// <returns>Table with Organisations data.</returns>
        private DataTable CreateOrganisationsDataTable()
        {
            DataTable result = new DataTable();
            result.Columns.Add("Organisation Id", typeof(long));
            result.Columns.Add("Name", typeof(string));
            result.Columns.Add("Creation Date", typeof(DateTime));
            result.Columns.Add("Disabled Date", typeof(DateTime));
            result.Columns.Add("Enabled", typeof(bool));

            OrganisationCollection organisations = OrganisationManager.Instance.GetOrganisations(null);
            foreach (BusinessObjects.Organisation organisation in organisations)
            {
                DataRow row = result.NewRow();
                row["Organisation Id"] = organisation.OrganisationId;
                row["Name"] = organisation.Name;
                row["Creation Date"] = organisation.CreationDateTime;
                row["Enabled"] = !organisation.DisabledDateTime.HasValue;
                if (organisation.DisabledDateTime.HasValue)
                {
                    row["Disabled Date"] = organisation.DisabledDateTime.Value;
                }

                result.Rows.Add(row);
            }

            return result;
        }

        #endregion Private Methods
    }
}
