//-----------------------------------------------------------------------
// <copyright file="Timesheet.aspx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a Timesheet.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.Secure
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Web.UI;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class that represents timesheet page.
    /// </summary>
    public partial class Timesheet : SecurePage
    {
        #region Private Members

        /// <summary>Id of the current timesheet, if one has been selected.</summary>
        private long? timesheetId = null;

        #endregion Private Members

        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!this.IsPostBack)
            {
                this.PopulatePage();
            }
            else
            {
                this.ViewStateManager.TimesheetTable.SaveAttendancesRepeaterData(this.AttendancesRepeater);
                this.ViewStateManager.TimesheetTable.SaveActivitiesRepeaterData(this.ActivitiesRepeater);
                this.PopulateAttendanceSumFields();
                this.PopulateActivitySumFields();
            }
        }

        /// <summary>
        /// Event Handler for on pre Render.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);
        }

        /// <summary>
        /// Event Handler for SubmitButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void SubmitButton_Clicked(object sender, EventArgs e)
        {
            // Make sure list item is selected
            if (this.TimesheetStatusList.SelectedIndex <= 0)
            {
                return;
            }

            // Get the selected action from the TimesheetStatusList
            BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions action = (BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions)Enum.Parse(
                typeof(BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions),
                this.TimesheetStatusList.SelectedValue);

            // Variables that will hold the values retreived from getting timesheet from table.
            BusinessObjects.Timesheet timesheet = null;
            TimesheetAttendanceEntryCollection attendances = null;
            TimesheetActivityEntryCollection activities = null;

            // Check to see if user is approver
            if (this.ViewStateManager.TimesheetTable.Timesheet != null
                && this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.TimesheetId > 0
                && TimesheetManager.Instance.IsTimesheetApprover(this.UserId, this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.TimesheetId))
            {
                // Approver can only set the status of the timesheet
                BusinessLogicLayer.TimesheetManager.Instance.SetTimesheetStatus(
                    this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.TimesheetId,
                    this.UserId,
                    Convert.ToInt16(action));

                // Send email to interested parties.
                MailManager.SendTimesheetStatusChangedMessage(this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.TimesheetId, this);

                // Navigate to the new timesheet
                QueryStringManager manager = new QueryStringManager();
                manager.Add(CommonConstants.QueryStringVariables.TimesheetId, this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.TimesheetId.ToString());
                Response.Redirect(QueryStringManager.AddValueToUrl(
                    CommonConstants.Urls.Secure.Timesheet,
                    CommonConstants.QueryStringVariables.Request,
                    manager.Encrypt()));
            }
            else
            {
                // Create timesheet business objects from the html timesheet table.
                bool result = this.ViewStateManager.TimesheetTable.GetTimesheet(
                    SessionManager.UserId, 
                    out timesheet, 
                    out attendances, 
                    out activities);
                if (!result)
                {
                    this.AddErrorMessage(Resources.Main.AtleastOneAttendanceAndActivityMustBeEntered);
                    return;
                }
                else
                {
                    // Save the timesheet.
                    long timesheetId = BusinessLogicLayer.TimesheetManager.Instance.SaveTimesheet((short)action, timesheet, attendances, activities);

                    if (timesheetId >= 0)
                    {
                        // Send email to approver telling them that the timesheet has been submitted.
                        MailManager.SendTimesheetStatusChangedMessage(timesheetId, this);

                        // Naigate to new timesheet
                        QueryStringManager manager = new QueryStringManager();
                        manager.Add(CommonConstants.QueryStringVariables.TimesheetId, timesheetId.ToString());
                        string url = QueryStringManager.AddValueToUrl(
                            CommonConstants.Urls.Secure.Timesheet,
                            CommonConstants.QueryStringVariables.Request,
                            manager.Encrypt());
                        Response.Redirect(url);
                    }
                    else
                    {
                        this.AddErrorMessage(Resources.Main.FailedToSubmitTimesheet);
                    }
                }
            }
        }

        /// <summary>
        /// Event Handler for AddAttendanceRowButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void AddAttendanceRowButton_Clicked(object sender, EventArgs e)
        {
            Button button = sender as Button;
            this.ViewStateManager.TimesheetTable.AddAttendanceRow(button.CommandArgument);
            this.PopulateAttendancesRepeater();
            this.PopulateAttendanceSumFields();
            this.PopulateActivitySumFields();
        }

        /// <summary>
        /// Event Handler for RemoveAttendanceRowButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void RemoveAttendanceRowButton_Clicked(object sender, EventArgs e)
        {
            Button button = sender as Button;
            this.ViewStateManager.TimesheetTable.RemoveAttendanceRow(button.CommandArgument);
            this.PopulateAttendancesRepeater();
            this.PopulateAttendanceSumFields();
            this.PopulateActivitySumFields();
        }

        /// <summary>
        /// Event Handler for BackButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void BackButton_Clicked(object sender, EventArgs e)
        {
            DateTime prevWeek = ViewStateManager.TimesheetTable.StartDate.AddDays(-1);
            BusinessObjects.Timesheet timesheet = TimesheetManager.Instance.GetTimesheet(this.UserId, Utilities.GetWeekEnd(prevWeek));
            if (timesheet == null)
            {
                // Create new blank timesheet
                this.ViewStateManager.TimesheetTable = new TimesheetTable(this.UserId, Utilities.GetWeekStart(prevWeek));
                this.DateLabel.Text = this.ViewStateManager.TimesheetTable.EndDate.ToString(CommonConstants.Configuration.DateFormat);
                this.OwnerDiv.Visible = false;
                this.CurrentStatusDiv.Visible = false;
                this.LastChangedDiv.Visible = false;
                this.ButtonsDiv.Visible = true;

                this.ViewStateManager.TimesheetTable.Clear();
                this.PopulateTimesheetStatusList();
                this.PopulateAttendancesRepeater();
                this.PopulateActivitiesRepeater();
                this.PopulateAttendanceSumFields();
                this.PopulateActivitySumFields();
            }
            else
            {
                // Naigate to existing timesheet
                QueryStringManager manager = new QueryStringManager();
                manager.Add(CommonConstants.QueryStringVariables.TimesheetId, timesheet.TimesheetId.ToString());
                Response.Redirect(QueryStringManager.AddValueToUrl(
                    CommonConstants.Urls.Secure.Timesheet,
                    CommonConstants.QueryStringVariables.Request,
                    manager.Encrypt()));
            }
        }

        /// <summary>
        /// Event Handler for ForwardButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void ForwardButton_Clicked(object sender, EventArgs e)
        {
            DateTime nextWeek = ViewStateManager.TimesheetTable.EndDate.AddDays(1);
            BusinessObjects.Timesheet timesheet = TimesheetManager.Instance.GetTimesheet(this.UserId, Utilities.GetWeekEnd(nextWeek));
            if (timesheet == null)
            {
                // Create new blank timesheet
                this.ViewStateManager.TimesheetTable = new TimesheetTable(this.UserId, Utilities.GetWeekStart(nextWeek));
                this.DateLabel.Text = this.ViewStateManager.TimesheetTable.EndDate.ToString(CommonConstants.Configuration.DateFormat);
                this.OwnerDiv.Visible = false;
                this.CurrentStatusDiv.Visible = false;
                this.LastChangedDiv.Visible = false;

                this.ViewStateManager.TimesheetTable.Clear();
                this.PopulateTimesheetStatusList();
                this.PopulateAttendancesRepeater();
                this.PopulateActivitiesRepeater();
                this.PopulateAttendanceSumFields();
                this.PopulateActivitySumFields();
            }
            else
            {
                // Navigate to existing timesheet
                QueryStringManager manager = new QueryStringManager();
                manager.Add(CommonConstants.QueryStringVariables.TimesheetId, timesheet.TimesheetId.ToString());
                Response.Redirect(QueryStringManager.AddValueToUrl(
                    CommonConstants.Urls.Secure.Timesheet,
                    CommonConstants.QueryStringVariables.Request,
                    manager.Encrypt()));
            }
        }

        /// <summary>
        /// Event Handler for AddActivityRowButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void AddActivityRowButton_Clicked(object sender, EventArgs e)
        {
            Button button = sender as Button;
            this.ViewStateManager.TimesheetTable.AddActivityRow(button.CommandArgument);
            this.PopulateActivitiesRepeater();
            this.PopulateAttendanceSumFields();
            this.PopulateActivitySumFields();
        }

        /// <summary>
        /// Event Handler for RemoveActivityRowButton click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void RemoveActivityRowButton_Clicked(object sender, EventArgs e)
        {
            Button button = sender as Button;
            this.ViewStateManager.TimesheetTable.RemoveActivityRow(button.CommandArgument);
            this.PopulateActivitiesRepeater();
            this.PopulateAttendanceSumFields();
            this.PopulateActivitySumFields();
        }

        /// <summary>
        /// Event Handler for JobCodeList SelectedIndexChanged.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void JobCodeList_SelectedIndexChanged(object sender, EventArgs e)
        {
            DropDownList jobCodeList = sender as DropDownList;
            RepeaterItem item = jobCodeList.NamingContainer as RepeaterItem;
            Label idLabel = item.FindControl("IdLabel") as Label;
            this.ViewStateManager.TimesheetTable.SetActivityId(idLabel.Text, Convert.ToInt64(jobCodeList.SelectedValue));
            this.PopulateActivitiesRepeater();
            this.PopulateAttendanceSumFields();
            this.PopulateActivitySumFields();
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the page.
        /// </summary>
        private void PopulatePage()
        {
            if (this.IsUserInOrganisation)
            {
                // Create the Timesheet table
                if (this.CheckForTimesheet())
                {
                    this.PopulateAttendancesRepeater();
                    this.PopulateActivitiesRepeater();
                    this.PopulateAttendanceSumFields();
                    this.PopulateActivitySumFields();
                    this.PopulateFieldsAndList();
                }
            }
            else
            {
                // User not in organisation so usr can't have timesheet
                this.TimesheetSection.Visible = false;
                this.NoOrganisationSection.Visible = true;
            }
        }

        /// <summary>
        /// Checks to see if a timesheet is selected.
        /// </summary>
        /// <returns>True if there is a timesheet selected.</returns>
        private bool CheckForTimesheet()
        {
            // only users in an organisation are allowed to have timesheets
            this.TimesheetSection.Visible = true;
            this.NoOrganisationSection.Visible = false;

            // Check if viewstate already has timesheet
            if (!this.ViewStateManager.HasTimesheetTable)
            {
                // Viewtate doesn't have timesheet, check querystring
                if (this.QueryStringManager.HasRequest)
                {
                    // Query string has request. Need to look for timsheet.
                    try
                    {
                        QueryStringManager manager = new QueryStringManager(this.QueryStringManager.Request);
                        if (manager.HasTimesheetId)
                        {
                            // Timesheet id found, create timesheet from database
                            if (TimesheetManager.Instance.CanAccessTimesheet(this.UserId, manager.TimesheetId))
                            {
                                // User is allowed to view this timesheet but may have read only access  
                                bool readOnly = TimesheetManager.Instance.IsTimesheetReadOnly(this.UserId, manager.TimesheetId);
                                if (TimesheetManager.Instance.IsTimesheetOwner(this.UserId, manager.TimesheetId) && readOnly)
                                {
                                    this.ButtonsDiv.Visible = false;
                                }

                                // If user is approver then stopfrom going backwards and forwards.
                                if (TimesheetManager.Instance.IsTimesheetApprover(this.UserId, manager.TimesheetId))
                                {
                                    this.BackButton.Visible = false;
                                    this.ForwardButton.Visible = false;
                                }

                                // Get the full timesheet
                                FullTimesheet fullTimesheet = TimesheetManager.Instance.GetFullTimesheet(manager.TimesheetId);
                                this.ViewStateManager.TimesheetTable = new TimesheetTable(fullTimesheet, readOnly);
                                this.timesheetId = manager.TimesheetId;
                            }
                            else
                            {
                                // Not allowed to view the timesheet
                                this.TimesheetSection.Visible = false;
                                this.NoOrganisationSection.Visible = false;
                                return false;
                            }
                        }
                        else
                        {
                            // Unknown request
                            Response.Redirect(CommonConstants.Urls.Secure.Timesheets);
                            return false;
                        }
                    }
                    catch
                    {
                        // Unknown request
                        Response.Redirect(CommonConstants.Urls.Error);
                        return false;
                    }
                }
                else
                {
                    // Nothing in querystring
                    BusinessObjects.Timesheet timesheet = TimesheetManager.Instance.GetTimesheet(this.UserId, Utilities.GetWeekEnd(DateTime.Now));
                    if (timesheet == null)
                    {
                        // Create new blank timesheet
                        this.ViewStateManager.TimesheetTable = new TimesheetTable(this.UserId, Utilities.GetWeekStart(DateTime.Now));
                    }
                    else
                    {
                        // Naigate to existing timesheet
                        QueryStringManager manager = new QueryStringManager();
                        manager.Add(CommonConstants.QueryStringVariables.TimesheetId, timesheet.TimesheetId.ToString());
                        Response.Redirect(QueryStringManager.AddValueToUrl(
                            CommonConstants.Urls.Secure.Timesheet,
                            CommonConstants.QueryStringVariables.Request,
                            manager.Encrypt()));
                        return false;
                    }
                }
            }

            return true;
        }

        /// <summary>
        /// Populates the Populate Attendances Repeater.
        /// </summary>
        private void PopulateAttendancesRepeater()
        {
            this.AttendancesRepeater.DataSource = this.ViewStateManager.TimesheetTable.AttendanceDataTable;
            this.AttendancesRepeater.DataBind();
        }

        /// <summary>
        /// Populates the Populate Activities Repeater.
        /// </summary>
        private void PopulateActivitiesRepeater()
        {
            this.ActivitiesRepeater.DataSource = this.ViewStateManager.TimesheetTable.ActivitiesDataTable;
            this.ActivitiesRepeater.DataBind();

            foreach (RepeaterItem item in this.ActivitiesRepeater.Items)
            {
                Label idLabel = item.FindControl("IdLabel") as Label; 
                DropDownList jobCodeList = item.FindControl("JobCodeList") as DropDownList;
                DropDownList activityList = item.FindControl("ActivityList") as DropDownList;
                long activityId = this.ViewStateManager.TimesheetTable.GetActivityId(idLabel.Text);
                this.ViewStateManager.TimesheetTable.PopulateJobCodeList(jobCodeList, activityId);
                this.ViewStateManager.TimesheetTable.PopulateActivityList(activityList, Convert.ToInt64(jobCodeList.SelectedValue));
                activityList.SelectedValue = activityId.ToString();
            }
        }

        /// <summary>
        /// Populates TimesheetStatus List.
        /// </summary>
        private void PopulateTimesheetStatusList()
        {
            TimesheetStatusCollection statuses = TimesheetManager.Instance.GetValidTimesheetStatues(this.UserId, this.timesheetId);
            if (statuses.Count > 0)
            {
                this.ButtonsDiv.Visible = true;

                // populate the list
                this.TimesheetStatusList.Items.Clear();
                this.TimesheetStatusList.Items.Add(NullItem);
                foreach (TimesheetStatus status in statuses)
                {
                    BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions action
                        = (BusinessLogicLayer.Utilities.CommonConstants.TimesheetActions)status.TimesheetStatusId;
                    this.TimesheetStatusList.Items.Add(new ListItem(action.ToString(), action.ToString()));
                }
            }
            else
            {
                // Hide the status list section as the user can't change the status to anything
                this.ButtonsDiv.Visible = false;
            }
        }

        /// <summary>
        /// Populates the Attendance Sum Row.
        /// </summary>
        private void PopulateAttendanceSumFields()
        {
            decimal total = 0;
            decimal sum = 0;
            sum = this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Saturday);
            total += sum;
            this.SatAttendanceSum.InnerText = sum.ToString();
            this.SatAttendanceSum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Saturday))
            {
                this.SatAttendanceSum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Sunday);
            total += sum;
            this.SunAttendanceSum.InnerText = sum.ToString();
            this.SunAttendanceSum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Sunday))
            {
                this.SunAttendanceSum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Monday);
            total += sum;
            this.MonAttendanceSum.InnerText = sum.ToString();
            this.MonAttendanceSum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Monday))
            {
                this.MonAttendanceSum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Tuesday);
            total += sum;
            this.TueAttendanceSum.InnerText = sum.ToString();
            this.TueAttendanceSum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Tuesday))
            {
                this.TueAttendanceSum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Wednesday);
            total += sum;
            this.WedAttendanceSum.InnerText = sum.ToString();
            this.WedAttendanceSum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Wednesday))
            {
                this.WedAttendanceSum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Thursday);
            total += sum;
            this.ThurAttendanceSum.InnerText = sum.ToString();
            this.ThurAttendanceSum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Thursday))
            {
                this.ThurAttendanceSum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Friday);
            total += sum;
            this.FriAttendanceSum.InnerText = sum.ToString();
            this.FriAttendanceSum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Friday))
            {
                this.FriAttendanceSum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            this.TotalAttendanceSum.InnerText = total.ToString();
        }

        /// <summary>
        /// Populates the activity sum row.
        /// </summary>
        private void PopulateActivitySumFields()
        {
            decimal total = 0;
            decimal sum = 0;
            sum = this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Saturday);
            total += sum;
            this.SatActivitySum.InnerText = sum.ToString();
            this.SatActivitySum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Saturday))
            {
                this.SatActivitySum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Sunday);
            total += sum;
            this.SunActivitySum.InnerText = sum.ToString();
            this.SatActivitySum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Sunday))
            {
                this.SunActivitySum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Monday);
            total += sum;
            this.MonActivitySum.InnerText = sum.ToString();
            this.MonActivitySum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Monday))
            {
                this.MonActivitySum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Tuesday);
            total += sum;
            this.TueActivitySum.InnerText = sum.ToString();
            this.TueActivitySum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Tuesday))
            {
                this.TueActivitySum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Wednesday);
            total += sum;
            this.WedActivitySum.InnerText = sum.ToString();
            this.WedActivitySum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Wednesday))
            {
                this.WedActivitySum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Thursday);
            total += sum;
            this.ThurActivitySum.InnerText = sum.ToString();
            this.ThurActivitySum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Thursday))
            {
                this.ThurActivitySum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            sum = this.ViewStateManager.TimesheetTable.GetActivitySum(DayOfWeek.Friday);
            total += sum;
            this.FriActivitySum.InnerText = sum.ToString();
            this.FriActivitySum.Attributes["class"] = string.Empty;
            if (sum != this.ViewStateManager.TimesheetTable.GetAttendanceSum(DayOfWeek.Friday))
            {
                this.FriActivitySum.Attributes["class"] = CommonConstants.CssStyles.SumErrorClassName;
            }

            this.TotalActivitySum.InnerText = total.ToString();
        }

        /// <summary>
        /// Populates the data that is not within the timesheet.
        /// </summary>
        private void PopulateFieldsAndList()
        {
            this.PopulateTimesheetStatusList();
            this.DateLabel.Text = this.ViewStateManager.TimesheetTable.EndDate.ToString(CommonConstants.Configuration.DateFormat);

            // Populate the timesheets info
            if (this.ViewStateManager.TimesheetTable.Timesheet != null)
            {
                this.OwnerLabel.Text = UserManager.Instance.GetUser(this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.UserId).FullName;
                this.CurrentStatusLabel.Text = ((BusinessLogicLayer.Utilities.CommonConstants.TimesheetStatuses)this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.TimesheetStatusId).ToString();
                this.LastChangedLabel.Text = this.ViewStateManager.TimesheetTable.Timesheet.TimeSheet.StatusChangeDateTime.ToString();
            }
            else
            {
                this.OwnerDiv.Visible = false;
                this.CurrentStatusDiv.Visible = false;
                this.LastChangedDiv.Visible = false;
            }
        }

        #endregion Private Methods
    }
}
