//-----------------------------------------------------------------------
// <copyright file="ActivityFilter.ascx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a ActivityFilter control.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.WebUserControls
{
    using System;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using BusinessObjects.Filters;
    using TimesheetWebApplication.App_Code;
    
    /// <summary>
    /// Class that represents ActivityFilter control. Displays the organisation list depending on whether the 
    /// user has access rights to select an organisation.
    /// </summary>
    public partial class ActivityFilter : BaseUserControl
    {
        #region Public Properties

        /// <summary>Gets or sets controls style.</summary>
        public string CssClass
        {
            get { return this.FilterDiv.Attributes["class"]; }
            set { this.FilterDiv.Attributes["class"] = value; }
        }

        /// <summary>
        /// Gets the selected job code from the filter.
        /// </summary>
        public long? JobCodeId
        {
            get
            {
                if (this.JobCodeList.SelectedIndex <= 0)
                {
                    return null;
                }

                return Convert.ToInt64(this.JobCodeList.SelectedValue);
            }
        }

        /// <summary>Gets or sets the validation group.</summary>
        public string ValidationGroup
        {
            get 
            {
                return this.OrganisationRequiredFieldValidator.ValidationGroup; 
            }

            set 
            {
                this.OrganisationRequiredFieldValidator.ValidationGroup = value;
                this.JobCodeRequiredFieldValidator.ValidationGroup = value;
            }
        }

        #endregion Public Properties

        #region Protected Methods

        /// <summary>
        /// Event Handler for on OnInit.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (!this.IsPostBack)
            {
                this.PopulateControl();
            }
        }

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// Event Handler for OrganisationList index changed.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void OrganisationList_IndexChanged(object sender, EventArgs e)
        {
            this.PopulateJobCodeList(Convert.ToInt32(this.OrganisationList.SelectedValue));
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the control.
        /// </summary>
        private void PopulateControl()
        {
            SecurePage securePage = this.Page as SecurePage;
            if (securePage.IsUserAdministrator)
            {
                this.OrganisationDiv.Visible = true;
                this.PopulateOrganisationList();
            }
            else
            {
                if (!securePage.IsUserInOrganisation)
                {
                    this.OrganisationDiv.Visible = true;
                    this.PopulateOrganisationList();
                }
                else
                {
                    // Don't show org
                    this.OrganisationDiv.Visible = false;
                    this.PopulateJobCodeList(securePage.UserOrganisationId);
                }
            }

            /*User user = UserManager.Instance.GetUser((this.Page as SecurePage).UserId);
            if (!user.OrganisationId.HasValue)
            {
                // Show the org list because user is not part of an organisation
                this.OrganisationDiv.Visible = true;
                this.PopulateOrganisationList();
            }
            else
            {
                // Don't show org
                this.OrganisationDiv.Visible = false;
                this.PopulateJobCodeList(user.OrganisationId.Value);
            }*/
        }

        /// <summary>
        /// Populates the Organisation List.
        /// </summary>
        private void PopulateOrganisationList()
        {
            OrganisationCollection orgs = OrganisationManager.Instance.GetOrganisations(null);
            this.OrganisationList.Items.Clear();
            this.OrganisationList.Items.Add(BasePage.NullItem);
            foreach (Organisation org in orgs)
            {
                this.OrganisationList.Items.Add(new ListItem(org.Name, org.OrganisationId.ToString()));
            }

            this.JobCodeList.Items.Clear();
            this.JobCodeList.Items.Add(BasePage.NullItem);
        }

        /// <summary>
        /// Populates the Job Code List.
        /// </summary>
        /// <param name="organisationId">Id of the organisation.</param>
        private void PopulateJobCodeList(int organisationId)
        {
            BusinessObjects.Filters.JobCodeFilter filter = new BusinessObjects.Filters.JobCodeFilter();
            filter.OrganisationId = organisationId;
            JobCodeCollection jobCodes = JobCodeManager.Instance.GetJobCodes(filter);
            this.JobCodeList.Items.Clear();
            this.JobCodeList.Items.Add(BasePage.NullItem);
            foreach (JobCode jobCode in jobCodes)
            {
                this.JobCodeList.Items.Add(new ListItem(jobCode.Name, jobCode.JobCodeId.ToString()));
            }
        }

        #endregion Private Methods
    }
}