//-----------------------------------------------------------------------
// <copyright file="Clock.ascx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a Clock control.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.WebUserControls
{
    using System;
    using System.Drawing;
    using System.Drawing.Imaging;

    /// <summary>
    /// Class that represents Clock control.
    /// </summary>
    public partial class Clock : System.Web.UI.UserControl
    {
        #region Private Members

        /// <summary>Holds outer Radius of the clock.</summary>
        private int outerRadius = 60;

        /// <summary>Holds inner Radius of the clock.</summary>
        private int innerRadius = 50;

        /// <summary>Holds the fill colour of the clock.</summary>
        private Color fillColour = Color.Blue;

        /// <summary>Holds the border colour of the clock.</summary>
        private Color borderColour = Color.Red;

        /// <summary>Holds the background colour of the clock.</summary>
        private Color backgroundColour = Color.Transparent;

        /// <summary>Height of the hour dial.</summary>
        private int hourDialHeight = 40;

        /// <summary>Width of the hour dial.</summary>
        private int hourDialWidth = 10;

        /// <summary>Colour of the hour dial.</summary>
        private Color hourDialColour = Color.Black;

        /// <summary>Height of the min dial.</summary>
        private int minDialHeight = 40;

        /// <summary>Width of the min dial.</summary>
        private int minDialWidth = 10;

        /// <summary>Colour of the min dial.</summary>
        private Color minDialColour = Color.Black;

        #endregion Private Members

        #region Public Properties

        /// <summary>Gets or sets the clocks inner radius.</summary>
        public int InnerRadius
        {
            get 
            { 
                return this.innerRadius; 
            }

            set 
            {
                if (value > this.OuterRadius)
                {
                    throw new ArgumentOutOfRangeException();
                }

                this.innerRadius = value; 
            }
        }

        /// <summary>Gets or sets the clocks outer radius.</summary>
        public int OuterRadius
        {
            get 
            { 
                return this.outerRadius; 
            }

            set 
            {
                if (value < this.InnerRadius)
                {
                    throw new ArgumentOutOfRangeException();
                }

                this.outerRadius = value; 
            }
        }

        /// <summary>Gets or sets the border colour.</summary>
        public Color BorderColour
        {
            get { return this.borderColour; }
            set { this.borderColour = value; }
        }

        /// <summary>Gets or sets the background Colour.</summary>
        public Color BackgroundColour
        {
            get { return this.backgroundColour; }
            set { this.backgroundColour = value; }
        }

        /// <summary>Gets or sets the fill colour.</summary>
        public Color FillColour
        {
            get { return this.fillColour; }
            set { this.fillColour = value; }
        }

        /// <summary>
        /// Gets the center point of the Clock. Assumes that the clocks top left is 0,0.
        /// </summary>
        public Point CenterPoint
        {
            get
            {
                Point point = new Point();
                point.X = this.OuterRadius;
                point.Y = this.OuterRadius;
                return point;
            }
        }

        /// <summary>
        /// Gets or sets the hour dial height.
        /// </summary>
        public int HourDialHeight
        {
            get
            {
                return this.hourDialHeight;
            }

            set
            {
                if (value < 0)
                {
                    throw new ArgumentOutOfRangeException();
                }

                this.hourDialHeight = value;
            }
        }

        /// <summary>
        /// Gets or sets the hour dial width.
        /// </summary>
        public int HourDialWidth
        {
            get
            {
                return this.hourDialWidth;
            }

            set
            {
                if (value < 0)
                {
                    throw new ArgumentOutOfRangeException();
                }

                this.hourDialWidth = value;
            }
        }

        /// <summary>Gets or sets the hour dial Colour.</summary>
        public Color HourDialColour
        {
            get { return this.hourDialColour; }
            set { this.hourDialColour = value; }
        }

        /// <summary>
        /// Gets or sets the min dial height.
        /// </summary>
        public int MinDialHeight
        {
            get
            {
                return this.minDialHeight;
            }

            set
            {
                if (value < 0)
                {
                    throw new ArgumentOutOfRangeException();
                }

                this.minDialHeight = value;
            }
        }

        /// <summary>
        /// Gets or sets the min dial width.
        /// </summary>
        public int MinDialWidth
        {
            get
            {
                return this.minDialWidth;
            }

            set
            {
                if (value < 0)
                {
                    throw new ArgumentOutOfRangeException();
                }

                this.minDialWidth = value;
            }
        }

        /// <summary>Gets or sets the min dial Colour.</summary>
        public Color MinDialColour
        {
            get { return this.minDialColour; }
            set { this.minDialColour = value; }
        }

        #endregion Public Properties

        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Set the context of this page 
            this.Context.Response.ContentType = "image/gif";

            // Create Bitmap using outer radius
            Bitmap objBitmap = new Bitmap(this.OuterRadius * 2, this.OuterRadius * 2);

            // Create graphics
            Graphics objGraphics = Graphics.FromImage(objBitmap);
            objGraphics.Clear(this.BackgroundColour);
            objGraphics.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;

            // Initialize Graphics Class
            int radiusDifference = this.OuterRadius - this.InnerRadius;

            // Draw outer radius
            objGraphics.FillEllipse(new SolidBrush(this.borderColour), 0, 0, this.OuterRadius * 2, this.OuterRadius * 2);

            // Draw inner radius
            objGraphics.FillEllipse(new SolidBrush(this.FillColour), 0 + radiusDifference, 0 + radiusDifference, this.InnerRadius * 2, this.InnerRadius * 2);

            DateTime now = DateTime.Now;

            // Draw Hour dial
            int hour = now.Hour > 12 ? now.Hour - 12 : now.Hour;
            double hourDegree = (double)360 / (double)12 / (double)60;   
            double currHourDialDegrees = hourDegree * ((hour * 60) + now.Minute);
            double currHourDialRadians = (Math.PI / 180) * currHourDialDegrees;
            Point topHourPoint = new Point(this.CenterPoint.X, this.CenterPoint.Y - this.HourDialHeight);
            Point hourPoint = this.RotatePoint(topHourPoint, this.CenterPoint, currHourDialRadians);
            Point[] hourPoints = new Point[2];
            hourPoints[0] = this.CenterPoint;
            hourPoints[1] = hourPoint;
            Pen hourPen = new Pen(this.HourDialColour, this.HourDialWidth);
            objGraphics.DrawLines(hourPen, hourPoints);

            // Draw Min Dial
            double minDegree = (double)360 / (double)60;
            double currMinDialDegrees = minDegree * now.Minute;
            double currMinDialRadians = (Math.PI / 180) * currMinDialDegrees;
            Point topMinPoint = new Point(this.CenterPoint.X, this.CenterPoint.Y - this.MinDialHeight);
            Point minPoint = this.RotatePoint(topMinPoint, this.CenterPoint, currMinDialRadians);
            Point[] minPoints = new Point[2];
            minPoints[0] = this.CenterPoint;
            minPoints[1] = minPoint;
            Pen minPen = new Pen(this.MinDialColour, this.MinDialWidth);
            objGraphics.DrawLines(minPen, minPoints);

            // Display Bitmap
            objBitmap.Save(Response.OutputStream, ImageFormat.Gif);
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Rotates a point around another point by a number of degrees.
        /// </summary>
        /// <param name="pointToRotate">The point to rotate.</param>
        /// <param name="rotationPoint">The center point to rotate around.</param>
        /// <param name="angle">The angle in radians to rotate.</param>
        /// <returns>The new poistion of the point.</returns>
        private Point RotatePoint(Point pointToRotate, Point rotationPoint, double angle)
        {
            double x = ((pointToRotate.X - rotationPoint.X) * Math.Cos(angle)) - ((rotationPoint.Y - pointToRotate.Y) * Math.Sin(angle));
            double y = ((rotationPoint.Y - pointToRotate.Y) * Math.Cos(angle)) - ((pointToRotate.X - rotationPoint.X) * Math.Sin(angle));
            Point point = new Point((int)x, (int)y);
            return point;
        }

        #endregion Private Methods
    }
}