//-----------------------------------------------------------------------
// <copyright file="Login.ascx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a Login control.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.WebUserControls
{
    using System;
    using System.Web;
    using System.Web.Security;
    using BusinessLogicLayer;
    using BusinessObjects;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class that represents Login control.
    /// </summary>
    public partial class Login : BaseUserControl
    {
        #region Private Members

        /// <summary>Holds Controls stytle.</summary>
        private string cssClass;

        /// <summary>Holds redirectUrl.</summary>
        private string redirectUrl = "~/";

        /// <summary>Holds emailText.</summary>
        private string emailText = "Email:";

        /// <summary>Holds password.</summary>
        private string passwordText = "Password:";

        /// <summary>Holds Log in.</summary>
        private string loginText = "Log In";

        /// <summary>Holds Header text.</summary>
        private string headerText = "Log In";

        /// <summary>Holds Keep Me Logged In text.</summary>
        private string keepMeLoggedInText = "Keep me logged in";

        /// <summary>Holds whether or not to show Keep Me Logged In Section.</summary>
        private bool showKeepMeLoggedInSection = true;

        #endregion Private Members

        #region Public Properties

        /// <summary>Gets or sets controls style.</summary>
        public string CssClass
        {
            get { return this.cssClass; }
            set { this.cssClass = value; }
        }

        /// <summary>Gets or sets RedirectUrl.</summary>
        public string RedirectUrl
        {
            get { return this.redirectUrl; }
            set { this.redirectUrl = value; }
        }

        /// <summary>Gets or sets Email Text.</summary>
        public string EmailText
        {
            get { return this.emailText; }
            set { this.emailText = value; }
        }

        /// <summary>Gets or sets Password Text.</summary>
        public string PasswordText
        {
            get { return this.passwordText; }
            set { this.passwordText = value; }
        }

        /// <summary>Gets or sets Login Text.</summary>
        public string LoginText
        {
            get { return this.loginText; }
            set { this.loginText = value; }
        }

        /// <summary>Gets or sets Header Text.</summary>
        public string HeaderText
        {
            get { return this.headerText; }
            set { this.headerText = value; }
        }

        /// <summary>Gets or sets 'Keep Me Logged In' Text.</summary>
        public string KeepMeLoggedInText
        {
            get { return this.keepMeLoggedInText; }
            set { this.keepMeLoggedInText = value; }
        }

        /// <summary>Gets or sets a value indicating whether or not to show 'Keep Me Logged In' Section.</summary>
        public bool ShowKeepMeLoggedInSection
        {
            get { return this.showKeepMeLoggedInSection; }
            set { this.showKeepMeLoggedInSection = value; }
        }

        #endregion Public Properties

        #region Protected Methods

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Add style to login control
            this.LoginDiv.Attributes.Add("class", this.cssClass);

            // Add names to labels and buoons
            this.EmailLabel.InnerText = this.EmailText;
            this.PasswordLabel.InnerText = this.PasswordText;
            this.LoginInButton.Text = this.LoginText;
            this.HeaderLabel.InnerText = this.HeaderText;
            this.KeepMeLoggedInCheckbox.Text = this.KeepMeLoggedInText;
            this.KeepMeLoggedInCheckbox.Visible = this.ShowKeepMeLoggedInSection;
        }

        /// <summary>
        /// Event Handler for when login page is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void LoginInButton_Clicked(object sender, EventArgs e)
        {
            // Check if information is correct
            try
            {
                User user = UserManager.Instance.GetUser(this.EmailTextbox.Text);
                if (user != null)
                {
                    if (user.DisabledDateTime.HasValue || !user.ActivationDateTime.HasValue)
                    {
                        // Account is disabled or not activated.
                        this.ErrorLabel.InnerText = Resources.Main.FailedToLoginUser;
                    }
                    else
                    {
                        if (UserManager.Instance.AuthenticateUser(this.EmailTextbox.Text, this.PasswordTextbox.Text) == true)
                        {
                            // Login Information is correct.
                            if (Global.CreateTicket(this.Page, this.EmailTextbox.Text, this.KeepMeLoggedInCheckbox.Checked, CommonConstants.Configuration.SessionTimeoutMins) == true)
                            {
                                Response.Redirect(this.redirectUrl);
                            }
                            else
                            {
                                this.ErrorLabel.InnerText = Resources.Main.FailedToLogin;
                            }
                        }
                        else
                        {
                            // Login Information is incorrect.
                            this.ErrorLabel.InnerText = Resources.Main.InvalidUsernameOrPassword;
                        }
                    }
                }
                else
                {
                    // No user found
                    this.ErrorLabel.InnerText = Resources.Main.InvalidUsernameOrPassword;
                }
            }
            catch
            {
                // No user found
                this.ErrorLabel.InnerText = Resources.Main.InvalidUsernameOrPassword;
            }
        }

        /// <summary>
        /// Creates a forms authentication ticket for a validated user so that the user can access
        /// the secure pages. The authentication timeout value is also set so the user will be logged out
        /// afetr a certain amount of time.
        /// </summary>
        /// <param name="userName">The name of the user to login</param>
        /// <param name="stayLoggedIn">Whether or not we want the store the user login details</param>
        /// <param name="timeMin">The time in minutes for when the users authentication session will timeout</param>
        /// <returns>True on success.</returns>
        private bool CreateTicket(string userName, bool stayLoggedIn, int timeMin)
        {
            try
            {
                Session.Clear();
                Session.Abandon();
                FormsAuthentication.Initialize();
                User user = UserManager.Instance.GetUser(userName);
                string role = RoleManager.Instance.GetRole(user.RoleId).Name.ToLower();

                // create the Authentication ticket with timeout info
                FormsAuthenticationTicket ticket = new FormsAuthenticationTicket(
                    1,
                    userName,
                    DateTime.Now,
                    DateTime.Now.AddMinutes(timeMin),
                    stayLoggedIn,
                    role,
                    FormsAuthentication.FormsCookiePath);

                string hash = FormsAuthentication.Encrypt(ticket);
                HttpCookie cookie = new HttpCookie(FormsAuthentication.FormsCookieName, hash);
                if (ticket.IsPersistent)
                {
                    cookie.Expires = ticket.Expiration;
                }

                HttpContext.Current.Response.Cookies.Add(cookie);
                return true;
            }
            catch
            {
                FormsAuthentication.SignOut();
                Session.Abandon();
                return false;
            }
        }

        #endregion Protected Methods
    }
}