//-----------------------------------------------------------------------
// <copyright file="Pager.ascx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a Pager control.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.WebUserControls
{
    using System;
    using System.Web.UI.WebControls;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class that represents Pager control.
    /// </summary>
    public partial class Pager : BaseUserControl
    {
        /// <summary>
        /// Delegate to Handle on page change.
        /// </summary>
        /// <param name="pageNumber">Number of page.</param>
        public delegate void PageChangedHandler(int pageNumber);

        /// <summary>Gets or sets Event Handler for when page is changed.</summary>
        public event PageChangedHandler PageChanged;

        #region Public Properties

        /// <summary>Gets or sets the css class.</summary>
        public string CssClass
        {
            get { return this.ButtonsDiv.Attributes["class"]; }
            set { this.ButtonsDiv.Attributes["class"] = value; }
        }

        /// <summary>Gets or sets the current page number.</summary>
        public int CurrentPageNumber
        {
            get 
            {
                if (ViewState[this.ClientID + "CurrentPageNumber"] == null)
                {
                    ViewState[this.ClientID + "CurrentPageNumber"] = 1;
                }

                return Convert.ToInt32(ViewState[this.ClientID + "CurrentPageNumber"]); 
            }

            set 
            { 
                ViewState[this.ClientID + "CurrentPageNumber"] = value; 
            }
        }

        /// <summary>Gets or sets the Total Number of pages.</summary>
        public int TotalPages
        {
            get { return Convert.ToInt32(ViewState[this.ClientID + "TotalPages"]); }
            set { ViewState[this.ClientID + "TotalPages"] = value; }
        }

        /// <summary>Gets or sets the maximum number of paging buttons to show.</summary>
        public int MaxButtons
        {
            get { return Convert.ToInt32(ViewState[this.ClientID + "MaxButtons"]); }
            set { ViewState[this.ClientID + "MaxButtons"] = value; }
        }

        #endregion Public Properties

        #region Protected Methods

        /// <summary>
        /// Event Handler for on OnInit.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
        }

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            this.CreateButtons();
        }

        /// <summary>
        /// Event Handler for on pre Render.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnPreRender(EventArgs e)
        {
            this.CreateButtons();
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Creates the dynamic buttons of the pager. Uses the current page number and total pages to determine
        /// exactly how many buttons to create and where to create them from.
        /// </summary>
        private void CreateButtons()
        {
            this.ButtonsDiv.Controls.Clear();

            if (this.TotalPages > 1)
            {
                // First Button
                Button firstPageButton = new Button();
                this.ButtonsDiv.Controls.Add(firstPageButton);
                firstPageButton.ID = this.ButtonsDiv.ClientID + "firstPageButton";
                firstPageButton.Text = "<<";
                firstPageButton.CommandArgument = "1";
                firstPageButton.Click += new EventHandler(this.Button_Click);

                // Prev Button
                Button prevPageButton = new Button();
                this.ButtonsDiv.Controls.Add(prevPageButton);
                prevPageButton.ID = this.ButtonsDiv.ClientID + "prevPageButton";
                prevPageButton.Text = "<";
                prevPageButton.CommandArgument = Convert.ToString(this.CurrentPageNumber == 1 ? this.CurrentPageNumber : this.CurrentPageNumber - 1);
                prevPageButton.Click += new EventHandler(this.Button_Click);

                int startPage = 1;
                while (this.CurrentPageNumber >= startPage + this.MaxButtons)
                {
                    startPage += this.MaxButtons;
                }

                for (int i = startPage; i < startPage + this.MaxButtons; i++)
                {
                    if (i <= this.TotalPages)
                    {
                        // Page Button
                        Button pageButton = new Button();
                        this.ButtonsDiv.Controls.Add(pageButton);
                        pageButton.ID = this.ButtonsDiv.ClientID + "PageButton" + i.ToString();
                        pageButton.Text = i.ToString();
                        pageButton.CommandArgument = i.ToString();
                        pageButton.Click += new EventHandler(this.Button_Click);
                    }
                    else
                    {
                        break;
                    }
                }

                // next Button
                Button nextPageButton = new Button();
                this.ButtonsDiv.Controls.Add(nextPageButton);
                nextPageButton.ID = this.ButtonsDiv.ClientID + "nextPageButton";
                nextPageButton.Text = ">";
                nextPageButton.CommandArgument = Convert.ToString(this.CurrentPageNumber == this.TotalPages ? this.CurrentPageNumber : this.CurrentPageNumber + 1);
                nextPageButton.Click += new EventHandler(this.Button_Click);

                // last Button
                Button lastPageButton = new Button();
                this.ButtonsDiv.Controls.Add(lastPageButton);
                lastPageButton.ID = this.ButtonsDiv.ClientID + "lastPageButton";
                lastPageButton.Text = ">>";
                lastPageButton.CommandArgument = this.TotalPages.ToString();
                lastPageButton.Click += new EventHandler(this.Button_Click);
            }
        }

        /// <summary>
        /// Event Handler for on Button Click.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Button_Click(object sender, EventArgs e)
        {
            if (this.PageChanged != null)
            {
                int pageNumber = Convert.ToInt32((sender as Button).CommandArgument);
                this.CurrentPageNumber = pageNumber;
                this.PageChanged(pageNumber);
            }
        }

        #endregion Private Methods
    }
}