//-----------------------------------------------------------------------
// <copyright file="TimesheetFilter.ascx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a TimesheetFilter control.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.WebUserControls
{
    using System;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class that represents TimesheetFilter control.
    /// </summary>
    public partial class TimesheetFilter : BaseUserControl
    {
        #region Public Properties

        /// <summary>Gets or sets controls style.</summary>
        public string CssClass
        {
            get { return this.FilterDiv.Attributes["class"]; }
            set { this.FilterDiv.Attributes["class"] = value; }
        }

        /// <summary>Gets or sets the EndDate.</summary>
        public DateTime? EndDate
        {
            get 
            {
                if (!string.IsNullOrEmpty(this.EndDateTextBox.Text))
                {
                    try
                    {
                        return Convert.ToDateTime(this.EndDateTextBox.Text);
                    }
                    catch
                    {
                        return null;
                    }
                }

                return null;
            }

            set 
            {
                if (value.HasValue)
                {
                    this.EndDateTextBox.Text = value.Value.ToString(CommonConstants.Configuration.DateFormat);
                }
                else
                {
                    this.EndDateTextBox.Text = string.Empty;
                }
            }
        }

        /// <summary>Gets or sets the StartDate.</summary>
        public DateTime? StartDate
        {
            get
            {
                if (!string.IsNullOrEmpty(this.StartDateTextBox.Text))
                {
                    try
                    {
                        return Convert.ToDateTime(this.StartDateTextBox.Text);
                    }
                    catch
                    {
                        return null;
                    }
                }

                return null;
            }

            set
            {
                if (value.HasValue)
                {
                    this.StartDateTextBox.Text = value.Value.ToString(CommonConstants.Configuration.DateFormat);
                }
                else
                {
                    this.StartDateTextBox.Text = string.Empty;
                }
            }
        }

        /// <summary>Gets or sets the TimedheetStatus Id.</summary>
        public short? TimesheetStatusId
        {
            get
            {
                if (this.StatusList.SelectedIndex == 0)
                {
                    return null;
                }

                return Convert.ToInt16(this.StatusList.SelectedValue);
            }

            set
            {
                this.StatusList.SelectedValue = value.ToString();
            }
        }

        /// <summary>Gets or sets the user Id.</summary>
        public long? UserId
        {
            get
            {
                if (!this.ShowUserSelect)
                {
                    return (this.Page as SecurePage).UserId;
                }

                return null;
            }

            set
            {
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the user can be selected.
        /// </summary>
        public bool ShowUserSelect
        {
            get
            {
                return this.UserDiv.Visible;
            }

            set
            {
                this.UserDiv.Visible = value;
            }
        }

        /// <summary>Gets or sets the validation group.</summary>
        public string ValidationGroup
        {
            get
            {
                return this.StartDateTextBoxRegularExpressionValidator.ValidationGroup;
            }

            set
            {
                this.StartDateTextBoxRegularExpressionValidator.ValidationGroup = value;
                this.EndDateTextBoxRegularExpressionValidator.ValidationGroup = value;
            }
        }

        #endregion Public Properties

        #region Public Methods

        /// <summary>
        /// Sets the data of the control to match the filter.
        /// </summary>
        /// <param name="filter">Filter to set to controls on page.</param>
        public void SetData(BusinessObjects.Filters.TimesheetFilter filter)
        {
            if (filter.TimesheetStatusId.HasValue)
            {
                this.StatusList.SelectedValue = filter.TimesheetStatusId.Value.ToString();
            }
            else
            {
                this.StatusList.SelectedIndex = -1;
            }

            if (filter.StartDateTime.HasValue)
            {
                this.StartDateTextBox.Text = filter.StartDateTime.Value.ToString(CommonConstants.Configuration.DateFormat);
            }
            else
            {
                this.StartDateTextBox.Text = string.Empty;
            }

            if (filter.EndDateTime.HasValue)
            {
                this.EndDateTextBox.Text = filter.EndDateTime.Value.ToString(CommonConstants.Configuration.DateFormat);
            }
            else
            {
                this.EndDateTextBox.Text = string.Empty;
            }
        }

        #endregion Public Methods

        #region Protected Methods

        /// <summary>
        /// Event Handler for on OnInit.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (!this.IsPostBack)
            {
                this.PopulateControl();
            }
        }

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the control.
        /// </summary>
        private void PopulateControl()
        {
            this.StartDateTextBoxCalendarExtender.Format = CommonConstants.Configuration.DateFormat;
            this.StartDateTextBoxWatermarkExtender.WatermarkText = CommonConstants.Configuration.DateFormat;
            this.StartDateTextBoxRegularExpressionValidator.ValidationExpression = CommonConstants.Configuration.DateRegex;
            this.EndDateTextBoxCalendarExtender.Format = CommonConstants.Configuration.DateFormat;
            this.EndDateTextBoxWatermarkExtender.WatermarkText = CommonConstants.Configuration.DateFormat;
            this.EndDateTextBoxRegularExpressionValidator.ValidationExpression = CommonConstants.Configuration.DateRegex;
            this.PopulateTimesheetStatusList();
        }

        /// <summary>
        /// Populates the Timesheet Status List.
        /// </summary>
        private void PopulateTimesheetStatusList()
        {
            TimesheetStatusCollection statuses = TimesheetStatusManager.Instance.GetTimesheetStatuses(null);
            this.StatusList.Items.Clear();
            this.StatusList.Items.Add(BasePage.NullItem);
            foreach (TimesheetStatus status in statuses)
            {
                this.StatusList.Items.Add(new ListItem(status.Name, status.TimesheetStatusId.ToString()));
            }
        }

        #endregion Private Methods
    }
}