//-----------------------------------------------------------------------
// <copyright file="UserFilter.ascx.cs" company="Yasar Bhatti">Copyright(c) Yasar Bhatti. All rights reserved.</copyright>
// <author>Yasar Bhatti</author>
// <summary>Holds class that represents a UserFilter control.</summary>
//-----------------------------------------------------------------------
namespace TimesheetWebApplication.WebUserControls
{
    using System;
    using System.Web.UI.WebControls;
    using BusinessLogicLayer;
    using BusinessObjects;
    using BusinessObjects.Collections;
    using TimesheetWebApplication.App_Code;

    /// <summary>
    /// Class that represents UserFilter control. Only admins or managers are allowed to view this control.
    /// </summary>
    public partial class UserFilter : BaseUserControl
    {
        #region Public Properties

        /// <summary>Gets or sets controls style.</summary>
        public string CssClass
        {
            get { return this.FilterDiv.Attributes["class"]; }
            set { this.FilterDiv.Attributes["class"] = value; }
        }

        /// <summary>Gets the User.</summary>
        public string User
        {
            get { return this.UserTextBox.Text; }
        }

        /// <summary>Gets the Approver.</summary>
        public string Approver
        {
            get { return this.ApproverTextBox.Text; }
        }

        /// <summary>Gets the Gender Id.</summary>
        public byte? GenderId
        {
            get
            {
                if (this.GenderList.SelectedIndex == 0)
                {
                    return null;
                }

                return Convert.ToByte(this.GenderList.SelectedValue);
            }
        }

        /// <summary>Gets the Role Id.</summary>
        public short? RoleId
        {
            get
            {
                // Get users org if the organisation section is hidden
                if (this.RoleDiv.Visible == false)
                {
                    SecurePage page = this.Page as SecurePage;
                    return Convert.ToInt16(BusinessLogicLayer.Utilities.CommonConstants.Roles.Standard);
                }

                if (this.RoleList.SelectedIndex == 0)
                {
                    return null;
                }

                return Convert.ToInt16(this.RoleList.SelectedValue);
            }
        }

        /// <summary>Gets the organisation Id.</summary>
        public int? OrganisationId
        {
            get
            {
                // Get users org if the organisation section is hidden
                if (this.OrganisationDiv.Visible == false)
                {
                    SecurePage page = this.Page as SecurePage;
                    return page.UserOrganisationId;
                }

                if (this.OrganisationList.SelectedIndex == 0)
                {
                    return null;
                }

                return Convert.ToInt32(this.OrganisationList.SelectedValue);
            }
        }

        /// <summary>Gets or sets the validation group.</summary>
        public string ValidationGroup
        {
            get
            {
                return null;
            }

            set
            {
            }
        }

        #endregion Public Properties

        #region Protected Methods

        /// <summary>
        /// Event Handler for on OnInit.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);

            if (!this.IsPostBack)
            {
                SecurePage page = this.Page as SecurePage;
                if (page.IsUserStandard)
                {
                    this.Visible = false;
                }
                else
                {
                    this.PopulateControl();
                }
            }
        }

        /// <summary>
        /// Event Handler for on page load.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
        }

        #endregion Protected Methods

        #region Private Methods

        /// <summary>
        /// Populates the control.
        /// </summary>
        private void PopulateControl()
        {
            SecurePage page = this.Page as SecurePage;
            this.PopulateGenderList();

            if (page.IsUserAdministrator)
            {
                // User is admin
                this.PopulateRoleList();
                this.PopulateOrganisationList();
            }
            else
            {
                // User is not admin

                // Hide role
                this.RoleDiv.Visible = false;

                // Only show org list if user not in org
                if (!page.IsUserInOrganisation)
                {
                    this.PopulateOrganisationList();
                }
                else
                {
                    this.OrganisationDiv.Visible = false;
                }
            }  
        }

        /// <summary>
        /// Populates the gender List.
        /// </summary>
        private void PopulateGenderList()
        {
            GenderCollection genders = GenderManager.Instance.GetGenders(null);
            this.GenderList.Items.Clear();
            this.GenderList.Items.Add(BasePage.NullItem);
            foreach (Gender gender in genders)
            {
                this.GenderList.Items.Add(new ListItem(gender.Name, gender.GenderId.ToString()));
            }
        }

        /// <summary>
        /// Populates the role List.
        /// </summary>
        private void PopulateRoleList()
        {
            RoleCollection roles = RoleManager.Instance.GetRoles(null);
            this.RoleList.Items.Clear();
            this.RoleList.Items.Add(BasePage.NullItem);
            foreach (Role role in roles)
            {
                this.RoleList.Items.Add(new ListItem(role.Name, role.RoleId.ToString()));
            }
        }

        /// <summary>
        /// Populates the organisation List.
        /// </summary>
        private void PopulateOrganisationList()
        {
            OrganisationCollection organisations = OrganisationManager.Instance.GetOrganisations(null);
            this.OrganisationList.Items.Clear();
            this.OrganisationList.Items.Add(BasePage.NullItem);
            foreach (Organisation organisation in organisations)
            {
                this.OrganisationList.Items.Add(new ListItem(organisation.Name, organisation.OrganisationId.ToString()));
            }
        }

        #endregion Private Methods
    }
}